#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_anova_single_factor.h"
#include "kspread_inputrange1groupbox.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>

#define INPUT_NAME "Input Range"

KSpreadDlgAnovaSingleFactor::KSpreadDlgAnovaSingleFactor(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgAnovaSingleFactor::KSpreadDlgAnovaSingleFactor() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);

	KSpreadInputRange1GroupBox* inputGroup = new KSpreadInputRange1GroupBox(this, mainBox, "groupedByGroup");
	QVBoxLayout* layout = inputGroup->layout();
	QGrid* alphaGrid = new QGrid(2, inputGroup);
	layout->addWidget(alphaGrid);
	new QLabel("Alpha: ", alphaGrid);
	new QLineEdit("0.05", alphaGrid, "alphaEdit");

	new KSpreadOutputGroupBox(this, mainBox);
}

void KSpreadDlgAnovaSingleFactor::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();
	QString inputRange = map[INPUT_NAME];
	QRect rect = getRange(inputRange);
	SheetOrientation orientation = (map["groupedByGroup"] == "groupedByColumnsButton") ? OrientationColumn : OrientationRow;
	QStringVector* labelsVectorPtr;
	labelsVectorPtr = getLabels(getSheet(inputRange), rect, orientation, (map["labelsCheckBox"] == "1"));
	QStringVector& labelsVector = *labelsVectorPtr;
	FloatVectorVector* floatVectorVectorPtr;
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, orientation)))
		return;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	hlpfunc_remove_empty_vectors(floatVectorVector, labelsVector);
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector, 2))
		return;
	double alphaNum;
	QString value = map["alphaEdit"];
	if (!validateNumber("Alpha", value, 0, 1))
		return;
	alphaNum = value.toDouble();
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, OrientationAll)))
		return;
	FloatVector& floatVectorAll = (*floatVectorVectorPtr)[0];	
	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, 6, 11 + floatVectorVector.size(), col, row)))
		return;
	EvaluationError error;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	setCell(outputSheet, col, row, this->name());
	row++;
	row++;
	setCell(outputSheet, col, row, "SUMMARY");
	row++;
	setCell(outputSheet, col, row, "Groups");
	setCell(outputSheet, col + 1, row, "Count");
	setCell(outputSheet, col + 2, row, "Sum");
	setCell(outputSheet, col + 3, row, "Average");
	setCell(outputSheet, col + 4, row, "Variance");
	setTitlesRow(outputSheet, col, col + 4, row);
	for (uint i = 0; i<floatVectorVector.size(); i++)
	{
		row++;
		FloatVector& vector = floatVectorVector[i];
		QString& label = labelsVector[i];
		double result;
		setCell(outputSheet, col, row, label);
		error = NoError;
		dafunc_count(vector, error, result);
		setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_sum(vector, error, result);
		setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_average(vector, error, result);
		setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_variance(vector, error, result);
		setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
	}
	setEndRow(outputSheet, col, col + 4, row);
	row++;
	row++;
	row++;
	setCell(outputSheet, col, row, "ANOVA");
	row++;
	setCell(outputSheet, col, row, "Source of Variation");
	setCell(outputSheet, col + 1, row, "SS");
	setCell(outputSheet, col + 2, row, "df");
	setCell(outputSheet, col + 3, row, "MS");
	setCell(outputSheet, col + 4, row, "F");
	setCell(outputSheet, col + 5, row, "P-value");
	setCell(outputSheet, col + 6, row, "F crit");
	setTitlesRow(outputSheet, col, col + 6, row);
	row++;
	double result;
	setCell(outputSheet, col, row, "Between Groups");
	error = NoError;
	dafunc_anova_ss_effect(floatVectorVector, floatVectorAll, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_df_effect(floatVectorVector, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_ms_effect(floatVectorVector,  floatVectorAll, error, result);
	setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_single_f_stat(floatVectorVector,  floatVectorAll, error, result);
	setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_single_f_dist(floatVectorVector,  floatVectorAll, error, result);
	setCell(outputSheet, col + 5, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_single_f_crit(floatVectorVector, floatVectorAll, alphaNum, error, result);
	setCell(outputSheet, col + 6, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "Within Groups");
	error = NoError;
	dafunc_anova_single_ss_error(floatVectorVector, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_single_df_error(floatVectorVector, floatVectorAll, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_single_ms_error(floatVectorVector,  floatVectorAll, error, result);
	setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
	row++;
	row++;
	setCell(outputSheet, col, row, "Total");
	error = NoError;
	dafunc_anova_ss_total(floatVectorAll, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_df_total(floatVectorAll, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	setEndRow(outputSheet, col, col + 6, row);
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}


KSpreadDlgAnovaSingleFactor::~KSpreadDlgAnovaSingleFactor()
{
	kdDebug() <<"KSpreadDlgAnovaSingleFactor::~KSpreadDlgAnovaSingleFactor() " << endl;
}

#include "kspread_dlg_da_anova_single_factor.moc"
