#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_anova_two_factor.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>

#define INPUT_NAME "Input Range"

KSpreadDlgAnovaTwoFactor::KSpreadDlgAnovaTwoFactor(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgAnovaTwoFactor::KSpreadDlgAnovaTwoFactor() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	QGroupBox* inputGroup = new QGroupBox("Input", mainBox, "inputGroup");
	QVBoxLayout* inputLayout = new QVBoxLayout(inputGroup, marginHint(), spacingHint());
	QGrid* grid1 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid1);
	new QLabel("Input Range: ", grid1);
	new QEditButton(grid1, "Input Range");
	QGrid* grid2 = new QGrid(1, inputGroup);
	inputLayout->addWidget(grid2);
	new QCheckBox("Labels", grid2, "labelsCheckBox");
	QGrid* grid3 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid3);
	new QLabel("Alpha: ", grid3);
	new QLineEdit("0.05", grid3, "alphaEdit");

	new KSpreadOutputGroupBox(this, mainBox);
}

void KSpreadDlgAnovaTwoFactor::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();

	QString inputRange = map[INPUT_NAME];
	QRect rectR = getRange(inputRange);
	bool labels = (map["labelsCheckBox"] == "1");

	QStringVector labelsVector[2];
	FloatVectorVector floatVectorVector[2];

	QRect rect[2];

	rect[0] = rect[1] = rectR;
	if (labels)
	{
		rect[0].setTop(rectR.top() + 1);
		rect[1].setLeft(rectR.left() + 1);
	}
	FloatVectorVector* floatVectorVectorPtr;
	SheetOrientation orientation = OrientationRow;
	for (int i = 0; i < 2; i++)
	{
		QStringVector* labelsVectorPtr;
		labelsVectorPtr = getLabels(getSheet(inputRange), rect[i], orientation, labels);
		labelsVector[i] = *labelsVectorPtr;
		if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect[i], orientation)))
			return;
		floatVectorVector[i] = *floatVectorVectorPtr;
		labelsVectorPtr = getLabels(getSheet(inputRange), rect[i], orientation);
		hlpfunc_remove_empty_vectors(floatVectorVector[i], labelsVector[i]);
		if (!checkMinVectorSize(INPUT_NAME, floatVectorVector[i], 2))
			return;
		orientation = OrientationColumn;
	}
	double alphaNum;
	QString value = map["alphaEdit"];
	if (!validateNumber("Alpha", value, 0, 1))
		return;
	alphaNum = value.toDouble();
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect[0], OrientationAll)))
		return;
	FloatVector& floatVectorAll = (*floatVectorVectorPtr)[0];
	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, 6, 12 + floatVectorVector[0].size() + floatVectorVector[1].size(), col, row)))
		return;
	EvaluationError error;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	setCell(outputSheet, col, row, this->name());
	row++;
	row++;
	setCell(outputSheet, col, row, "SUMMARY");
	setCell(outputSheet, col + 1, row, "Count");
	setCell(outputSheet, col + 2, row, "Sum");
	setCell(outputSheet, col + 3, row, "Average");
	setCell(outputSheet, col + 4, row, "Variance");
	setTitlesRow(outputSheet, col, col + 4, row);
	for (int i = 0; i < 2; i++, row++)
		for (uint j = 0; j < floatVectorVector[i].size(); j++)
		{
			row++;
			FloatVector& vector = floatVectorVector[i][j];
			QString& label = labelsVector[i][j];
			double result;
			setCell(outputSheet, col, row, label);
			error = NoError;
			dafunc_count(vector, error, result);
			setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
			error = NoError;
			dafunc_sum(vector, error, result);
			setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
			error = NoError;
			dafunc_average(vector, error, result);
			setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
			error = NoError;
			dafunc_variance(vector, error, result);
			setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
		}
	setEndRow(outputSheet, col, col + 4, row - 1);
	row++;
	row++;
	setCell(outputSheet, col, row, "ANOVA");
	row++;
	setCell(outputSheet, col, row, "Source of Variation");
	setCell(outputSheet, col + 1, row, "SS");
	setCell(outputSheet, col + 2, row, "df");
	setCell(outputSheet, col + 3, row, "MS");
	setCell(outputSheet, col + 4, row, "F");
	setCell(outputSheet, col + 5, row, "P-value");
	setCell(outputSheet, col + 6, row, "F crit");
	setTitlesRow(outputSheet, col, col + 6, row);
	row++;
	double result;
	setCell(outputSheet, col, row, "Rows");
	setCell(outputSheet, col, row + 1, "Columns");
	for (int i = 0; i < 2; i++, row++)
	{
		error = NoError;
		dafunc_anova_ss_effect(floatVectorVector[i], floatVectorAll, error, result);
		setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_df_effect(floatVectorVector[i], error, result);
		setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_ms_effect(floatVectorVector[i],  floatVectorAll, error, result);
		setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_two_f_stat(floatVectorVector[i],  floatVectorVector[!i], floatVectorAll, error, result);
		setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_two_f_dist(floatVectorVector[i],  floatVectorVector[!i], floatVectorAll, error, result);
		setCell(outputSheet, col + 5, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_two_f_crit(floatVectorVector[i],  floatVectorVector[!i], alphaNum, error, result);
		setCell(outputSheet, col + 6, row, hlpfunc_result_string(error, result));
	}
	setCell(outputSheet, col, row, "Error");
	error = NoError;
	dafunc_anova_two_ss_error(floatVectorVector[0], floatVectorVector[1], floatVectorAll, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_two_df_error(floatVectorVector[0], floatVectorVector[1], error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_two_ms_error(floatVectorVector[0], floatVectorVector[1], floatVectorAll, error, result);
	setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
	row++;
	row++;
	setCell(outputSheet, col, row, "Total");
	error = NoError;
	dafunc_anova_ss_total(floatVectorAll, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_df_total(floatVectorAll, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	setEndRow(outputSheet, col, col + 6, row);
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}


KSpreadDlgAnovaTwoFactor::~KSpreadDlgAnovaTwoFactor()
{
	kdDebug() <<"KSpreadDlgAnovaTwoFactor::~KSpreadDlgAnovaTwoFactor() " << endl;
}

#include "kspread_dlg_da_anova_two_factor.moc"
