#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_anova_two_factor_repl.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>
#include <kmessagebox.h>

#define INPUT_NAME "Input Range"
#define ROWS_SAMPLE_NAME "Rows per sample"

#define max(v1,v2) (v1 > v2 ? v1 : v2)

KSpreadDlgAnovaTwoFactorReplication::KSpreadDlgAnovaTwoFactorReplication(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgAnovaTwoFactorReplication::KSpreadDlgAnovaTwoFactorReplication() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	QGroupBox* inputGroup = new QGroupBox("Input", mainBox, "inputGroup");
	QVBoxLayout* inputLayout = new QVBoxLayout(inputGroup, marginHint(), spacingHint());

	QGrid* grid1 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid1);
	new QLabel("Input Range: ", grid1);
	QEditButton* inputRangeEB = new QEditButton(grid1, "Input Range");
	inputRangeEB->setMargin(3);

	QGrid* grid2 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid2);
	QString labelName = ROWS_SAMPLE_NAME;
	new QLabel(labelName + ": ", grid2);
	QGrid* grid2a = new QGrid(1, grid2);
	new QLineEdit(grid2a, "rowsPerSampleEdit");
	grid2a->setMargin(3);

	QGrid* grid3 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid3);
	new QLabel("Alpha: ", grid3);
	QGrid* grid3a = new QGrid(1, grid3);
	new QLineEdit("0.05", grid3a, "alphaEdit");
	grid3a->setMargin(3);

	new KSpreadOutputGroupBox(this, mainBox);
}

void KSpreadDlgAnovaTwoFactorReplication::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();

	QString inputRange = map[INPUT_NAME];
	QRect rectR = getRange(inputRange);

	QStringVector labelsVector[2];
	FloatVectorVector floatVectorVector[2];

	QRect rect[2];

	rect[0] = rect[1] = rectR;
	rect[0].setTop(rectR.top() + 1);
	rect[1].setLeft(rectR.left() + 1);

	FloatVectorVector* floatVectorVectorPtr;
	SheetOrientation orientation = OrientationRow;
	for (int i = 0; i < 2; i++)
	{
		QStringVector* labelsVectorPtr;
		labelsVectorPtr = getLabels(getSheet(inputRange), rect[i], orientation, true);
		labelsVector[i] = *labelsVectorPtr;
		orientation = OrientationColumn;
	}
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect[1], orientation, NoEmpty)))
		return;
	floatVectorVector[1] = *floatVectorVectorPtr;
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector[1], 2))
		return;
	uint  rowsPerSampleNum;
	QString value = map["rowsPerSampleEdit"];
	if (!validatePositiveNumber(ROWS_SAMPLE_NAME, value))
		return;
	rowsPerSampleNum = value.toUInt();
	if (!validateDataPoints(ROWS_SAMPLE_NAME, value, floatVectorVector[1][0].size()))
		return;
	if (floatVectorVector[1][0].size() % rowsPerSampleNum != 0)
	{
		QString errorStr = "Each sample must contain the same number of rows.";
		KMessageBox::information(this, errorStr);
		return;
	}
	uint rowLoop = floatVectorVector[1][0].size() / rowsPerSampleNum;
	double alphaNum;
	value = map["alphaEdit"];
	if (!validateNumber("Alpha", value, 0, 1))
		return;
	alphaNum = value.toDouble();
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect[0], OrientationAll)))
		return;
	FloatVector& floatVectorAll = (*floatVectorVectorPtr)[0];
	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, max(floatVectorVector[1]. size() + 1, 6), 11 + 6 * (rowLoop + 1) , col, row)))
		return;
	EvaluationError error;
	FloatVectorVector cellVectorVector;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	setCell(outputSheet, col, row, this->name());
	row++;
	row++;
	setCell(outputSheet, col, row, "SUMMARY");
	for (uint i = 0; i < labelsVector[1].size(); i++)
		setCell(outputSheet, col + i + 1, row, labelsVector[1][i]);
	setCell(outputSheet, col + labelsVector[1].size() + 1, row, "Total");
	row++;
	for (uint i = 0; i < rowLoop; i++, row += 6)
	{
		uint startRow = i * rowsPerSampleNum;
		uint endRow = startRow + rowsPerSampleNum;
		setCell(outputSheet, col, row, labelsVector[0][startRow]);
		setLabelRow(outputSheet, col, col + floatVectorVector[1].size() + 1, row);
		printStatisticsNames(outputSheet, col, row + 1);
		FloatVector totalVector;
		int totalCount = 0;
		for (uint j = 0; j < floatVectorVector[1].size(); j++)
		{
			FloatVector rowVector;
			int count = 0;
			for (uint k = startRow; k < endRow; k++, count++, totalCount++)
			{
				rowVector[count] = floatVectorVector[1][j][k];
				totalVector[totalCount] = rowVector[count];
			}
			cellVectorVector.push_back(rowVector);
			evaluateStatistics(outputSheet, rowVector, col + j + 1, row + 1);
		}
		evaluateStatistics(outputSheet, totalVector, col + floatVectorVector[1].size() + 1, row + 1);
		floatVectorVector[0].push_back(totalVector);
	}
	setCell(outputSheet, col, row, "Total");
	setLabelRow(outputSheet, col, col + floatVectorVector[1].size(), row);
	printStatisticsNames(outputSheet, col, row + 1);
	for (uint j = 0; j < floatVectorVector[1].size(); j++)
		evaluateStatistics(outputSheet, floatVectorVector[1][j], col + j + 1, row + 1);
	row += 7;
	setCell(outputSheet, col, row, "ANOVA");
	row++;
	setCell(outputSheet, col, row, "Source of Variation");
	setCell(outputSheet, col + 1, row, "SS");
	setCell(outputSheet, col + 2, row, "df");
	setCell(outputSheet, col + 3, row, "MS");
	setCell(outputSheet, col + 4, row, "F");
	setCell(outputSheet, col + 5, row, "P-value");
	setCell(outputSheet, col + 6, row, "F crit");
	setTitlesRow(outputSheet, col, col + 6, row);
	double result;
	setCell(outputSheet, col, row + 1, "Sample");
	setCell(outputSheet, col, row + 2, "Columns");
	for (uint i = 0; i < 2; i++)
	{
		row++;
		error = NoError;
		dafunc_anova_ss_effect(floatVectorVector[i], floatVectorAll, error, result);
		setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_df_effect(floatVectorVector[i], error, result);
		setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_ms_effect(floatVectorVector[i],  floatVectorAll, error, result);
		setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_two_repl_f_stat(floatVectorVector[i],  floatVectorVector[!i], cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
		setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_two_repl_f_dist(floatVectorVector[i],  floatVectorVector[!i], cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
		setCell(outputSheet, col + 5, row, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_anova_two_repl_f_crit(floatVectorVector[i],  floatVectorVector[!i], rowsPerSampleNum, alphaNum, error, result);
		setCell(outputSheet, col + 6, row, hlpfunc_result_string(error, result));
	}
	row++;
	setCell(outputSheet, col, row, "Interaction");
	error = NoError;
	dafunc_anova_two_repl_ss_interaction(floatVectorVector[0], floatVectorVector[1], cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_two_repl_df_interaction(floatVectorVector[0], floatVectorVector[1], error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_two_repl_ms_interaction(floatVectorVector[0], floatVectorVector[1], cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
	setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));	
	error = NoError;
	dafunc_anova_two_repl_interaction_f_stat(floatVectorVector[0],  floatVectorVector[1], cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
	setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_two_repl_interaction_f_dist(floatVectorVector[0],  floatVectorVector[1], cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
	setCell(outputSheet, col + 5, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_two_repl_interaction_f_crit(floatVectorVector[0],  floatVectorVector[1], rowsPerSampleNum, alphaNum, error, result);
	setCell(outputSheet, col + 6, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "Within");
	error = NoError;
	dafunc_anova_two_repl_ss_error(cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_two_repl_df_error(floatVectorVector[0], floatVectorVector[1], rowsPerSampleNum, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	dafunc_anova_two_repl_ms_error(floatVectorVector[0], floatVectorVector[1], cellVectorVector, floatVectorAll, rowsPerSampleNum, error, result);
	setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
	row++;
	row++;
	setCell(outputSheet, col, row, "Total");
	error = NoError;
	dafunc_anova_ss_total(floatVectorAll, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_anova_df_total(floatVectorAll, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	setEndRow(outputSheet, col, col + 6, row);
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}

void KSpreadDlgAnovaTwoFactorReplication::printStatisticsNames(KSpread::Sheet* sheet, int col, int row)
{
	setCell(sheet, col, row, "Count");
	setCell(sheet, col, row + 1, "Sum");
	setCell(sheet, col, row + 2, "Average");
	setCell(sheet, col, row + 3, "Variance");	
}

void KSpreadDlgAnovaTwoFactorReplication::evaluateStatistics(KSpread::Sheet* sheet, FloatVector vector, int col, int row)
{
	EvaluationError error;
	double result;
	error = NoError;
	dafunc_count(vector, error, result);
	setCell(sheet, col, row, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_sum(vector, error, result);
	setCell(sheet, col, row + 1, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_average(vector, error, result);
	setCell(sheet, col, row + 2, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_variance(vector, error, result);
	setCell(sheet, col, row + 3, hlpfunc_result_string(error, result));
}

void KSpreadDlgAnovaTwoFactorReplication::setLabelRow(KSpread::Sheet* sheet, int colStart, int colEnd,  int row)
{
	for (int i = colStart; i <= colEnd; i++)
		setCellStyle(sheet, i, row, NoLine, FatLine, FontItalic, StandardAlign);
}

KSpreadDlgAnovaTwoFactorReplication::~KSpreadDlgAnovaTwoFactorReplication()
{
	kdDebug() <<"KSpreadDlgAnovaTwoFactorReplication::~KSpreadDlgAnovaTwoFactorReplication() " << endl;
}

#include "kspread_dlg_da_anova_two_factor_repl.moc"
