#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_correlation.h"
#include "kspread_inputrange1groupbox.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qvbox.h>

#define INPUT_NAME "Input Range"

KSpreadDlgCorrelation::KSpreadDlgCorrelation(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgCorrelation::KSpreadDlgCorrelation() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	
	new KSpreadInputRange1GroupBox(this, mainBox, "groupedByGroup");

	new KSpreadOutputGroupBox(this, mainBox);
}

void KSpreadDlgCorrelation::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();
	QString inputRange = map[INPUT_NAME];
	QRect rect = getRange(inputRange);
	SheetOrientation orientation = (map["groupedByGroup"] == "groupedByColumnsButton") ? OrientationColumn : OrientationRow;
	QStringVector* labelsVectorPtr;
	labelsVectorPtr = getLabels(getSheet(inputRange), rect, orientation, (map["labelsCheckBox"] == "1"));
	QStringVector& labelsVector = *labelsVectorPtr;
	FloatVectorVector* floatVectorVectorPtr;
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, orientation, EmptyShift)))
		return;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	hlpfunc_remove_empty_vectors(floatVectorVector, labelsVector);
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector, 1))
		return;
	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, floatVectorVector.size(), floatVectorVector.size(), col, row)))
		return;
	EvaluationError error;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	for (uint i = 0; i<labelsVector.size(); i++)
	{
		QString& label = labelsVector[i];
		setCell(outputSheet, col + i + 1, row, label);
	}
	setTitlesRow(outputSheet, col, col + floatVectorVector.size(), row);
	for (uint i = 0; i<floatVectorVector.size(); i++)
	{
		row++;
		QString& label = labelsVector[i];
		setCell(outputSheet, col, row, label);
		for (uint j = 0; j< i + 1; j++)
		{
			FloatVector vector1 = floatVectorVector[i];
			FloatVector vector2 = floatVectorVector[j];
			hlpfunc_remove_not_paired_elements(vector1, vector2);
			double result;
			error = NoError;
			dafunc_correlation(vector1, vector2, error, result);
			setCell(outputSheet, col + j + 1, row, hlpfunc_result_string(error, result));
		}
	}
	setEndRow(outputSheet, col, col + floatVectorVector.size(), row);
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}


KSpreadDlgCorrelation::~KSpreadDlgCorrelation()
{
	kdDebug() <<"KSpreadDlgCorrelation::~KSpreadDlgCorrelation() " << endl;
}

#include "kspread_dlg_da_correlation.moc"
