#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_descriptive_statistics.h"
#include "kspread_inputrange1groupbox.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qgrid.h>
#include  <qlabel.h>
#include <qcheckbox.h>
#include <kmessagebox.h>

#define INPUT_NAME "Input Range"

KSpreadDlgDescriptiveStatistics::KSpreadDlgDescriptiveStatistics(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgDescriptiveStatistics::KSpreadDlgDescriptiveStatistics() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);

	new KSpreadInputRange1GroupBox(this, mainBox, "groupedByGroup");

	KSpreadOutputGroupBox* outputGroup = new KSpreadOutputGroupBox(this, mainBox);
	QVBoxLayout* layout = outputGroup->layout();
	QGrid* outputOptions = new QGrid(2, outputGroup);
	layout->addWidget(outputOptions);
	new QCheckBox("Summary statistics", outputOptions, "summaryStatisticsCheckBox");
	new QWidget(outputOptions);
	QCheckBox* confidenceLevelCheckBox = new QCheckBox("Confidance Level for Mean: ", outputOptions, "confidenceLevelCheckBox");
	QGrid* confidenceLevelEditGrid = new QGrid(2, outputOptions);
	QLineEdit* confidenceLevelEdit = new QLineEdit("95", confidenceLevelEditGrid, "confidenceLevelEdit");
	new QLabel("%", confidenceLevelEditGrid);
	QCheckBox* kthLargestCheckBox = new QCheckBox("Kth Largest: ", outputOptions,  "kthLargestCheckBox");
	QLineEdit* kthLargestEdit = new QLineEdit("1", outputOptions,  "kthLargestEdit");
	QCheckBox* kthSmallestCheckBox = new QCheckBox("Kth Smallest: ", outputOptions,  "kthSmallestCheckBox");
	QLineEdit* kthSmallestEdit = new QLineEdit("1", outputOptions,  "kthSmallestEdit");
	outputGroup->setMinimumHeight(200);

	confidenceLevelEdit->setEnabled(false);
	kthLargestEdit->setEnabled(false);
	kthSmallestEdit->setEnabled(false);

	connectFocus(confidenceLevelCheckBox, confidenceLevelEdit);
	connectFocus(kthLargestCheckBox, kthLargestEdit);
	connectFocus(kthSmallestCheckBox, kthSmallestEdit);
}

void KSpreadDlgDescriptiveStatistics::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();
	QString inputRange = map[INPUT_NAME];
	QRect rect = getRange(inputRange);
	SheetOrientation orientation = (map["groupedByGroup"] == "groupedByColumnsButton") ? OrientationColumn : OrientationRow;
	QStringVector* labelsVectorPtr;
	labelsVectorPtr = getLabels(getSheet(inputRange), rect, orientation, (map["labelsCheckBox"] == "1"));
	QStringVector& labelsVector = *labelsVectorPtr;
	FloatVectorVector* floatVectorVectorPtr;
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, orientation)))
		return;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	hlpfunc_remove_empty_vectors(floatVectorVector, labelsVector);
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector, 1))
		return;
	bool summaryStatistics = (map["summaryStatisticsCheckBox"] == "1");
	bool confidenceLevel = (map["confidenceLevelCheckBox"] == "1");
	bool kthLargest = (map["kthLargestCheckBox"] == "1");
	bool kthSmallest = (map["kthSmallestCheckBox"] == "1");
	int numOfRows = 0;
	if (summaryStatistics)
	{
		numOfRows += 12;
	}
	double confidenceLevelNum = -1;
	if (confidenceLevel)
	{
		numOfRows += 1;
		QString value = map["confidenceLevelEdit"];
		if (!validateNumber("Confidence", value, 0, 100))
			return;
		confidenceLevelNum = value.toDouble();
	}
	int maxVectorSize = hlpfunc_max_vector_size(floatVectorVector);
	int kthLargestNum = -1;
	if (kthLargest)
	{
		numOfRows += 1;
		QString value = map["kthLargestEdit"];
		if (!validateDataPoints("Kth Largest", value, maxVectorSize))
			return;
		kthLargestNum = value.toInt();
	}
	int kthSmallestNum = -1;
	if (kthSmallest)
	{
		numOfRows += 1;
		QString value = map["kthSmallestEdit"];
		if (!validateDataPoints("Kth Smallest", value, maxVectorSize))
			return;
		kthSmallestNum = value.toInt();
	}
	if (!numOfRows)
	{
		KMessageBox::information( this, "Please choose at least one statistical option.");
		return;
	}
	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, 2 * floatVectorVector.size() - 1, numOfRows + 1, col, row)))
		return;
	EvaluationError error;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	int tempCol = col;
	int tempRow = row;
	for (uint i = 0; i<floatVectorVector.size(); i++, tempCol++, tempCol++)
	{
		tempRow = row;
		FloatVector& vector = floatVectorVector[i];
		QString& label = labelsVector[i];
		setCell(outputSheet, tempCol, tempRow, label);
		tempRow++;
		double result;
		if (summaryStatistics)
		{
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Mean");
			error = NoError;
			dafunc_average(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Standard Error");
			error = NoError;
			dafunc_standard_error(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Median");
			error = NoError;
			dafunc_median(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Standard Deviation");
			error = NoError;
			dafunc_standard_deviation(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Sample Variance");
			error = NoError;
			dafunc_variance(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Kutrosis");
			error = NoError;
			dafunc_kurtosis(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Skewness");
			error = NoError;
			dafunc_skewness(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Range");
			error = NoError;
			dafunc_range(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Minimum");
			error = NoError;
			dafunc_minimum(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Maximum");
			error = NoError;
			dafunc_maximum(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Sum");
			error = NoError;
			dafunc_sum(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Count");
			error = NoError;
			dafunc_count(vector, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
		}
		if (kthLargest)
		{
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Largest(" + QString::number(kthLargestNum) + ")");
			error = NoError;
			dafunc_maximum(vector, error, result, kthLargestNum);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
		}
		if (kthSmallest)
		{
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Smallest(" + QString::number(kthSmallestNum) + ")");
			error = NoError;
			dafunc_minimum(vector, error, result, kthSmallestNum);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
		}
		if (confidenceLevel)
		{
			tempRow++;
			setCell(outputSheet, tempCol, tempRow, "Confidence Level(" + QString::number(confidenceLevelNum) + "%)");
			error = NoError;
			dafunc_confidence_level(vector, confidenceLevelNum / 100, error, result);
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
		}
	}
	setTitlesRow(outputSheet, col, tempCol - 1, row);
	setEndRow(outputSheet, col, tempCol - 1, tempRow);
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}


KSpreadDlgDescriptiveStatistics::~KSpreadDlgDescriptiveStatistics()
{
	kdDebug() <<"KSpreadDlgDescriptiveStatistics::~KSpreadDlgDescriptiveStatistics() " << endl;
}

#include "kspread_dlg_da_descriptive_statistics.moc"
