#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_fourier_analysis.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <kmessagebox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>

#define INPUT_NAME "Input Range"

KSpreadDlgFourierAnalysis::KSpreadDlgFourierAnalysis(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgFourierAnalysis::KSpreadDlgFourierAnalysis() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	QGroupBox* inputGroup = new QGroupBox("Input", mainBox, "inputGroup");
	QVBoxLayout* inputLayout = new QVBoxLayout(inputGroup, marginHint(), spacingHint());
	QGrid* grid1 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid1);
	new QLabel("Input Range: ", grid1);
	new QEditButton(grid1, "Input Range");
	QGrid* grid2 = new QGrid(1, inputGroup);
	inputLayout->addWidget(grid2);
	new QCheckBox("Labels", grid2, "labelsCheckBox");

	KSpreadOutputGroupBox* outputGroup = new KSpreadOutputGroupBox(this, mainBox);
	QVBoxLayout* layout = outputGroup->layout();
	QGrid* grid4 = new QGrid(1, outputGroup);
	layout->addWidget(grid4);
	new QCheckBox("Inverse", grid4, "inverseCheckBox");
}

void KSpreadDlgFourierAnalysis::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();

	QString inputRange = map[INPUT_NAME];
	QRect rect = getRange(inputRange);
	SheetOrientation orientation;
	QStringVector* labelsVectorPtr;
	if ((orientation = detectOrientation(INPUT_NAME, rect)) == OrientationAll)
		return;
	labelsVectorPtr = getLabels(getSheet(inputRange), rect, orientation, (map["labelsCheckBox"] == "1"));
	QStringVector& labelsVector = *labelsVectorPtr;
	FloatVectorVector* floatVectorVectorPtr;
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, orientation)))
		return;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	hlpfunc_remove_empty_vectors(floatVectorVector, labelsVector);
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector, 1))
		return;
	FloatVector& floatVector = floatVectorVector[0];
	int vectorSize = (int) floatVector.size();
	if (vectorSize < 2 || (vectorSize & (vectorSize - 1)))
	{
		QString errorStr = "The number of input values must be a power of 2. For example: 2, 4, 8, 16, 32, 64, 128, ...";
		KMessageBox::information(this, errorStr);
		return;
	}
	int col, row;
	int addRow, addColumn;
	int areaWidth, areaHeight;
	bool inverse = (map["inverseCheckBox"] == "1");
	FloatVector realOut, imagOut;
	dafunc_fast_fourier_transform(floatVector, inverse, realOut, imagOut);

	KSpread::Sheet* outputSheet;
	if (orientation == OrientationColumn)
	{
		addRow = 1;
		addColumn = 0;
		areaWidth = 0;
		areaHeight = realOut.size() - 1;
	}
	else
	{
		addRow = 0;
		addColumn = 1;
		areaWidth = realOut.size() - 1;
		areaHeight = 0;
	}
	if (!(outputSheet = createOutput(map, areaWidth, areaHeight, col, row)))
		return;

// drawing begin
	kspreadView->doc()->emitBeginOperation();
	for (uint i = 0; i < realOut.size(); i++)
	{
		QString real = hlpfunc_result_string(realOut[i]);
		QString imag = hlpfunc_result_string(imagOut[i]);
		QString value = real;
		if (imag != "0")
		{
			if (imagOut[i] > 0)
				value += "+";
			value += imag + "i";
		}
		setCell(outputSheet, col, row, value);
		row +=addRow;
		col += addColumn;
	}
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}

KSpreadDlgFourierAnalysis::~KSpreadDlgFourierAnalysis()
{
	kdDebug() <<"KSpreadDlgFourierAnalysis::~KSpreadDlgFourierAnalysis() " << endl;
}

#include "kspread_dlg_da_fourier_analysis.moc"
