#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_histogram.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <math.h>

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <kmessagebox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>

#define INPUT_NAME "Input Range"

KSpreadDlgHistogram::KSpreadDlgHistogram(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgHistogram::KSpreadDlgHistogram() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	QGroupBox* inputGroup = new QGroupBox("Input", mainBox, "inputGroup");
	QVBoxLayout* inputLayout = new QVBoxLayout(inputGroup, marginHint(), spacingHint());
	QGrid* grid1 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid1);
	new QLabel("Input Range: ", grid1);
	QEditButton* inputRangeEB = new QEditButton(grid1, "Input Range");
	inputRangeEB->setMargin(3);
	new QLabel("Bin Range: ", grid1);
	QEditButton* binRangeEB = new QEditButton(grid1, "Bin Range");
	binRangeEB->setMargin(3);
	binRangeEB->setCanBeEmpty(true);
	QGrid* grid2 = new QGrid(1, inputGroup);
	inputLayout->addWidget(grid2);
	new QCheckBox("Labels", grid2, "labelsCheckBox");

	KSpreadOutputGroupBox* outputGroup = new KSpreadOutputGroupBox(this, mainBox);
	QVBoxLayout* layout = outputGroup->layout();
	QGrid* grid3 = new QGrid(1, outputGroup);
	layout->addWidget(grid3);
	new QCheckBox("Pareto (sorted histogram)", grid3, "sortedHistogramCheckBox");
	new QCheckBox("Cumulative Percentage", grid3, "cumulativePercentageCheckBox");
	new QCheckBox("Chart Ouput", grid3, "chartOutputCheckBox");

	inputRangeEB->setFocus();
}

void KSpreadDlgHistogram::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();
	QString inputRange = map[INPUT_NAME];
	bool labels = (map["labelsCheckBox"] == "1");
	bool sortedHistogram = (map["sortedHistogramCheckBox"] == "1");
	bool cumulativePercentage = (map["cumulativePercentageCheckBox"] == "1");
	bool chartOutput = (map["chartOutputCheckBox"] == "1");
	QString inputName = INPUT_NAME;
	FloatVector* floatVectorPtr;
	if (!(floatVectorPtr = getVector(inputName, inputRange, 2, labels, *(new QString()))))
		return;
	FloatVector& floatVector = *floatVectorPtr;
	QString binRange = map["Bin Range"];
	QString binTitle = "Bin";
	FloatVector binVector;
	if (binRange == "")
	{
		EvaluationError error;
		double count;
		error = NoError;
		dafunc_count(floatVector, error, count);
		int intervals = (int) sqrt(count);
		double min;
		error = NoError;
		dafunc_minimum(floatVector, error, min);
		double max;
		error = NoError;
		dafunc_maximum(floatVector, error, max);
		double increment = (max - min) / intervals;
		for (int i = 0; i < intervals; i++)
			binVector[i] = min + (i  * increment);
	}
	else
	{
		if (!(floatVectorPtr = getVector("Bin Range", binRange, 1, labels, binTitle)))
			return;
		binVector = *floatVectorPtr;
		hlpfunc_sort(binVector, binVector, SortAscending);
	}
	FloatVector freqVector;
	EvaluationError error = NoError;
	dafunc_frequency_vector(floatVector, binVector, error, freqVector);
	binVector[binVector.size()] = HUGE_VAL;
	int col, row;
	int width = ((1 + sortedHistogram) * (2 + cumulativePercentage)) - 1;
	int height = binVector.size();
	int valuesIt = -1;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, width, height, col, row)))
		return;
	double result;
	ChartParameters chartParams;
	FloatVector xValues;
	if (chartOutput)
	{
		chartParams.startPoint = QPoint(selectionStartPoint->x(), selectionEndPoint->y() + 1);
		chartParams.size = QSize(5, 25);
		chartParams.chartType = ChartBar;
		chartParams.chartName = this->name();
		chartParams.xAxisName =  binTitle;
		chartParams.yAxisName = "Frequency";
		chartParams.xLabels = new QStringVector();
		QStringVector* tempVector = new QStringVector();
		tempVector->push_back(chartParams.yAxisName);
		chartParams.yLabels = tempVector;
		chartParams.xValues = new FloatVectorVector();
	}
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	int tempCol = col;

	for (uint j = 0; j <= sortedHistogram; j++, tempCol += (2 + cumulativePercentage))
	{
		setCell(outputSheet, tempCol, row, binTitle);
		setCell(outputSheet, tempCol + 1, row, "Frequency");
		if (cumulativePercentage)
			setCell(outputSheet, tempCol + 2, row, "Value %");
		FloatVector tempBinVector;
		FloatVector tempFreqVector;
		if (j > 0)
		{
			FloatVectorVector vectorVector;
			vectorVector.push_back(binVector);
			vectorVector.push_back(freqVector);
			hlpfunc_sort(vectorVector, 1, vectorVector, SortDescending);
			tempBinVector = vectorVector[0];
			tempFreqVector = vectorVector[1];
		}
		else
		{
			tempBinVector = binVector;
			tempFreqVector = freqVector;
		}
		int tempRow = row + 1;
		bool chart = chartOutput && (!sortedHistogram || (sortedHistogram && j > 0));
		for (uint i = 0; i < tempBinVector.size(); i++, tempRow++)
		{
			result = tempBinVector[i];
			QString binStart = result == HUGE_VAL ? "More" : hlpfunc_result_string(error, result);
			setCell(outputSheet, tempCol, tempRow, binStart);
			if (chart)
				chartParams.xLabels->push_back(binStart);
			result = tempFreqVector[i];
			setCell(outputSheet, tempCol + 1, tempRow, hlpfunc_result_string(error, result));
			if (chart)
				xValues[++valuesIt] = result;
			if (cumulativePercentage)
			{
				error = NoError;
				dafunc_frequency_cumulative_percentage(floatVector, tempFreqVector, i, error, result);
				setCell(outputSheet, tempCol + 2, tempRow, hlpfunc_result_string(error, result, 4) + "%");
			}
		}
	}
	setTitlesRow(outputSheet, col, col + width, row);
	setEndRow(outputSheet, col, col + width, row + height);
	if (chartOutput)
	{
		chartParams.xValues->push_back(xValues);
		insertChart(&chartParams);
	}
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}

FloatVector* KSpreadDlgHistogram::getVector(QString inputName, QString inputRange, uint minDataPoints, bool labels, QString& title)
{
	QRect rect = getRange(inputRange);
	SheetOrientation orientation;
	if (rect.width() == 1)
		orientation = OrientationColumn;
	else
	if (rect.height() == 1)
		orientation = OrientationRow;
	else
		orientation = OrientationAll;
	QStringVector* labelsVectorPtr = getLabels(getSheet(inputRange), rect, orientation, labels);
	FloatVectorVector* floatVectorVectorPtr;
	if (!(floatVectorVectorPtr = getFloats(inputName, getSheet(inputRange), rect, OrientationAll)))
		return NULL;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	QStringVector labelsVector;
	if (orientation != OrientationAll)
	{
		labelsVector = *labelsVectorPtr;
		if (labels)
			title = labelsVector[0];
	}
	hlpfunc_remove_empty_vectors(floatVectorVector, labelsVector);
	if (!checkMinVectorSize(inputName, floatVectorVector, 1))
		return NULL;
	FloatVector& floatVector = floatVectorVector[0];
	if (!checkMinVectorSize(inputName, floatVector, minDataPoints))
		return NULL;
	return &floatVector;
}

KSpreadDlgHistogram::~KSpreadDlgHistogram()
{
	kdDebug() <<"KSpreadDlgHistogram::~KSpreadDlgHistogram() " << endl;
}

#include "kspread_dlg_da_histogram.moc"
