#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_moving_average.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <kmessagebox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>

#define INPUT_NAME "Input Range"

KSpreadDlgMovingAverage::KSpreadDlgMovingAverage(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgMovingAverage::KSpreadDlgMovingAverage() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	QGroupBox* inputGroup = new QGroupBox("Input", mainBox, "inputGroup");
	QVBoxLayout* inputLayout = new QVBoxLayout(inputGroup, marginHint(), spacingHint());
	QGrid* grid1 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid1);
	new QLabel("Input Range: ", grid1);
	new QEditButton(grid1, "Input Range");
	QGrid* grid2 = new QGrid(1, inputGroup);
	inputLayout->addWidget(grid2);
	new QCheckBox("Labels", grid2, "labelsCheckBox");
	QGrid* grid3 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid3);
	new QLabel("Interval: ", grid3);
	new QLineEdit(grid3, "intervalEdit");

	KSpreadOutputGroupBox* outputGroup = new KSpreadOutputGroupBox(this, mainBox);
	QVBoxLayout* layout = outputGroup->layout();
	QGrid* grid4 = new QGrid(2, outputGroup);
	layout->addWidget(grid4);
	new QCheckBox("Chart Ouput", grid4, "chartOutputCheckBox");
	new QCheckBox("Standard Errors", grid4, "standardErrorsCheckBox");
}

void KSpreadDlgMovingAverage::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();

	QString inputRange = map[INPUT_NAME];
	QRect rect = getRange(inputRange);
	SheetOrientation orientation;
	QStringVector* labelsVectorPtr;
	if ((orientation = detectOrientation(INPUT_NAME, rect)) == OrientationAll)
		return;
	labelsVectorPtr = getLabels(getSheet(inputRange), rect, orientation, (map["labelsCheckBox"] == "1"));
	QStringVector& labelsVector = *labelsVectorPtr;
	FloatVectorVector* floatVectorVectorPtr;
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, orientation)))
		return;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	hlpfunc_remove_empty_vectors(floatVectorVector, labelsVector);
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector, 1))
		return;
	FloatVector& floatVector = floatVectorVector[0];
	if (!checkMinVectorSize(INPUT_NAME, floatVector, 4))
		return;
	int intervalNum;
	QString value = map["intervalEdit"];
	if (value == "")
		intervalNum = 3;
	else
	{
		if (!validateDataPoints("Interval", value, floatVector.size()))
			return;
		intervalNum = value.toInt();
	}
	bool standardErrors = (map["standardErrorsCheckBox"] == "1");
	bool chartOutput = (map["chartOutputCheckBox"] == "1");
	int vectorSize = (int) floatVector.size();
	if (intervalNum < 2 || intervalNum > vectorSize ||
		(standardErrors && (2 * intervalNum > vectorSize + 1)))
	{
		QString errorStr = "Estimated output table will extend beyond the bounds of the worksheet. Please choose different reference.";
		KMessageBox::information(this, errorStr);
		return;
	}
	int col, row;
	int addRow, addColumn;
	int areaWidth, areaHeight;

	KSpread::Sheet* outputSheet;
	if (orientation == OrientationColumn)
	{
		addRow = 1;
		addColumn = 0;
		areaWidth = standardErrors;
		areaHeight = vectorSize - 1;
	}
	else
	{
		addRow = 0;
		addColumn = 1;
		areaWidth = vectorSize - 1;
		areaHeight = standardErrors;
	}
	if (!(outputSheet = createOutput(map, areaWidth, areaHeight, col, row)))
		return;
	EvaluationError error;
	double result;
	ChartParameters chartParams;
	int chartCount = -1;
	FloatVector actualVector;
	FloatVector forecastVector;
	if (chartOutput)
	{
		chartParams.startPoint = QPoint(selectionStartPoint->x(), selectionEndPoint->y() + 1);
		chartParams.size = QSize(5, 25);
		chartParams.chartType = ChartLine;
		chartParams.chartName = this->name();
		chartParams.xAxisName = "Value";
		chartParams.yAxisName = "Data Point";
		chartParams.xLabels = new QStringVector();
		QStringVector* tempVector = new QStringVector();
		tempVector->push_back("Actual");
		tempVector->push_back("Forecast");
		chartParams.yLabels = tempVector;
		chartParams.xValues = new FloatVectorVector();
	}
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	for (int i = 0; i < vectorSize; i++)
	{
		error = NoError;
		dafunc_moving_average(floatVector, i, intervalNum, error, result);
		setCell(outputSheet, col, row, hlpfunc_result_string(error, result));
		if (chartOutput && error == NoError)
		{
			chartCount++;
			actualVector[chartCount] = floatVector[i];
			forecastVector[chartCount] = result;
			chartParams.xLabels->push_back(QString::number(i + 1));
		}
		if (standardErrors)
		{
			error = NoError;
			dafunc_moving_average_standard_error(floatVector, i, intervalNum, error, result);
			setCell(outputSheet, col + addRow, row + addColumn, hlpfunc_result_string(error, result));
		}
		row +=addRow;
		col += addColumn;
	}
	if (chartOutput)
	{
		chartParams.xValues->push_back(actualVector);
		chartParams.xValues->push_back(forecastVector);
		insertChart(&chartParams);
	}
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}


KSpreadDlgMovingAverage::~KSpreadDlgMovingAverage()
{
	kdDebug() <<"KSpreadDlgMovingAverage::~KSpreadDlgMovingAverage() " << endl;
}

#include "kspread_dlg_da_moving_average.moc"
