#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_rank_percentile.h"
#include "kspread_inputrange1groupbox.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qvbox.h>
#include <qcheckbox.h>
#include <kmessagebox.h>

#define INPUT_NAME "Input Range"

KSpreadDlgRankPercentile::KSpreadDlgRankPercentile(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgRankPercentile::KSpreadDlgRankPercentile() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);

	new KSpreadInputRange1GroupBox(this, mainBox, "groupedByGroup");

	new KSpreadOutputGroupBox(this, mainBox);
}

void KSpreadDlgRankPercentile::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();
	QString inputRange = map[INPUT_NAME];
	QRect rect = getRange(inputRange);
	SheetOrientation orientation = (map["groupedByGroup"] == "groupedByColumnsButton") ? OrientationColumn : OrientationRow;
	QStringVector* labelsVectorPtr;
	labelsVectorPtr = getLabels(getSheet(inputRange), rect, orientation, (map["labelsCheckBox"] == "1"));
	QStringVector& labelsVector = *labelsVectorPtr;
	FloatVectorVector* floatVectorVectorPtr;
	IntVectorVector* missingVectorVectorPtr = new IntVectorVector();
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, orientation, EmptyShift, missingVectorVectorPtr)))
		return;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	IntVectorVector& missingVectorVector = *missingVectorVectorPtr;
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector, 1))
		return;
	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, 4 * floatVectorVector.size() - 1, floatVectorVector[0].size() + missingVectorVector[0].size(), col, row)))
		return;
	double result;
	EvaluationError error;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	int tempCol = col;
	int tempRow = row;
	for (uint i = 0; i<floatVectorVector.size(); i++, tempCol += 4)
	{
		QString& label = labelsVector[i];
		FloatVector& floatVector = floatVectorVector[i];
		IntVector& missingKeys = missingVectorVector[i];
		FloatVector sortedVector;
		hlpfunc_sort(floatVector, sortedVector, SortDescending);
		setCell(outputSheet, tempCol, row, "Point");
		setCell(outputSheet, tempCol + 1, row, label);
		setCell(outputSheet, tempCol + 2, row, "Rank");
		setCell(outputSheet, tempCol + 3, row, "Percent");
		tempRow = row + 1;
		for (uint j = 0; j < sortedVector.size(); j++, tempRow++)
		{
			uint point = *hlpfunc_find_key(floatVector, sortedVector[j]);
			setCell(outputSheet, tempCol, tempRow, QString::number(point + 1));
			setCell(outputSheet, tempCol + 1, tempRow, QString::number(sortedVector[j]));
			floatVector.remove(point);
			error = NoError;
			dafunc_rank(sortedVector, j, error, result);
			setCell(outputSheet, tempCol + 2, tempRow, hlpfunc_result_string(error, result));
			error = NoError;
			dafunc_percent(sortedVector, j, error, result);
			setCell(outputSheet, tempCol + 3, tempRow, hlpfunc_result_string(error, result, 4) + "%");
		}
		error = NA;
		for (int j = 0; j < (int) missingKeys.size(); j++, tempRow++)
		{
			uint point = missingKeys[j];
			setCell(outputSheet, tempCol, tempRow, QString::number(point + 1));
			setCell(outputSheet, tempCol + 1, tempRow, "");
			setCell(outputSheet, tempCol + 2, tempRow, hlpfunc_result_string(error, result));
			setCell(outputSheet, tempCol + 3, tempRow, hlpfunc_result_string(error, result));
		}
	}
	setTitlesRow(outputSheet, col, tempCol - 1, row);
	setEndRow(outputSheet, col, tempCol - 1, tempRow - 1);
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}


KSpreadDlgRankPercentile::~KSpreadDlgRankPercentile()
{
	kdDebug() <<"KSpreadDlgRankPercentile::~KSpreadDlgRankPercentile() " << endl;
}

#include "kspread_dlg_da_rank_percentile.moc"
