#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_regression.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <math.h>

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <kmessagebox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qgroupbox.h>

#define INPUT_NAME "Input Range"

KSpreadDlgRegression::KSpreadDlgRegression(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgRegression::KSpreadDlgRegression() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	QGroupBox* inputGroup = new QGroupBox("Input", mainBox, "inputGroup");
	QVBoxLayout* inputLayout = new QVBoxLayout(inputGroup, marginHint(), spacingHint());
	QGrid* grid1 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid1);
	new QLabel("Input Y Range: ", grid1);
	QEditButton* inputYRangeEB = new QEditButton(grid1, "Input Y Range");
	inputYRangeEB->setMargin(3);
	new QLabel("Input X Range: ", grid1);
	QEditButton* inputXRangeEB = new QEditButton(grid1, "Input X Range");
	inputXRangeEB->setMargin(3);
	QGrid* grid2 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid2);
	new QCheckBox("Labels", grid2, "labelsCheckBox");
	new QCheckBox("Constant is Zero", grid2, "constantZeroCheckBox");
	QCheckBox* confidenceLevelCheckBox = new QCheckBox("Confidance Level: ", grid2, "confidenceLevelCheckBox");
	QGrid* confidenceLevelEditGrid = new QGrid(2, grid2);
	confidenceLevelEditGrid->setMargin(3);
	QLineEdit* confidenceLevelEdit = new QLineEdit("95", confidenceLevelEditGrid, "confidenceLevelEdit");
	new QLabel("%", confidenceLevelEditGrid);

	KSpreadOutputGroupBox* outputGroup = new KSpreadOutputGroupBox(this, mainBox);
	QVBoxLayout* layout = outputGroup->layout();
	QGroupBox* residualsGroupBox = new QGroupBox("Residuals", outputGroup);
	layout->addWidget(residualsGroupBox);
	QVBoxLayout* residualsLayout = new QVBoxLayout(residualsGroupBox, marginHint(), spacingHint());
	QGrid* grid3 = new QGrid(1, residualsGroupBox);
	residualsLayout->addWidget(grid3);
	new QCheckBox("Residuals", grid3, "residualsCheckBox");
	new QCheckBox("Standarized Residuals", grid3, "standarizedResidualsCheckBox");

//	inputYRangeEB->setFocus();
	confidenceLevelEdit->setEnabled(false);

	connectFocus(confidenceLevelCheckBox, confidenceLevelEdit);
}

#define getError(error) (error == NoError ? NoError : error)

void KSpreadDlgRegression::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();
	FloatVectorVector floatVectorVectorT[2];
	QStringVector labelsVectorT[2];
	QString name[2];
	QString variableRange[2];
	QRect rect[2];
	SheetOrientation orientation[2];
	QString varStr[2] = {"Y", "X"};
	bool labels = (map["labelsCheckBox"] == "1");
	for (int i = 0; i < 2; i++)
	{
		name[i] = "Input " + varStr[i] + " Range";
		variableRange[i] = map[name[i]];
		rect[i] = getRange(variableRange[i]);
		if (i > 0)
			orientation[i] = OrientationColumn;
		else
			if ((orientation[i] = detectOrientation(name[i], rect[i])) == OrientationAll)
				return;
		QStringVector* labelsVectorPtr;
		labelsVectorPtr = getLabels(getSheet(variableRange[i]), rect[i], orientation[i], labels);
		labelsVectorT[i] = *labelsVectorPtr;
		FloatVectorVector* floatVectorVectorPtr;
		if (!(floatVectorVectorPtr = getFloats(name[i], getSheet(variableRange[i]), rect[i], orientation[i], NoEmpty)))
			return;
		floatVectorVectorT[i] = *floatVectorVectorPtr;
	}
	QString& varYLabel = labelsVectorT[0][0];
	FloatVector& varYVector = floatVectorVectorT[0][0];
	QStringVector& varXlabelsVector = labelsVectorT[1];
	FloatVectorVector& varXVectorVector = floatVectorVectorT[1];
	bool confidenceLevel = (map["confidenceLevelCheckBox"] == "1");
	bool constantZero = (map["constantZeroCheckBox"] == "1");
	bool residuals = (map["residualsCheckBox"] == "1");
	bool standardResiduals = (map["standarizedResidualsCheckBox"] == "1");
	double confidenceLevelNum = -1;
	if (confidenceLevel)
	{
		QString value = map["confidenceLevelEdit"];
		if (!validateNumber("Confidence", value, 0, 100))
			return;
		confidenceLevelNum = value.toDouble();
	}
	FloatVector varBVector;
	EvaluationError varBError = NoError;
	dafunc_regression_b_variables(varXVectorVector, varYVector, constantZero, varBError, varBVector);
	FloatVector stdErrorVector;
	EvaluationError stdErrorError = varBError;
	dafunc_regression_v_variables(varBVector, varXVectorVector, varYVector, constantZero, stdErrorError, stdErrorVector);
	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, 6 + confidenceLevel  * 2, 16 + varXVectorVector.size() + residuals * (6 + varYVector.size()), col, row)))
		return;
	EvaluationError error;
	double result;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	setCell(outputSheet, col, row, "SUMMARY OUTPUT");
	row += 2;
	setCell(outputSheet, col, row, "Regression Statistics");
	setTitlesRow(outputSheet, col, col + 1, row);
	row++;
	setCell(outputSheet, col, row, "Multiple R");
	error = NoError;
	dafunc_regression_multiple_r(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "R Square");
	error = getError(varBError);
	dafunc_regression_r_square(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "Adjusted R Square");
	error = getError(varBError);
	dafunc_regression_adjusted_r_square(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "Standard Error");
	error = getError(varBError);
	dafunc_regression_standard_error(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "Observations");
	error = NoError;
	dafunc_count(varYVector, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));	
	setEndRow(outputSheet, col, col + 1, row);
	row += 2;
	setCell(outputSheet, col, row, "ANOVA");
	row++;
	setCell(outputSheet, col + 1, row, "df");
	setCell(outputSheet, col + 2, row, "SS");
	setCell(outputSheet, col + 3, row, "MS");
	setCell(outputSheet, col + 4, row, "F");
	setCell(outputSheet, col + 5, row, "Signifcance F");
	setTitlesRow(outputSheet, col, col + 5, row);
	row++;
	setCell(outputSheet, col, row, "Regression");
	error = getError(varBError);
	dafunc_regression_regression_df(varBVector, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = getError(varBError);
	dafunc_regression_regression_ss(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	error = getError(varBError);
	dafunc_regression_regression_ms(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
	error = getError(varBError);
	dafunc_regression_f_stat(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
	error = getError(varBError);
	dafunc_regression_f_dist(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 5, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "Residuals");
	error = getError(varBError);
	dafunc_regression_residuals_df(varBVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = getError(varBError);
	dafunc_regression_residuals_ss(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	error = getError(varBError);
	dafunc_regression_residuals_ms(varBVector, varXVectorVector, varYVector, constantZero, error, result);
	setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
	row++;
	setCell(outputSheet, col, row, "Total");
	dafunc_regression_total_df(varYVector, constantZero, error, result);
	setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
	error = getError(varBError);
	dafunc_regression_total_ss(varYVector, constantZero, error, result);
	setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
	setEndRow(outputSheet, col, col + 5, row);
	row += 2;
	setCell(outputSheet, col + 1, row, "Coefficents");
	setCell(outputSheet, col + 2, row, "Standard Error");
	setCell(outputSheet, col + 3, row, "t Stat");
	setCell(outputSheet, col + 4, row, "P-value");
	setCell(outputSheet, col + 5, row, "Lower 95%");
	setCell(outputSheet, col + 6, row, "Upper 95%");
	if (confidenceLevel)
	{
		QString num = QString::number(confidenceLevelNum) + "%";
		setCell(outputSheet, col + 7, row, "Lower " + num);
		setCell(outputSheet, col + 8, row, "Upper " + num);
	}
	setTitlesRow(outputSheet, col, col + 6 + confidenceLevel * 2, row);
	row++;
	for (uint i = 0; i < varBVector.size(); i++, row++)
	{
		QString labelStr;
		if (i > 0)
			labelStr = labels ? varXlabelsVector[i - 1] : "X Variable " + QString::number(i + 1);
		else
			labelStr = "Intercept";
		setCell(outputSheet, col, row, labelStr);
		if (constantZero && i == 0)
		{
			error = NoError;
			result = 0;
			setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
			uint loop = 6 + 2 * confidenceLevel;
			error = NA;
			for (uint i = 2; i <= loop; i++)
				setCell(outputSheet, col + i, row, hlpfunc_result_string(error, result));
		}
		else
		{
			error = getError(varBError);
			result = varBVector[i];
			setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
			error = getError(stdErrorError);
			result = stdErrorVector[i];
			setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
			error = getError(stdErrorError);
			dafunc_regression_t_stat(varBVector, stdErrorVector, i, error, result);
			setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
			error = getError(stdErrorError);
			dafunc_regression_t_test(varYVector, varBVector, stdErrorVector, i, constantZero,  error, result);
			setCell(outputSheet, col + 4, row, hlpfunc_result_string(error, result));
			error = getError(stdErrorError);
			dafunc_regression_t_crit_lower(varYVector, varBVector, stdErrorVector, 95, i, constantZero, error, result);
			setCell(outputSheet, col + 5, row, hlpfunc_result_string(error, result));
			error = getError(stdErrorError);
			dafunc_regression_t_crit_upper(varYVector, varBVector, stdErrorVector, 95, i, constantZero, error, result);
			setCell(outputSheet, col + 6, row, hlpfunc_result_string(error, result));
			if (confidenceLevel)
			{
				error = getError(stdErrorError);
				dafunc_regression_t_crit_lower(varYVector, varBVector, stdErrorVector, confidenceLevelNum, i, constantZero, error, result);
				setCell(outputSheet, col + 7, row, hlpfunc_result_string(error, result));
				error = getError(stdErrorError);
				dafunc_regression_t_crit_upper(varYVector, varBVector, stdErrorVector, confidenceLevelNum, i, constantZero, error, result);
				setCell(outputSheet, col + 8, row, hlpfunc_result_string(error, result));
			}
		}
	}
	row--;
	setEndRow(outputSheet, col, col + 6 + confidenceLevel * 2, row);
	if (residuals)
	{
		row += 4;
		setCell(outputSheet, col, row, "RESIDUAL OUTPUT");
		row += 2;
		setCell(outputSheet, col, row, "Observation");
		setCell(outputSheet, col + 1, row, "Predicted " + varYLabel);
		setCell(outputSheet, col + 2, row, "Residuals");
		if (standardResiduals)
			setCell(outputSheet, col + 3, row, "Standard Residuals");
		setTitlesRow(outputSheet, col, col + 2 + standardResiduals, row);
		row++;
		for (uint i = 0; i < varYVector.size(); i++, row++)
		{
			setCell(outputSheet, col, row, QString::number(i + 1));
			error = getError(varBError);
			dafunc_regression_predicted_y(varBVector, varXVectorVector, i, error, result);
			setCell(outputSheet, col + 1, row, hlpfunc_result_string(error, result));
			error = getError(varBError);
			dafunc_regression_residuals(varBVector, varXVectorVector, varYVector, i, error, result);
			setCell(outputSheet, col + 2, row, hlpfunc_result_string(error, result));
			error = getError(varBError);
			if (standardResiduals)
			{
				error = getError(varBError);
				dafunc_regression_standarized_residuals(varBVector, varXVectorVector, varYVector, i, error, result);
				setCell(outputSheet, col + 3, row, hlpfunc_result_string(error, result));
			}
		}
		row--;
		setEndRow(outputSheet, col, col + 2 + standardResiduals, row);
	}
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}

KSpreadDlgRegression::~KSpreadDlgRegression()
{
	kdDebug() <<"KSpreadDlgRegression::~KSpreadDlgRegression()" << endl;
}

#include "kspread_dlg_da_regression.moc"
