#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_sampling.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <kmessagebox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlayout.h>

#define INPUT_NAME "Input Range"

KSpreadDlgSampling::KSpreadDlgSampling(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgSampling::KSpreadDlgSampling() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);
	QGroupBox* inputGroup = new QGroupBox("Input", mainBox, "inputGroup");
	QVBoxLayout* inputLayout = new QVBoxLayout(inputGroup, marginHint(), spacingHint());
	QGrid* grid1 = new QGrid(2, inputGroup);
	inputLayout->addWidget(grid1);
	new QLabel("Input Range: ", grid1);
	new QEditButton(grid1, INPUT_NAME);
	QGrid* grid2 = new QGrid(1, inputGroup);
	inputLayout->addWidget(grid2);
	new QCheckBox("Labels", grid2, "labelsCheckBox");

	QButtonGroup* samplingMethodGroup = new QButtonGroup("Sampling Method", mainBox, "samplingMethodGroup");
	QVBoxLayout* samplingMethodLayout = new QVBoxLayout(samplingMethodGroup, marginHint(), spacingHint());
	QGrid* grid4 = new QGrid(1, samplingMethodGroup);
	grid4->setMargin(5);
	samplingMethodLayout->addWidget(grid4);
	QRadioButton* periodicButton = new QRadioButton("Periodic", grid4, "periodicButton");
	QGrid* grid4a = new QGrid(2, grid4);
	new QLabel("Period: ", grid4a);
	QLineEdit* periodEdit = new QLineEdit(grid4a, "periodEdit");
	QGrid* grid5 = new QGrid(1, samplingMethodGroup);
	grid5->setMargin(5);

	samplingMethodLayout->addWidget(grid5);
	QRadioButton* randomButton = new QRadioButton("Random", grid5, "randomButton");
	QGrid* grid5a = new QGrid(2, grid5);
	new QLabel("Number of Samples: ", grid5a);
	QLineEdit* randomEdit = new QLineEdit(grid5a, "randomEdit");

	KSpreadOutputGroupBox* outputGroup = new KSpreadOutputGroupBox(this, mainBox);
	QVBoxLayout* layout = outputGroup->layout();
	QGrid* grid6 = new QGrid(1, outputGroup);
	layout->addWidget(grid6);

	samplingMethodGroup->insert(periodicButton);
	samplingMethodGroup->insert(randomButton);

	connectFocus(periodicButton,  periodEdit);
	connectFocus(randomButton,  randomEdit);

	periodicButton->toggle();
	randomEdit->setEnabled(false);
}

QString KSpreadDlgSampling::getValueString(uint position, FloatVector& vector)
{
	return vector.contains(position) ? hlpfunc_result_string(NoError, vector[position]) : "";
}


void KSpreadDlgSampling::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();

	QString inputRange = map[INPUT_NAME];
	QRect rect = getRange(inputRange);
	SheetOrientation orientation;

	if (rect.width() == 1)
		orientation = OrientationColumn;
	else
	if (rect.height() == 1)
		orientation = OrientationRow;
	else
		orientation = OrientationAll;

	getLabels(getSheet(inputRange), rect, orientation, (map["labelsCheckBox"] == "1"));
	FloatVectorVector* floatVectorVectorPtr;
	if (!(floatVectorVectorPtr = getFloats(INPUT_NAME, getSheet(inputRange), rect, OrientationAll, EmptyShift)))
		return;
	FloatVectorVector& floatVectorVector = *floatVectorVectorPtr;
	FloatVector& floatVector = floatVectorVector[0];
	if (!checkMinVectorSize(INPUT_NAME, floatVector, 1))
		return;
	uint vectorSize = rect.width() * rect.height();
	QString number;
	int col, row;
	bool periodic = (map["samplingMethodGroup"] == "periodicButton");
	KSpread::Sheet* outputSheet;
	uint areaHeight;
	uint periodNum = 0, randomNum = 0;
	if (periodic)
	{
		QString number = map["periodEdit"];
		if (!validateDataPoints("Period", number, vectorSize))
			return;
		periodNum = number.toUInt();
		areaHeight = (vectorSize - periodNum) / periodNum;
	}
	else
	{
		QString number = map["randomEdit"];
		if (!validateDataPoints("Number of Samples", number))
			return;
		randomNum = number.toUInt();
		areaHeight = randomNum - 1;
	}
	if (!(outputSheet = createOutput(map, 0, areaHeight, col, row)))
		return;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	if (periodic)
		for (uint i = periodNum - 1; i < vectorSize; i += periodNum, row++)
			setCell(outputSheet, col, row, getValueString(i, floatVector));
	else
	{
		EvaluationError error = NoError;
		FloatVector randomVector;
		dafunc_random_numbers(floatVector, vectorSize - 1, randomNum, error, randomVector);
		for (uint i = 0; i < randomVector.size(); i ++, row++)
			setCell(outputSheet, col, row, getValueString(i, randomVector));
	}
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}

KSpreadDlgSampling::~KSpreadDlgSampling()
{
	kdDebug() <<"KSpreadDlgSampling::~KSpreadDlgSampling() " << endl;
}

#include "kspread_dlg_da_sampling.moc"
