#include "kspread_functions_data_analysis.h"

#include "kspread_dlg_da_ztest_two_sample.h"
#include "kspread_inputrange2groupbox.h"
#include "kspread_outputgroupbox.h"
#include "kspread_dlg_data_analysis_template.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_doc.h"

#include <kdebug.h>
#include <qvbox.h>
#include <qgrid.h>

#define INPUT_NAME "Variables ranges"

KSpreadDlgZTestTwoSample::KSpreadDlgZTestTwoSample(KSpread::View* parent, const char* name)
    : KSpreadDlgDataAnalysisTemplate(parent, name, false, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgZTestTwoSample::KSpreadDlgZTestTwoSample() " << endl;
	
	kspreadView = parent;
	
	QVBox* mainBox = new QVBox(this);
	setMainWidget(mainBox);

	new KSpreadInputRange2GroupBox(this, mainBox, "inputGroup", true, true);

	new KSpreadOutputGroupBox(this, mainBox);
}

void KSpreadDlgZTestTwoSample::accept()
{
	if (!validateReferences())
		return;
	QWidgetMap map = allWidgets();
	FloatVectorVector floatVectorVectorT[2];
	QStringVector labelsVectorT[2];
	QString name[2];
	QString varianceName[2];
	QString variableRange[2];
	QRect rect[2];
	SheetOrientation orientation[2];
	for (int i = 0; i < 2; i++)
	{
		name[i] = "Variable " + QString::number(i + 1) + " Range";
		variableRange[i] = map[name[i]];
		rect[i] = getRange(variableRange[i]);
		if ((orientation[i] = detectOrientation(name[i], rect[i])) == OrientationAll)
			return;

		QStringVector* labelsVectorPtr;
		labelsVectorPtr = getLabels(getSheet(variableRange[i]), rect[i], orientation[i], (map["labelsCheckBox"] == "1"));
		labelsVectorT[i] = *labelsVectorPtr;
		FloatVectorVector* floatVectorVectorPtr;
		if (!(floatVectorVectorPtr = getFloats(name[i], getSheet(variableRange[i]), rect[i], orientation[i])))
			return;
		floatVectorVectorT[i] = *floatVectorVectorPtr;
		varianceName[i] = "Variance " + QString::number(i + 1);
	}
	QStringVector& labelsVector = *hlpfunc_merge_labels(labelsVectorT[0], labelsVectorT[1]);
	FloatVectorVector& floatVectorVector =  *hlpfunc_merge_vectors_vectors(floatVectorVectorT[0], floatVectorVectorT[1]);
	hlpfunc_remove_empty_vectors(floatVectorVector, labelsVector);
	if (!checkMinVectorSize(INPUT_NAME, floatVectorVector, 2))
		return;
	double meanDifferenceNum;
	QString value1 = map["meanDifferenceEdit"];
	if (value1 == "")
		meanDifferenceNum = 0;
	else
	{
		if (!validatePositiveNumber("Hypothesized Mean Difference", value1))
			return;
		meanDifferenceNum = value1.toDouble();
	}
	double knownVarianceNum[2];
	for (int i = 0; i < 2; i++)
	{
		QString value = map[varianceName[i]];
		if (!validatePositiveNumber(varianceName[i] , value))
			return;
		knownVarianceNum[i] = value.toDouble();
	}
	double alphaNum;
	QString value2 = map["alphaEdit"];
	if ( !validateNumber("Alpha", value2, 0, 1))
		return;
	alphaNum = value2.toDouble();

	int col, row;
	KSpread::Sheet* outputSheet;
	if (!(outputSheet = createOutput(map, 2, 11, col, row)))
		return;
	EvaluationError error;
// drawing begin
	kspreadView->doc()->emitBeginOperation();
	setCell(outputSheet, col, row, this->name());
	row++;
	row++;
	setCell(outputSheet, col, row + 1, "Mean");
	setCell(outputSheet, col, row + 2, "Known Variance");
	setCell(outputSheet, col, row + 3, "Observations");
	setCell(outputSheet, col, row + 4, "Hypothesized Mean Difference");
	setCell(outputSheet, col, row + 5, "z Stat");
	setCell(outputSheet, col, row + 6, "P(Z<=z) one-tail");
	setCell(outputSheet, col, row + 7, "z Critical one-tail");
	setCell(outputSheet, col, row + 8, "P(Z<=z) two-tail");
	setCell(outputSheet, col, row + 9, "z Critical two-tail");
	for (uint i = 1; i <= 2; i++)
	{
		FloatVector& vector = floatVectorVector[i - 1];
		QString& label = labelsVector[i - 1];
		setCell(outputSheet, col + i, row, label);
		double result;
		error = NoError;
		dafunc_average(vector, error, result);
		setCell(outputSheet, col + i, row + 1, hlpfunc_result_string(error, result));
		error = NoError;
		result =  knownVarianceNum[i - 1];
		setCell(outputSheet, col + i, row + 2, hlpfunc_result_string(error, result));
		error = NoError;
		dafunc_count(vector, error, result);
		setCell(outputSheet, col + i, row + 3, hlpfunc_result_string(error, result));
	}
	double result;
	error = NoError;
	result =  meanDifferenceNum;
	setCell(outputSheet, col + 1, row + 4, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_z_stat(floatVectorVector[0],  floatVectorVector[1], knownVarianceNum[0], knownVarianceNum[1], meanDifferenceNum, error, result);
	setCell(outputSheet, col + 1, row + 5, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_z_test(floatVectorVector[0],  floatVectorVector[1], knownVarianceNum[0], knownVarianceNum[1], meanDifferenceNum, 1, error, result);
	setCell(outputSheet, col + 1, row + 6, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_z_crit(alphaNum, 1, error, result);
	setCell(outputSheet, col + 1, row + 7, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_z_test(floatVectorVector[0],  floatVectorVector[1], knownVarianceNum[0], knownVarianceNum[1], meanDifferenceNum, 2, error, result);
	setCell(outputSheet, col + 1, row + 8, hlpfunc_result_string(error, result));
	error = NoError;
	dafunc_z_crit(alphaNum, 2, error, result);
	setCell(outputSheet, col + 1, row + 9, hlpfunc_result_string(error, result));
	setTitlesRow(outputSheet, col, col + 2, row);
	setEndRow(outputSheet, col, col + 2, row + 9);
	kspreadView->doc()->emitEndOperation();
// drawing end
	KSpreadDlgDataAnalysisTemplate::accept();
}


KSpreadDlgZTestTwoSample::~KSpreadDlgZTestTwoSample()
{
	kdDebug() <<"KSpreadDlgZTestTwoSample::~KSpreadDlgZTestTwoSample() " << endl;
}

#include "kspread_dlg_da_ztest_two_sample.moc"
