#include "kspread_dlg_data_analysis_main.h"

#include "kspread_dlg_da_anova_single_factor.h"
#include "kspread_dlg_da_anova_two_factor_repl.h"
#include "kspread_dlg_da_anova_two_factor.h"
#include "kspread_dlg_da_correlation.h"
#include "kspread_dlg_da_covariance.h"
#include "kspread_dlg_da_descriptive_statistics.h"
#include "kspread_dlg_da_exponential_smoothing.h"
#include "kspread_dlg_da_ftest_two_sample.h"
#include "kspread_dlg_da_fourier_analysis.h"
#include "kspread_dlg_da_histogram.h"
#include "kspread_dlg_da_moving_average.h"
#include "kspread_dlg_da_rank_percentile.h"
#include "kspread_dlg_da_regression.h"
#include "kspread_dlg_da_sampling.h"
#include "kspread_dlg_da_ttest_paired_two_sample.h"
#include "kspread_dlg_da_ttest_two_sample_eq_var.h"
#include "kspread_dlg_da_ttest_two_sample_uneq_var.h"
#include "kspread_dlg_da_ztest_two_sample.h"

#include "../../kspread_view.h"

#include <kapplication.h>
#include <kdebug.h>
#include <qevent.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qgrid.h>
	
KSpreadDlgDataAnalysisMain::KSpreadDlgDataAnalysisMain( KSpread::View* parent, const char* name)
	: KDialogBase( parent, name, true, name, Ok|Cancel)
{
	kdDebug() <<"KSpreadDlgDataAnalysisMain::KSpreadDlgDataAnalysisMain() " << endl;
	
	kspreadView = parent;
	
	QGrid *grid = new QGrid(1, this);
	setMainWidget(grid);
	new QLabel("Analysis Tools:", grid);
	functions = new QListBox(grid);
	functions->insertItem("Anova: Single Factor");
	functions->insertItem("Anova: Two Factor With Replication");
	functions->insertItem("Anova: Two Factor Without Replication");
	functions->insertItem("Correlation");
	functions->insertItem("Covariance");
	functions->insertItem("Descriptive Statistics");
	functions->insertItem("Exponential Smoothing");
	functions->insertItem("F-Test Two-Sample for Variances");
	functions->insertItem("Fourier Analysis");
	functions->insertItem("Histogram");
	functions->insertItem("Moving Average");
	functions->insertItem("Rank and Percentile");
	functions->insertItem("Regression");
	functions->insertItem("Sampling");
	functions->insertItem("t-Test: Paired Two Sample for Means");
	functions->insertItem("t-Test: Two Sample for Means Assuming Equal Variances");
	functions->insertItem("t-Test: Two Sample for Means Assuming Unequal Variances");
	functions->insertItem("z-Test: Two Sample for Means");
	functions->setCurrentItem(0);
	functions->setMinimumSize(370, 100);

	connect(this, SIGNAL(closeClicked()), SLOT(delayedDestruct()));
	connect(this, SIGNAL(cancelClicked()), SLOT(delayedDestruct()));
}
	
void KSpreadDlgDataAnalysisMain::accept()
{
	fDialog = NULL;
	const char* text = functions->currentText().ascii();
	switch(functions->currentItem()) 
	{
		case 0: fDialog = new KSpreadDlgAnovaSingleFactor(kspreadView, text); break;
		case 1: fDialog = new KSpreadDlgAnovaTwoFactorReplication(kspreadView, text); break;
		case 2: fDialog = new KSpreadDlgAnovaTwoFactor(kspreadView, text); break;
		case 3: fDialog = new KSpreadDlgCorrelation(kspreadView, text); break;
		case 4: fDialog = new KSpreadDlgCovariance(kspreadView, text); break;
		case 5: fDialog = new KSpreadDlgDescriptiveStatistics(kspreadView, text); break;
		case 6: fDialog = new KSpreadDlgExponentialSmoothing(kspreadView, text); break;
		case 7: fDialog = new KSpreadDlgFTestTwoSample(kspreadView, text); break;
		case 8: fDialog = new KSpreadDlgFourierAnalysis(kspreadView, text); break;
		case 9: fDialog = new KSpreadDlgHistogram(kspreadView, text); break;
		case 10: fDialog = new KSpreadDlgMovingAverage(kspreadView, text); break;
		case 11: fDialog = new KSpreadDlgRankPercentile(kspreadView, text); break;
		case 12: fDialog = new KSpreadDlgRegression(kspreadView, text); break;
		case 13: fDialog = new KSpreadDlgSampling(kspreadView, text); break;
		case 14: fDialog = new KSpreadDlgTTestPairedTwoSample(kspreadView, text); break;
		case 15: fDialog = new KSpreadDlgTTestTwoSampleEqualVariances(kspreadView, text); break;
		case 16: fDialog = new KSpreadDlgTTestTwoSampleUnequalVariances(kspreadView, text); break;
		case 17: fDialog = new KSpreadDlgZTestTwoSample(kspreadView, text); break;
		default:
			kdDebug() <<"KSpreadDlgDataAnalysisMain::accept(): error" << endl;
			return;
	}
	connect(fDialog, SIGNAL(rejected()), this, SLOT(slotFunctionCancelled()));
	connect(fDialog, SIGNAL(accepted()), this, SLOT(slotFunctionExecuted()));
	this->hide();
	fDialog->show();
}
	
void KSpreadDlgDataAnalysisMain::slotFunctionCancelled()
{
	this->show();
	kdDebug() <<"KSpreadDlgDataAnalysisMain::slotFunctionCancelled()" << endl; 
}

void KSpreadDlgDataAnalysisMain::slotFunctionExecuted()
{
	delayedDestruct();
	kdDebug() <<"KSpreadDlgDataAnalysisMain::slotFunctionExecuted()" << endl;
}
	
KSpreadDlgDataAnalysisMain::~KSpreadDlgDataAnalysisMain()
{
	kdDebug() <<"KSpreadDlgDataAnalysisMain::~KSpreadDlgDataAnalysisMain() " << endl;
}
	
#include "kspread_dlg_data_analysis_main.moc"
