#ifndef __kspread_dlg_data_analysis_template__
#define __kspread_dlg_data_analysis_template__

#include "../dataAnalysis_solver/kdialogeditbutton.h"
#include "kspread_functions_hlp.h"

class KSpread::View;

enum ChartType {ChartBar, ChartLine};

typedef struct
{
	QPoint startPoint;
	QSize size;
	ChartType chartType;
	QString chartName;
	QString xAxisName;
	QString yAxisName;
	QStringVector* xLabels;
	QStringVector* yLabels;
	FloatVectorVector* xValues;
}
ChartParameters;

class KSpreadDlgDataAnalysisTemplate : public KDialogEditButton
{
	Q_OBJECT
public:
	KSpreadDlgDataAnalysisTemplate(KSpread::View* parent, const char* name=0, bool modal=true, const char* caption=0, int buttonMask=Ok|Apply|Cancel);
protected:
	QPoint* selectionStartPoint;
	QPoint* selectionEndPoint;
	bool validateReferences();
	bool checkMinVectorSize(QString name, FloatVectorVector& vector, uint minSize);
	bool checkMinVectorSize(QString name, FloatVector& vector, uint minSize);
	bool checkVectorSizeEquals(QString name, FloatVectorVector& vectorVector);
	SheetOrientation detectOrientation(QString name, QRect rect);
	FloatVectorVector* getFloats(QString name, KSpread::Sheet* sheet, QRect rect,  SheetOrientation orientation = OrientationAll, ValuePolice valuePolice = IgnoreEmpty, IntVectorVector* missingVectorVector = NULL);
	QStringVector* getLabels(KSpread::Sheet* sheet, QRect& rect, SheetOrientation orientation, bool labels = false);
	KSpread::Sheet* createOutput(QWidgetMap& widgetMap, int width, int height,  int& startCol, int& startRow);
	void setCell(KSpread::Sheet* sheet, int col, int row, QString text, LineStyle topLine = NoLine, LineStyle bottomLine = NoLine, FontType font = FontNormal, CellAlign align = StandardAlign);
	void setCellStyle(KSpread::Sheet* sheet, int col, int row, LineStyle topLine, LineStyle bottomLine, FontType font = FontNormal, CellAlign align = StandardAlign);
	void setTitlesRow(KSpread::Sheet* sheet, int colStart, int colEnd,  int row);
	void setEndRow(KSpread::Sheet* sheet, int colStart, int colEnd,  int row);
	void insertChart(ChartParameters* chartParams);
private:
	KSpread::Sheet* selectionSheet;
	QPen changeCellLineStyle(QPen currentPen, LineStyle);
	void setCellAlign(KSpread::Cell* cell, CellAlign align);
	int getScreenWidth(KSpread::Sheet* sheet, int cell1, int  cell2);
	int getScreenHeight(KSpread::Sheet* sheet, int cell1, int  cell2);
protected slots:
	virtual void accept();
signals:
	void accepted();
};

#endif
