#include "kspread_functions_data_analysis.h"
#include "mathfunc.h"

#include <kdebug.h>
#include <math.h>

#define min(v1,v2) (v1 < v2 ? v1 : v2)
#define max(v1,v2) (v1 > v2 ? v1 : v2)
#define even(v) ((v % 2 == 0) ? true : false)
#define odd(v) ((v % 2 == 0) ? false : true)
#define IS_NAN(val) (QString::number(val) == "nan")

double convert_to_percentage_helper(double value)
{
	value *= 10000;
	value = round(value);
	value /= 100;
	return value;
}

double power(double value, int p)
{
	return pow(value, p);
/*	if (p == 0)
		return 1;
	double res = value;
	for (int j = 1; j < p; j++)
		res *= value;
	return res;*/
}

void dafunc_minimum(FloatVector& vector, EvaluationError& error, double& result, uint position)
{
	if (error != NoError) return;
	if (position > vector.size())
	{
		error = Number;
		return;
	}
	position--;
	FloatVector vectorOut;
	hlpfunc_sort(vector, vectorOut, SortAscending);
	result = vectorOut[position];
}

void dafunc_maximum(FloatVector& vector, EvaluationError& error, double& result, uint position)
{
	if (error != NoError) return;
	if (position > vector.size())
	{
		error = Number;
		return;
	}
	position--;
	FloatVector vectorOut;
	hlpfunc_sort(vector, vectorOut, SortDescending);
	result = vectorOut[position];
}

void dafunc_range(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double minimum;
	dafunc_minimum(vector, error, minimum);
	if (error != NoError) return;
	double maximum;
	dafunc_maximum(vector, error, maximum);
	if (error != NoError) return;
	result = maximum - minimum;
}

void dafunc_sum(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	result = 0;
	for (uint i = 0; i < vector.size(); i++)
		result += vector[i];
}

void dafunc_count(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	result = vector.size();
}

void dafunc_average(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count;
	dafunc_count(vector, error, count);
	if (error != NoError) return;
	if (count == 0)
	{
		error = DivisionByZero;
		return;
	}
	dafunc_sum(vector, error, result);
	if (error != NoError) return;
	result = result / count;
}

void dafunc_median(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	FloatVector vectorOut;
	hlpfunc_sort(vector, vectorOut, SortAscending);
	double count;
	dafunc_count(vectorOut, error, count);
	if (error != NoError) return;
	int n = (int) count;
	if (even(n))
		result = (vectorOut[(n / 2) - 1] + vectorOut [n / 2]) / 2;
	else
		result = vectorOut[((n + 1) / 2)  - 1];
}

void sum_of_variation_power_helper(FloatVector& vector, EvaluationError& error, double& result, int p)
{
	if (error != NoError) return;
	double average;
	dafunc_average(vector, error, average);
	if (error != NoError) return;
	result = 0;
	for (uint i = 0; i < vector.size(); i++)
	{
		double temp = vector[i] - average;
		result += power(temp, p);
	}
}

void sum_of_squares_helper(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	sum_of_variation_power_helper(vector, error, result, 2);
}

void dafunc_variance(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count;
	dafunc_count(vector, error, count);
	if (error != NoError) return;
	if (count == 1)
	{
		error = DivisionByZero;
		return;
	}
	double ss;
	sum_of_squares_helper(vector, error, ss);
	if (error != NoError) return;
	result = ss / (count - 1);
}

void dafunc_standard_deviation(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double variance;
	dafunc_variance(vector, error, variance);
	if (error != NoError) return;
	result = sqrt(variance);
}

void dafunc_standard_error(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double std_dev;
	dafunc_standard_deviation(vector, error, std_dev);
	if (error != NoError) return;
	double count;
	dafunc_count(vector, error, count);
	if (error != NoError) return;
	result = std_dev / sqrt(count);
}


void dafunc_skewness(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n;
	dafunc_count(vector, error, n);
	if (error != NoError) return;
	if (n  < 3)
	{
		error = DivisionByZero;
		return;
	}
	if (error != NoError) return;
	double average;
	dafunc_average(vector, error, average);
	if (error != NoError) return;
	double std_dev;
	dafunc_standard_deviation(vector, error, std_dev);
	if (error != NoError) return;
	if (std_dev == 0)
	{
		error = DivisionByZero;
		return;
	}
	double variation = 0;
	for (unsigned int i = 0; i < vector.size(); i++)
	{
		double temp = (vector[i] - average) / std_dev;
		variation += pow(temp, 3);
	}
	result = (n / ((n - 1)*(n - 2))) * variation;
}

void dafunc_kurtosis(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n;
	dafunc_count(vector, error, n);
	if (error != NoError) return;
	if (n < 4)
	{
		error = DivisionByZero;
		return;
	}
	if (error != NoError) return;
	double average;
	dafunc_average(vector, error, average);
	if (error != NoError) return;
	double std_dev;
	dafunc_standard_deviation(vector, error, std_dev);
	if (error != NoError) return;
	if (std_dev == 0)
	{
		error = DivisionByZero;
		return;
	}
	double variation = 0;
	for (unsigned int i = 0; i < vector.size(); i++)
	{
		double temp = (vector[i] - average) / std_dev;
		variation += pow(temp, 4);
	}
	result = (((n * (n + 1)) / ((n - 1)*(n - 2)*(n - 3))) * variation) -
			((3 * power(n - 1, 2)) / ((n - 2)*(n - 3)));
}

void dafunc_anova_ss_total(FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	sum_of_squares_helper(vectorAll, error, result);
}

void dafunc_anova_df_total(FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count;
	dafunc_count(vectorAll, error, count);
	if (error != NoError) return;
	result = count - 1;
}

void dafunc_anova_single_ss_error(FloatVectorVector& vectorVector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	result = 0;
	for (uint i = 0; i < vectorVector.size(); i++)
	{
		FloatVector vector = vectorVector[i];
		double ss;
		sum_of_squares_helper(vector, error, ss);
		if (error != NoError) return;
		result += ss;
	}
}

void dafunc_anova_two_ss_error(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ss_total;
	dafunc_anova_ss_total(vectorAll, error, ss_total);
	if (error != NoError) return;
	double ss_error1;
	dafunc_anova_single_ss_error(vectorVector1, error, ss_error1);
	if (error != NoError) return;
	double ss_error2;
	dafunc_anova_single_ss_error(vectorVector2, error, ss_error2);
	if (error != NoError) return;
	result = ss_error1 + ss_error2 - ss_total;
}

void ss_helper(FloatVectorVector& vectorVector, EvaluationError& error, double& result, int p)
{
	double ss = 0;
	for (uint i = 0; i < vectorVector.size(); i++)
	{
		double sum;
		dafunc_sum(vectorVector[i], error, sum);
		if (error != NoError) return;
		ss += power(sum, p);
	}
	result = ss;
}

void dafunc_anova_two_repl_ss_error(FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (rowsPerSample < 1)
	{
		error = Impl;
		return;
	}
	if (error != NoError) return;
	double ss_total;
	dafunc_anova_ss_total(vectorAll, error, ss_total);
	if (error != NoError) return;
	double ss12;
	ss_helper(cellVectorVector, error, ss12, 2);
	if (error != NoError) return;
	double sumAll;
	dafunc_sum(vectorAll, error, sumAll);
	if (error != NoError) return;
	double countAll;
	dafunc_count(vectorAll, error, countAll);
	if (error != NoError) return;
	if (countAll == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ss_total - (ss12 / rowsPerSample) + (power(sumAll, 2) / countAll);
}

void dafunc_anova_single_df_error(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	result = vectorAll.size() - vectorVector.size();
}

void dafunc_anova_two_df_error(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	result = (vectorVector1.size() -1) * (vectorVector2.size() - 1);
}

void dafunc_anova_two_repl_df_error(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	result = vectorVector1.size() * vectorVector2.size() * (rowsPerSample - 1);
}


void dafunc_anova_single_ms_error(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ss_error;
	dafunc_anova_single_ss_error(vectorVector, error, ss_error);
	if (error != NoError) return;
	double df_error;
	dafunc_anova_single_df_error(vectorVector, vectorAll, error, df_error);
	if (error != NoError) return;
	if (df_error == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ss_error / df_error;
}

void dafunc_anova_two_ms_error(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ss_error;
	dafunc_anova_two_ss_error(vectorVector1, vectorVector2, vectorAll, error, ss_error);
	if (error != NoError) return;
	double df_error;
	dafunc_anova_two_df_error(vectorVector1, vectorVector2, error, df_error);
	if (error != NoError) return;
	if (df_error == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ss_error / df_error;
}

void dafunc_anova_two_repl_ms_error(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ss_error;
	dafunc_anova_two_repl_ss_error(cellVectorVector,  vectorAll, rowsPerSample, error, ss_error);
	if (error != NoError) return;
	double df_error;
	dafunc_anova_two_repl_df_error(vectorVector1, vectorVector2, rowsPerSample, error, df_error);
	if (df_error == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ss_error / df_error;
}

void dafunc_anova_ss_effect(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ss_total;
	dafunc_anova_ss_total(vectorAll, error, ss_total);
	if (error != NoError) return;
	double ss_error;
	dafunc_anova_single_ss_error(vectorVector, error, ss_error);
	if (error != NoError) return;
	result = ss_total - ss_error;
/*	double averageAll;
	dafunc_average(vectorAll, error, averageAll);
	if (error != NoError) return;
	result = 0;
	double max_count = -1;
	for (unsigned int i = 0; i < vectorVector.size(); i++)
	{
		FloatVector vector = vectorVector[i];
		double count;
		dafunc_count(vector, error, count);
		if (error != NoError) return;
		if (count > max_count)
			max_count = count;
		double average;
		dafunc_average(vector, error, average);
		if (error != NoError) return;
		double temp = average - averageAll;
		result += temp * temp;
	}
	result *= max_count;*/
}

void dafunc_anova_df_effect(FloatVectorVector& vectorVector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	result = vectorVector.size() - 1;
}

void dafunc_anova_ms_effect(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ss_effect;
	dafunc_anova_ss_effect(vectorVector, vectorAll, error, ss_effect);
	if (error != NoError) return;
	double df_effect;
	dafunc_anova_df_effect(vectorVector, error, df_effect);
	if (error != NoError) return;
	if (df_effect == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ss_effect / df_effect;
}

void dafunc_anova_two_repl_ss_interaction(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (rowsPerSample < 1)
	{
		error = Impl;
		return;
	}
	uint n1 = vectorVector1.size();
	uint n2 = vectorVector2.size();
	if (n1 < 1 || n2 < 1)
	{
		error = DivisionByZero;
		return;
	}
	if (error != NoError) return;
	double ss12;
	ss_helper(cellVectorVector, error, ss12, 2);
	if (error != NoError) return;
	double ss1;
	ss_helper(vectorVector1, error, ss1, 2);
	if (error != NoError) return;
	double ss2;
	ss_helper(vectorVector2, error, ss2, 2);
	if (error != NoError) return;
	double sumAll;
	dafunc_sum(vectorAll, error, sumAll);
	if (error != NoError) return;
	double countAll;
	dafunc_count(vectorAll, error, countAll);
	if (error != NoError) return;
	if (countAll == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = (ss12 / rowsPerSample) - (ss1 / (n2 * rowsPerSample)) - (ss2 / (n1 * rowsPerSample)) + (power(sumAll, 2) / countAll);
}

void dafunc_anova_two_repl_df_interaction(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	dafunc_anova_two_df_error(vectorVector1, vectorVector2, error, result);
}

void dafunc_anova_two_repl_ms_interaction(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ss_interaction;
	dafunc_anova_two_repl_ss_interaction(vectorVector1, vectorVector2, cellVectorVector, vectorAll, rowsPerSample, error, ss_interaction);
	if (error != NoError) return;
	double df_interaction;
	dafunc_anova_two_repl_df_interaction(vectorVector1, vectorVector2, error, df_interaction);
	if (error != NoError) return;
	if (df_interaction == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ss_interaction / df_interaction;
}

void dafunc_anova_single_f_stat(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ms_effect;
	dafunc_anova_ms_effect(vectorVector, vectorAll, error, ms_effect);
	if (error != NoError) return;
	double ms_error;
	dafunc_anova_single_ms_error(vectorVector, vectorAll, error, ms_error);
	if (error != NoError) return;
	if (ms_error == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ms_effect / ms_error;
}

void dafunc_anova_two_f_stat(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ms_effect;
	dafunc_anova_ms_effect(vectorVector1, vectorAll, error, ms_effect);
	if (error != NoError) return;
	double ms_error;
	dafunc_anova_two_ms_error(vectorVector1, vectorVector2, vectorAll, error, ms_error);
	if (error != NoError) return;
	if (ms_error == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ms_effect / ms_error;
}

void dafunc_anova_two_repl_f_stat(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ms_effect;
	dafunc_anova_ms_effect(vectorVector1, vectorAll, error, ms_effect);
	if (error != NoError) return;
	double ms_error;
	dafunc_anova_two_repl_ms_error(vectorVector1, vectorVector2, cellVectorVector, vectorAll, rowsPerSample, error, ms_error);
	if (error != NoError) return;
	if (ms_error == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ms_effect / ms_error;
}

void dafunc_anova_two_repl_interaction_f_stat(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double ms_interaction;
	dafunc_anova_two_repl_ms_interaction(vectorVector1, vectorVector2, cellVectorVector, vectorAll, rowsPerSample, error, ms_interaction);
	if (error != NoError) return;
	double ms_error;
	dafunc_anova_two_repl_ms_error(vectorVector1, vectorVector2, cellVectorVector, vectorAll, rowsPerSample, error, ms_error);
	if (error != NoError) return;
	if (ms_error == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ms_interaction / ms_error;
}

void dafunc_f_stat_variances(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double variance1;
	dafunc_variance(vector1, error, variance1);
	if (error != NoError) return;
	double variance2;
	dafunc_variance(vector2, error, variance2);
	if (error != NoError) return;
	if (variance2 == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = variance1 / variance2;
}

void dafunc_f_df(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	dafunc_anova_df_total(vector, error, result);
}

void dafunc_f_test(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_f_df(vector1, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_f_df(vector2, error, df2);
	if (error != NoError) return;
	double f_stat_variances;
	dafunc_f_stat_variances(vector1, vector2, error, f_stat_variances);
	if (error != NoError) return;
	if (df1 < 1 || df2 < 1)
	{
		error = Number;
		return;
	}
	double res = pf(f_stat_variances, df1, df2, FALSE, FALSE);
	if (res > 0.5)
		res = pf(f_stat_variances, df1, df2, TRUE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = res;
}

void f_dist_helper(double f, double df1, double df2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if ( f < 0 || df1 < 1 || df2 < 1)
	{
		error = Number;
		return;
	}
	double res = pf (f, df1, df2, FALSE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = res;
}

void dafunc_anova_single_f_dist(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_anova_df_effect(vectorVector, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_single_df_error(vectorVector, vectorAll, error, df2);
	if (error != NoError) return;
	double f;
	dafunc_anova_single_f_stat(vectorVector, vectorAll, error, f);
	if (error != NoError) return;
	f_dist_helper(f, df1, df2, error, result);
}

void dafunc_anova_two_f_dist(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_anova_df_effect(vectorVector1, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_two_df_error(vectorVector1, vectorVector2, error, df2);
	if (error != NoError) return;
	double f;
	dafunc_anova_two_f_stat(vectorVector1, vectorVector2, vectorAll, error, f);
	if (error != NoError) return;
	f_dist_helper(f, df1, df2, error, result);
}

void dafunc_anova_two_repl_f_dist(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_anova_df_effect(vectorVector1, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_two_repl_df_error(vectorVector1, vectorVector2, rowsPerSample, error, df2);
	if (error != NoError) return;
	double f;
	dafunc_anova_two_repl_f_stat(vectorVector1, vectorVector2, cellVectorVector, vectorAll, rowsPerSample, error, f);
	if (error != NoError) return;
	f_dist_helper(f, df1, df2, error, result);
}

void dafunc_anova_two_repl_interaction_f_dist(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, FloatVectorVector& cellVectorVector, FloatVector& vectorAll, uint rowsPerSample, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_anova_two_repl_df_interaction(vectorVector1, vectorVector2, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_two_repl_df_error(vectorVector1, vectorVector2, rowsPerSample, error, df2);
	if (error != NoError) return;
	double f;
	dafunc_anova_two_repl_interaction_f_stat(vectorVector1, vectorVector2, cellVectorVector, vectorAll, rowsPerSample, error, f);
	if (error != NoError) return;
	f_dist_helper(f, df1, df2, error, result);
}

//void dafunc_f_dist(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
//{
//	if (error != NoError) return;
//	dafunc_anova_single_f_dist(vectorVector, vectorAll, error, result);
//}

void f_crit_helper(double p, double df1, double df2, EvaluationError& error, double& result, double f_stat = 0.5)
{
	if (f_stat > 1 || error != NoError)
		p = 1 - p;
	if (p < 0 || p > 1)
	{
		error = Impl;
		return;
	}	
 	if (df1 < 1 || df2 < 1)
	{
		error = Number;
		return;
	}
	double res = qf (p, df1, df2, FALSE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = res;
}
void dafunc_anova_single_f_crit(FloatVectorVector& vectorVector, FloatVector& vectorAll, double p, EvaluationError& error, double& result)
{

	if (error != NoError) return;
	double df1;
	dafunc_anova_df_effect(vectorVector, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_single_df_error(vectorVector, vectorAll, error, df2);
	if (error != NoError) return;
	f_crit_helper(p, df1, df2, error, result);
}

void dafunc_anova_two_f_crit(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, double p, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_anova_df_effect(vectorVector1, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_two_df_error(vectorVector1, vectorVector2, error, df2);
	if (error != NoError) return;
	f_crit_helper(p, df1, df2, error, result);
}

void dafunc_anova_two_repl_f_crit(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2, uint rowsPerSample, double p, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_anova_df_effect(vectorVector1, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_two_repl_df_error(vectorVector1, vectorVector2, rowsPerSample, error, df2);
	if (error != NoError) return;
	f_crit_helper(p, df1, df2, error, result);
}

void dafunc_anova_two_repl_interaction_f_crit(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2,  uint rowsPerSample, double p, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_anova_two_repl_df_interaction(vectorVector1, vectorVector2, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_anova_two_repl_df_error(vectorVector1, vectorVector2, rowsPerSample, error, df2);
	if (error != NoError) return;
	f_crit_helper(p, df1, df2, error, result);
}

void dafunc_f_crit_variances(FloatVector& vector1, FloatVector& vector2, double p, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_f_df(vector1, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_f_df(vector2, error, df2);
	if (error != NoError) return;
	double f_stat;
	dafunc_f_stat_variances(vector1, vector2, error, f_stat);
	f_crit_helper(p, df1, df2, error, result, f_stat);
}

void cov_numerator_helper(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	if (n1 != n2)
	{
		error = Impl;
		return;
	}
	double average1;
	dafunc_average(vector1, error, average1);
	if (error != NoError) return;
	double average2;
	dafunc_average(vector2, error, average2);
	if (error != NoError) return;
	result = 0;
	for (uint j = 0; j < n1; j++)
	{
		double temp1 = vector1[j] - average1;
		double temp2 = vector2[j] - average2;
		result += temp1 * temp2;
	}
}

void dafunc_covariance(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double cov_numerator;
	cov_numerator_helper(vector1, vector2, error, cov_numerator);
	if (error != NoError) return;
	double count;
	dafunc_count(vector1, error, count);
	if (error != NoError) return;
	if (count == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = cov_numerator / count;
}

void dafunc_correlation(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (hlpfunc_vectors_equals(vector1, vector2))
	{
		result = 1;
		return;
	}
	double cov_numerator;
	cov_numerator_helper(vector1, vector2, error, cov_numerator);
	if (error != NoError) return;
	double sos1;
	sum_of_squares_helper(vector1, error, sos1);
	if (error != NoError) return;
	if (sos1 == 0)
	{
		error = DivisionByZero;
		return;
	}
	double sos2;
	sum_of_squares_helper(vector2, error, sos2);
	if (error != NoError) return;
	if (sos2 == 0)
	{
		error = DivisionByZero;
		return;
	}
	result =cov_numerator / sqrt(sos1 * sos2);
}


void dafunc_t_df_paired_means(FloatVector& vector, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	dafunc_anova_df_total(vector, error, result);
}

void dafunc_t_df_equal_variances(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	result = n1 + n2 - 2;
}

void dafunc_t_df_unequal_variances(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	if (n1 < 2)
	{
		error = DivisionByZero;
		return;
	}
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	if (n2 < 2)
	{
		error = DivisionByZero;
		return;
	}
	double variance1;
	dafunc_variance(vector1, error, variance1);
	if (error != NoError) return;
	double variance2;
	dafunc_variance(vector2, error, variance2);
	if (error != NoError) return;
 	double denominator = (power(variance1 / n1, 2) / (n1 - 1)) + (power(variance2 / n2, 2) / (n2 - 1));
	if (denominator == 0)
	{
		error = DivisionByZero;
		return;
	}
	double numerator = power((variance1 / n1) + (variance2 / n2), 2);
	result = round(numerator / denominator);
}

void dafunc_t_stat_paired_means(FloatVector& vector1, FloatVector& vector2, double meanDifference, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (meanDifference < 0)
	{
		error = Impl;
		return;
	}
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	if (n1 != n2)
	{
		error = Impl;
		return;
	}
	if (n1 == 0)
	{
		error = DivisionByZero;
		return;
	}
	FloatVector differenceVector;
	for (uint i = 0; i < n1; i++)
		differenceVector[i] = vector1[i] - vector2[i];
	double mean;
	dafunc_average(differenceVector, error, mean);
	if (error != NoError) return;
	double std_dev;
	dafunc_standard_deviation(differenceVector, error, std_dev);
	if (error != NoError) return;
	if (std_dev == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = (mean - meanDifference)  / (std_dev  / sqrt(n1));
}

void dafunc_t_stat_equal_variances(FloatVector& vector1, FloatVector& vector2, double meanDifference, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (meanDifference < 0)
	{
		error = Impl;
		return;
	}
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	if (meanDifference < 0)
	{
		error = Impl;
		return;
	}
	if (n1 == 0 || n2 == 0)
	{
		error = DivisionByZero;
		return;
	}
	double df;
	dafunc_t_df_equal_variances(vector1, vector2, error, df);
	if (error != NoError) return;
	if (df < 1)
	{
		error = Number;
		return;
	}
	double average1;
	dafunc_average(vector1, error, average1);
	if (error != NoError) return;
	double average2;
	dafunc_average(vector2, error, average2);
	if (error != NoError) return;
	double variance1;
	dafunc_variance(vector1, error, variance1);
	if (error != NoError) return;
	double variance2;
	dafunc_variance(vector2, error, variance2);
	if (error != NoError) return;
	double denominator = sqrt((((n1 - 1) * variance1 + (n2 - 1) * variance2) / df) * (1 / n1 + 1 / n2));
	if (denominator == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ((average1 - average2) - meanDifference) / denominator;
}

void dafunc_t_stat_unequal_variances(FloatVector& vector1, FloatVector& vector2, double meanDifference, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (meanDifference < 0)
	{
		error = Impl;
		return;
	}
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	if (n1 == 0 || n2 == 0)
	{
		error = DivisionByZero;
		return;
	}
	double average1;
	dafunc_average(vector1, error, average1);
	if (error != NoError) return;
	double average2;
	dafunc_average(vector2, error, average2);
	if (error != NoError) return;
	double variance1;
	dafunc_variance(vector1, error, variance1);
	if (error != NoError) return;
	double variance2;
	dafunc_variance(vector2, error, variance2);
	if (error != NoError) return;
	double denominator = sqrt((variance1 / n1) + (variance2 / n2));
	if (denominator == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ((average1 - average2) - meanDifference) / denominator;
}


enum TTestType {PairedMeans, EqualVariances, UnequalVariances};

void t_test_helper(FloatVector& vector1, FloatVector& vector2, uint tails, double meanDifference, TTestType type, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (tails < 1 || tails > 2 || meanDifference < 0)
	{
		error = Impl;
		return;
	}
	double df;
	double t_stat;
	switch (type)
	{
		case PairedMeans:
			dafunc_t_df_paired_means(vector1, error, df);
			if (error != NoError) return;
			dafunc_t_stat_paired_means(vector1, vector2, meanDifference, error, t_stat);
		break;
		case EqualVariances:
			dafunc_t_df_equal_variances(vector1, vector2, error, df);
			if (error != NoError) return;
			dafunc_t_stat_equal_variances(vector1, vector2, meanDifference, error, t_stat);
		break;
		case UnequalVariances:
			dafunc_t_df_unequal_variances(vector1, vector2, error, df);
			if (error != NoError) return;
			dafunc_t_stat_unequal_variances(vector1, vector2, meanDifference, error, t_stat);
		break;
	}
	if (error != NoError) return;
	int dof = (int) df;
	double res = tails * pt(fabs(t_stat), dof, FALSE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = res;
}

void dafunc_t_test_paired_means(FloatVector& vector1, FloatVector& vector2, uint tails, double meanDifference, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	t_test_helper(vector1, vector2, tails, meanDifference, PairedMeans, error, result);
}

void dafunc_t_test_equal_variances(FloatVector& vector1, FloatVector& vector2, uint tails, double meanDifference, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	t_test_helper(vector1, vector2, tails, meanDifference, EqualVariances, error, result);
}

void dafunc_t_test_unequal_variances(FloatVector& vector1, FloatVector& vector2, uint tails, double meanDifference, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	t_test_helper(vector1, vector2, tails, meanDifference, UnequalVariances, error, result);
}


void t_crit_helper(FloatVector& vector1, FloatVector& vector2, double p, uint tails, TTestType type, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (p < 0 || p > 1 || tails < 1 || tails >2)
	{
		error = Impl;
		return;
	}
	double df;
	switch (type)
	{
		case PairedMeans:
			double df1;
			dafunc_t_df_paired_means(vector1, error, df1);
			if (error != NoError) return;
			double df2;
			dafunc_t_df_paired_means(vector2, error, df2);
			if (error != NoError) return;
			if (df1 != df2)
			{
				error = Impl;
				return;
			}
			df = df1;
		break;
		case EqualVariances:
			dafunc_t_df_equal_variances(vector1, vector2, error, df);
		break;
		case UnequalVariances:
			dafunc_t_df_unequal_variances(vector1, vector2, error, df);
		break;
	}
	if (error != NoError) return;
	int dof = (int) df;
	double p_temp =(tails == 1 ? 2 : 1) * p;
	if (p_temp > 1 || dof < 1)
	{
		error = Number;
		return;
	}
	double res = qt(p / tails, dof, FALSE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = res;
}

void dafunc_t_crit_paired_means(FloatVector& vector1, FloatVector& vector2, double p, uint tails, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	t_crit_helper(vector1, vector2, p, tails, PairedMeans, error, result);
}

void dafunc_t_crit_equal_variances(FloatVector& vector1, FloatVector& vector2, double p, uint tails, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	t_crit_helper(vector1, vector2, p, tails, EqualVariances, error, result);
}

void dafunc_t_crit_unequal_variances(FloatVector& vector1, FloatVector& vector2, double p, uint tails, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	t_crit_helper(vector1, vector2, p, tails, UnequalVariances, error, result);
}

void dafunc_confidence_level(FloatVector& vector, double p, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	p = 1 - p;
	if (p < 0 || p > 1)
	{
		error = Impl;
		return;
	}
	double count;
	dafunc_count(vector, error, count);
	if (error != NoError) return;
	double std_dev;
	dafunc_standard_deviation(vector, error, std_dev);
	if (error != NoError) return;
	double t_crit;
	dafunc_t_crit_paired_means(vector, vector, p, 2, error, t_crit);
	if (error != NoError) return;
	result = t_crit * (std_dev / sqrt(count));
}

void dafunc_pooled_variance(FloatVectorVector& vectorVector, FloatVector& vectorAll, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double countAll;
	dafunc_count(vectorAll, error, countAll);
	if (error != NoError) return;
	double denominator = countAll - vectorVector.size();
	if (denominator == 0)
	{
		error = DivisionByZero;
		return;
	}
	double numerator = 0;
	for (uint i = 0; i < vectorVector.size(); i++)
	{
		double count;
		dafunc_count(vectorVector[i], error, count);
		if (error != NoError) return;
		double variance;
		dafunc_variance(vectorVector[i], error, variance);
		if (error != NoError) return;
		numerator += (count - 1) * variance;
	}
	result = numerator / denominator;
}

void dafunc_z_stat(FloatVector& vector1, FloatVector& vector2, double knownVariance1, double knownVariance2, double meanDifference, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (meanDifference < 0 || knownVariance1 < 0 || knownVariance2 < 0)
	{
		error = Impl;
		return;
	}
	double average1;
	dafunc_average(vector1, error, average1);
	if (error != NoError) return;
	double average2;
	dafunc_average(vector2, error, average2);
	if (error != NoError) return;
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	if (n1 == 0 || n2 == 0)
	{
		error = DivisionByZero;
		return;
	}
	double nominator = sqrt((knownVariance1 / n1) + (knownVariance2 / n2));
	if (nominator == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = ((average1 - average2) - meanDifference) / nominator;
}

void z_test_helper(double z_stat, EvaluationError& error, double& result)
{
	double res = pnorm(z_stat, 0, 1, TRUE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = res;
}

void dafunc_z_test(FloatVector& vector1, FloatVector& vector2, double knownVariance1, double knownVariance2, double meanDifference, uint tails, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (tails < 1 || tails >2 || meanDifference < 0 || knownVariance1 < 0 || knownVariance2 < 0)
	{
		error = Impl;
		return;
	}
	double z_stat;
	dafunc_z_stat(vector1, vector2, knownVariance1, knownVariance2, meanDifference, error, z_stat);
	if (error != NoError) return;
	double res;
	z_test_helper(z_stat, error, res);
	if (error != NoError) return;
	result = tails * res;
}

void dafunc_z_crit(double p, uint tails, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (p < 0 || p > 1 || tails < 1 || tails >2)
	{
		error = Impl;
		return;
	}
	double p_temp =(tails == 1 ? 2 : 1) * p;
	if (p_temp > 1)
	{
		error = Number;
		return;
	}
	double res = qnorm(p / tails, 0, 1, TRUE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = fabs(res);
}

void dafunc_moving_average(FloatVector& vector, uint position, uint interval, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count;
	dafunc_count(vector, error, count);
	if (error != NoError) return;
	int startPosition = (position - interval) + 1;
	int endPosition = position + 1;
	if (interval < 2 || interval > count || endPosition > count)
	{
		error = Impl;
		return;
	}
	if (startPosition < 0)
	{
		error = NA;
		return;
	}
	FloatVector rangeVector;
	int j = 0;
 	for (int i = startPosition; i < endPosition; i++, j++)
		rangeVector[j] =  vector[i];
	double average;
	dafunc_average(rangeVector, error, average);
	if (error != NoError) return;
	result = average;
}

void dafunc_moving_average_standard_error(FloatVector& vector, uint position, uint interval, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count;
	dafunc_count(vector, error, count);
	if (error != NoError) return;
	int startPosition = (position - interval) + 1;
	int endPosition = position + 1;
	if (interval < 2 || 2 * interval > count + 1 || endPosition > count)
	{
		error = Impl;
		return;
	}
	if (startPosition < 0)
	{
		error = NA;
		return;
	}
	double ss_variation = 0;
 	for (int i = startPosition; i < endPosition; i++)
	{
		double average;
		dafunc_moving_average(vector, i, interval, error, average);
		if (error != NoError) return;
		ss_variation += power(vector[i] - average, 2);
	}
	result = sqrt(ss_variation / interval);
}

void dafunc_rank(FloatVector& sortedVector, uint k, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n;
	dafunc_count(sortedVector, error, n);
	if (error != NoError) return;
	if (k > n - 1)
	{
		error = Impl;
		return;
	}
	double value = sortedVector[k];
	for (uint i = 0; i < n; i++)
		if (sortedVector[i] == value)
		{
			result = i + 1;
			return;
		}
}

void dafunc_percent(FloatVector& sortedVector, uint k, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n;
	dafunc_count(sortedVector, error, n);
	if (error != NoError) return;
	if (k > n - 1)
	{
		error = Impl;
		return;
	}
	if (n < 2)
	{
		result = 100;
		return;
	}
	double value = sortedVector[k];
	double max_k = -1;
	for (uint i = 0; i < n; i++)
		if (sortedVector[i] < value)
		{
			max_k = i - 1;
			break;
		}
	if (max_k < 0)
		max_k = n - 1;
	double p = (n - max_k - 1) / (n - 1);
	result = convert_to_percentage_helper(p);
}

#define DDC_PI (3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117)

uint numberOfBitsNeeded(uint powerOfTwo)
{
    for (uint i=0; ; i++ )
    {
        if ( powerOfTwo & (1 << i) )
            return i;
    }

    return 0;
}

uint reverseBits(uint index, uint numBits)
{
uint rev;
    for (uint i=rev=0; i < numBits; i++ )
    {
        rev = (rev << 1) | (index & 1);
        index >>= 1;
    }

    return rev;
}

void dafunc_fast_fourier_transform(FloatVector& RealIn, bool inverseTransform, FloatVector& realOut, FloatVector& imagOut)
{
	uint numBits;
	uint numSamples = RealIn.size();
	
	double angle_numerator = -2.0 * DDC_PI; 
	double tr, ti;
	
	if ( inverseTransform )
		angle_numerator = -angle_numerator;
	
	numBits = numberOfBitsNeeded (numSamples);
	
	for (uint i=0; i < numSamples; i++ )
	{
		uint j = reverseBits (i, numBits);
		realOut[j] = RealIn[i];
		imagOut[j] = 0.0;
	}
	
	uint blockSize;
	uint blockEnd = 1;
	for ( blockSize = 2; blockSize <= numSamples; blockSize <<= 1 )
	{
		double delta_angle = angle_numerator / (double)blockSize;
		double sm2 = sin ( -2 * delta_angle );
		double sm1 = sin ( -delta_angle );
		double cm2 = cos ( -2 * delta_angle );
		double cm1 = cos ( -delta_angle );
		double w = 2 * cm1;
		double ar0, ar1, ar2, ai0, ai1, ai2;
	
		for (uint i=0; i < numSamples; i += blockSize )
		{
		ar2 = cm2;
		ar1 = cm1;
	
		ai2 = sm2;
		ai1 = sm1;
		for (uint j=i, n=0; n < blockEnd; j++, n++ )
		{
			ar0 = w*ar1 - ar2;
			ar2 = ar1;
			ar1 = ar0;
	
			ai0 = w*ai1 - ai2;
			ai2 = ai1;
			ai1 = ai0;
	
			uint k = j + blockEnd;
			tr = ar0*realOut[k] - ai0*imagOut[k];
			ti = ar0*imagOut[k] + ai0*realOut[k];
	
			realOut[k] = realOut[j] - tr;
			imagOut[k] = imagOut[j] - ti;
	
			realOut[j] += tr;
			imagOut[j] += ti;
		}
		}
	
		blockEnd = blockSize;
	}
		
	if ( inverseTransform )
	{
		double denom = (double)numSamples;
		for (uint i=0; i < numSamples; i++ )
		{
			realOut[i] /= denom;
			imagOut[i] /= denom;
		}
	}
}

void dafunc_exponential_smoothing(FloatVector& vector, uint position, double alpha, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (position < 1)
	{
		error = NA;
		return;
	}
	else
	if (position < 2)
	{
		result = vector[0];
		return;
	}
	double res = 0;
	alpha = 1 - alpha;
	for (uint k = 0; k < position - 1; k++)
		res += power(1 - alpha, k) * vector[position - k - 1];
	result = (alpha * res) + (power(1 - alpha, position - 1) * vector[0]);
}

void dafunc_exponential_smoothing_standard_error(FloatVector& vector, uint position, double alpha, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count;
	dafunc_count(vector, error, count);
	if (error != NoError) return;
	if (count < 4)
	{
		error = Impl;
		return;
	}
	int startPosition = position - 3;
	int endPosition = position;
	if (startPosition < 1)
	{
		error = NA;
		return;
	}
	double ss_variation = 0;
 	for (int i = startPosition; i < endPosition; i++)
	{
		double exponential_smoothing;
		dafunc_exponential_smoothing(vector, i, alpha, error, exponential_smoothing);
		if (error != NoError) return;
 		ss_variation += power(vector[i] - exponential_smoothing, 2);
	}
	result = sqrt(ss_variation / 3);
}

void frequency_helper(FloatVector& floatVector, FloatVector& binVector, uint position, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double binCount;
	dafunc_count(binVector, error, binCount);
	if (error != NoError) return;
	if (position > binCount)
	{
		error = Impl;
		return;
	}
	double floatCount;
	dafunc_count(floatVector, error, floatCount);
	if (error != NoError) return;
	double res = 0;
	if (position == 0)
	{
		for (uint i = 0; i < floatCount; i++)
			if (floatVector[i] <= binVector[0])
					res++;
	}
	else
	if (position == binCount)
	{
		for (uint i = 0; i < floatCount; i++)
			if (floatVector[i] > binVector[(int) binCount - 1])
					res++;
	}
	else
	{
		for (uint i = 0; i < floatCount; i++)
			if (floatVector[i] > binVector[position - 1] && floatVector[i] <= binVector[position])
					res++;
	}
	result = res;
}

void dafunc_frequency_vector(FloatVector& floatVector, FloatVector& binVector, EvaluationError& error, FloatVector& freqVector)
{
	if (error != NoError) return;
	double binCount;
	dafunc_count(binVector, error, binCount);
	if (error != NoError) return;
	for (uint i = 0; i <= binCount; i++)
	{
		double freq;
		frequency_helper(floatVector, binVector, i, error, freq);
		if (error != NoError) return;
		freqVector[i] = freq;
	}
}

void dafunc_frequency_cumulative_percentage(FloatVector& floatVector, FloatVector& freqVector, uint position, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double freqCount;
	dafunc_count(freqVector, error, freqCount);
	if (error != NoError) return;
	if (position > freqCount - 1)
	{
		error = Impl;
		return;
	}
	double floatCount;
	dafunc_count(floatVector, error, floatCount);
	if (error != NoError) return;
	double sum = 0;
	for (uint i = 0; i <= position; i++)
		sum += freqVector[i];
	double res = sum / floatCount;
	result = convert_to_percentage_helper(res);
}

/*
#define E 1.0e-14
#define MAX_ITER 10000

void seidel_helper(FloatVectorVector& A, FloatVector& b, EvaluationError& error, FloatVector& x)
{
	if (error != NoError) return;
	uint n = A.size();
	double count;
	dafunc_count(b, error, count);
	if (error != NoError) return;
	if (n != count)
	{
		error = Impl;
		return;
	}
	x.clear();
	for (uint i = 0; i < n; i++)
	{
		dafunc_count(A[i], error, count);
		if (error != NoError) return;
		if (count != n)
		{
			error = Impl;
			return;
		}
		if (A[i][i] == 0)
		{
			error = DivisionByZero;
			return;
		}
		x[i] = 0;
	}
	for (uint iter = 0; iter < MAX_ITER; iter++)
		for (uint i = 0; i < n; i++) 
		{
			x[i] = b[i] / A[i][i];
			for (uint j = 0; j < i; j++)
				x[i] -= (A[i][j] / A[i][i]) * x[j];
			for (uint j= i + 1; j < n; j++)
				x[i] -= (A[i][j] / A[i][i]) * x[j];
			//
			x[n] = iter;
		}
}


void jacobi_helper(FloatVectorVector& A, FloatVector& b, EvaluationError& error, FloatVector& x)
{
	if (error != NoError) return;
	uint n = A.size();
	double count;
	dafunc_count(b, error, count);
	if (error != NoError) return;
	if (n != count)
	{
		error = Impl;
		return;
	}
	double M[n][n];
	double x2[n];
	x.clear();
	for (uint i = 0; i < n; i++)
	{
		dafunc_count(A[i], error, count);
		if (error != NoError) return;
		if (count != n)
		{
			error = Impl;
			return;
		}
		if (A[i][i] == 0)
		{
			error = DivisionByZero;
			return;
		}
		for (uint j = 0; j < n; j++)
			if (i == j)
				M[i][j] = 0;
			else
				M[i][j] = - (A[i][j] / A[i][i]);
		x[i] = 0;
	}

	for (uint iter = 0; iter < MAX_ITER; iter++)
	{
		for (uint i = 0; i < n; i++) 
		{
			x2[i] = b[i] / A[i][i];
			for (uint j = 0; j < n; j++)
				x2[i] += M[i][j] * x[j];
		}
		for (uint i = 0; i < n; i++)
			x[i] = x2[i];
		//
		x[n] = iter;
	}
}

void relaxation_helper(FloatVectorVector& A, FloatVector& b, EvaluationError& error, FloatVector& x)
{
	if (error != NoError) return;
	uint n = A.size();
	double count;
	dafunc_count(b, error, count);
	if (error != NoError) return;
	if (n != count)
	{
		error = Impl;
		return;
	}
	double alpha[n][n];
	double beta[n];
	double r[n];
	x.clear();
	for (uint i = 0; i < n; i++)
	{
		dafunc_count(A[i], error, count);
		if (error != NoError) return;
		if (count != n)
		{
			error = Impl;
			return;
		}
		if (A[i][i] == 0)
		{
			error = DivisionByZero;
			return;
		}
		for (uint j = 0; j < n; j++)
			if (A[i][j] == A[i][i])
				alpha[i][i] = -1;
			else
				alpha[i][j] = A[i][j] / (- A[i][i]);
		beta[i] = b[i] / A[i][i];
		r[i] = beta[i];
		x[i] = 0;
	}
	for (uint iter = 0; iter < MAX_ITER; iter++)
	{
		double max_r = -1;
		uint p = 0;
		for (uint i = 0; i < n; i++)
			if (fabs(r[i]) > max_r)
			{
				max_r = fabs(r[i]);
				p = i;
			}

		if (max_r <= E)
			break;
		x[p] += r[p];
		for (uint i = 0; i < n; i++)
			if (i != p)
				r[i] += r[p] * alpha[i][p];
		r[p] = 0;
		//
		x[n] = iter;
	}
}
*/

#define debug_print_matrix(m, s, n)\
	kdDebug() << "MATRIX (start) n=" << n - s << endl;\
	for (uint deb_x = s; deb_x < n; deb_x++)\
	{\
		QString deb_xyStr = "";\
		for (uint deb_y = s; deb_y < n; deb_y++)\
			deb_xyStr += QString::number(m[deb_x][deb_y]) + " ";\
		kdDebug() << deb_xyStr << endl;\
	}\
	kdDebug() << "MATRIX (end) n=" << n - s << endl;

#define debug_print_vector(v, s, n)\
	kdDebug() << "VECTOR (start) n=" << n - s << endl;\
	for (uint deb_x = s; deb_x < n; deb_x++)\
		kdDebug() << v[deb_x] << endl;\
	kdDebug() << "VECTOR (end) n=" << n - s << endl;

#define reverse_values(v1, v2, temp)\
	temp = v1;\
	v1 = v2;\
	v2 = temp;

void gauss_helper(FloatVectorVector& A, FloatVector& b, EvaluationError& error, FloatVector& x)
{
	if (error != NoError) return;
	uint n = A.size();
	double count;
	dafunc_count(b, error, count);
	if (error != NoError) return;
	if (n != count)
	{
		error = Impl;
		return;
	}
	double mA[n][n];
	double mB[n];
	uint x_map[n];
	x.clear();
	for (uint i = 0; i < n; i++)
	{
		dafunc_count(A[i], error, count);
		if (error != NoError) return;
		if (count != n)
		{
			error = Impl;
			return;
		}
//		if (A[i][i] == 0)
//		{
//			error = DivisionByZero;
//			return;
//		}
		for (uint j = 0; j < n; j++)
			mA[i][j] = A[i][j];
		mB[i] = b[i];
		x[i] = 0;
		x_map[i] = i;
	}
	double temp = 0;
	uint temp_i = 0;
	for (uint k = 0; k < n - 1; k++)
	{
		double max = fabs(mA[k][k]);
		uint max_i = k;
		uint max_j = k;
		for (uint i = k; i < n; i++)
			for (uint j = k; j < n; j++)
				if (fabs(mA[i][j]) > max)
				{
					max = fabs(mA[i][j]);
					max_i = i;
					max_j = j;
				}
		for (uint i = k; i < n; i++)
		{
			reverse_values(mA[k][i], mA[max_i][i], temp);
		}
		reverse_values(mB[k], mB[max_i], temp);
		for (uint i = k; i < n; i++)
		{
			reverse_values(mA[i][k], mA[i][max_j], temp);
		}
		reverse_values(x_map[k], x_map[max_j], temp_i);
		for (uint i = k + 1; i < n; i++)
		{
			if (mA[k][k] == 0)
				temp = 0;
			else
				temp = mA[i][k] / mA[k][k];
			for (uint j = k; j < n; j++)
				mA[i][j] -= temp * mA[k][j];
			mB[i] -= temp * mB[k];
		}
	}
	for (int i = n - 1; i >= 0; i--)
	{
		double sum = 0;
		for (uint j = i + 1; j < n; j++)
			sum += mA[i][j] * x[j];
		if (mA[i][i] == 0)
			x[i] = 0;
		else
			x[i] = (mB[i] - sum) / mA[i][i];
	}
	for (uint i = 0; i < n; i++)
		if (i != x_map[i])
		{
			uint addr = x_map[i];
			reverse_values(x[i], x[addr], temp);
			reverse_values(x_map[i], x_map[addr], temp_i);
		}
//	debug_print_matrix(mA, 0, n);
//	debug_print_vector(x_map, 0, n);
}

void gauss_matrix_inversion_helper(FloatVectorVector& A, EvaluationError& error, FloatVectorVector& invA)
{
	if (error != NoError) return;
	uint n = A.size();
	invA.clear();
	for (uint i = 0; i < n; i++)
	{
		double count;
		dafunc_count(A[i], error, count);
		if (error != NoError) return;
		if (count != n)
		{
			error = Impl;
			return;
		}
//		if (A[i][i] == 0)
//		{
//			error = DivisionByZero;
//			return;
//		}
		FloatVector invVector;
		for (uint j = 0; j < n; j++)
			invVector[j] = A[i][j];
		invA.push_back(invVector);
	}
	uint irow = 0, icol = 0;
	uint ipiv[n], indxr[n], indxc[n];
	double temp, pivinv;
	for(uint i = 0; i < n; i++)
		ipiv[i] = 0;
	for(uint i = 0; i < n; i++)
	{
		double big = 0;
		for(uint j = 0; j < n; j++)
		{
			if(ipiv[j] != 1)
				for(uint k = 0; k < n; k++)
					if(ipiv[k] == 0)
					{
						if (fabs(invA[j][k]) >= big)
						{
							big = fabs(invA[j][k]);
							irow = j;
							icol = k;
						}
						else
							if (ipiv[k] > 1)
							{
								error = Impl;
								return;
							}
					}
		}
		ipiv[icol]++;
		if (irow != icol)
			for (uint l = 0; l < n; l++)
			{
				reverse_values(invA[irow][l], invA[icol][l], temp);
			}
		indxr[i] = irow;
		indxc[i] = icol;
//		if (invA[icol][icol] == 0)
//		{
//			error = DivisionByZero;
//			return;
//		}
		if (invA[icol][icol] != 0)
		{
			pivinv=1 / invA[icol][icol];
			invA[icol][icol] = 1;
		for (uint l = 0; l < n; l++)
			invA[icol][l] *= pivinv;
		for (uint ll = 0; ll < n; ll++)
		{
			if(ll != icol)
			{
				temp = invA[ll][icol];
				invA[ll][icol] = 0;
				for(uint l = 0; l < n; l++)
					invA[ll][l] -= invA[icol][l] * temp;
			}
		}
		}
	}
	for (uint j = 0; j < n; j++)
	{
		uint l = n - 1 -  j;
		if (indxr[l] != indxc[l])
		{
			for (uint k = 0; k < n; k++)
			{
				reverse_values(invA[k][indxr[l]], invA[k][indxc[l]], temp);
			}
		}
	}
//	debug_print_matrix(invA, 0, n);
}

void sum_of_product_helper(FloatVector& vector1, FloatVector& vector2, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double n1;
	dafunc_count(vector1, error, n1);
	if (error != NoError) return;
	double n2;
	dafunc_count(vector2, error, n2);
	if (error != NoError) return;
	if (n1 != n2)
	{
		error = Impl;
		return;
	}
	result = 0;
	for (uint j = 0; j < n1; j++)
		result += vector1[j] * vector2[j];
}

void cov_matrix_helper(FloatVectorVector& X, bool constantZero, EvaluationError& error, FloatVectorVector& covM)
{
	if (error != NoError) return;
	for (uint i = 0; i < X.size(); i++)
	{
		FloatVector A_i;
		for (uint j = 0; j < X.size(); j++)
		{
			if (constantZero)
				sum_of_product_helper(X[i], X[j], error, A_i[j]);
			else
				cov_numerator_helper(X[i], X[j], error, A_i[j]);
			if (error != NoError) return;
		}
		covM.push_back(A_i);
	}
	
}

void dafunc_regression_b_variables(FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, FloatVector& b)
{
	if (error != NoError) return;
	double y_count;
	dafunc_count(y, error, y_count);
	if (error != NoError) return;
	if (y_count < 2)
	{
		error = Impl;
		return;
	}
	FloatVectorVector vA;
	cov_matrix_helper(X, constantZero, error, vA);
	if (error != NoError) return;
	FloatVector vB;
	for (uint i = 0; i < X.size(); i++)
	{
		if (constantZero)
			sum_of_product_helper(X[i], y, error, vB[i]);
		else
			cov_numerator_helper(X[i], y, error, vB[i]);
		if (error != NoError) return;
	}
	gauss_helper(vA, vB, error, b);
	if (error != NoError) return;
	double b0 = 0;
	if (!constantZero)
	{
		double sum = 0;
		for (uint i = 0; i < X.size(); i++)
		{
			double average_x;
			dafunc_average(X[i], error, average_x);
			if (error != NoError) return;
			sum += b[i] * average_x;
		}	
		double average_y;
		dafunc_average(y, error, average_y);
		if (error != NoError) return;
		b0 = average_y - sum;
	}	
	for (uint i = b.size(); i >= 1; i--)
		b[i] = b[i - 1];
	b[0] = b0;
}

void dafunc_regression_v_variables(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, FloatVector& stdErrorVector)
{
	if (error != NoError) return;
	double n;
	dafunc_count(y, error, n);
	if (error != NoError) return;
	double mse;
	dafunc_regression_residuals_ms(b, X, y, constantZero, error, mse);
	if (error != NoError) return;
	FloatVectorVector vA;
	cov_matrix_helper(X, constantZero, error, vA);
	if (error != NoError) return;
	FloatVectorVector invA;
	gauss_matrix_inversion_helper(vA, error, invA);
	if (error != NoError) return;
	for (uint i = 0; i < invA.size(); i++)
		stdErrorVector[i + 1] = sqrt(mse * invA[i][i]);
	FloatVector vD;
	uint nX = X.size();
	for (uint i = 0; i < nX; i++)
	{
		FloatVector vSum;
		double sum;
		dafunc_sum(X[i], error, sum);
		if (error != NoError) return;
		vD[i] = sum;
	}
	double A0 = 0;
	for (uint i = 0; i < nX; i++)
	{
		double element_i = 0;
		for (uint j = 0; j < nX; j++)
			element_i += invA[j][i] * vD[j];
		A0 += element_i * vD[i];
	}
	A0 = (1 + (A0 / n)) / n;
	stdErrorVector[0] = sqrt(mse * A0);
}

void dafunc_regression_t_stat(FloatVector& b, FloatVector& stdErrorVector, uint position, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count_b;
	dafunc_count(b, error, count_b);
	if (error != NoError) return;
	double count_stdErr;
	dafunc_count(stdErrorVector, error, count_stdErr);
	if (error != NoError) return;
	if (position >= count_b || position >= count_stdErr)
	{
		error = Impl;
		return;
	}
	if (stdErrorVector[position] == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = b[position] / stdErrorVector[position];
}

void dafunc_regression_t_test(FloatVector& y, FloatVector& b, FloatVector& stdErrorVector, uint position, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double t_stat;
	dafunc_regression_t_stat(b, stdErrorVector, position, error, t_stat);
	if (error != NoError) return;
	double df;
	dafunc_regression_residuals_df(b, y, constantZero, error, df);
	if (error != NoError) return;
	double res = 2 * pt(fabs(t_stat), df, FALSE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = res;
}

void regression_t_crit_helper(FloatVector& y, FloatVector& b, FloatVector& stdErrorVector, double confidanceLevel, uint position, bool constantZero, bool lower, EvaluationError& error, double& result)
{
	double df;
	dafunc_regression_residuals_df(b, y, constantZero, error, df);
	if (error != NoError) return;
	double p = 1 - (confidanceLevel / 100);
	double res = qt(p / 2, df, FALSE, FALSE);
	if (IS_NAN(res))
	{
		error = Number;
		return;
	}
	result = b[position] + (lower ? -1 : 1) * res * stdErrorVector[position];
}

void dafunc_regression_t_crit_lower(FloatVector& y, FloatVector& b, FloatVector& stdErrorVector, double confidenceLevel, uint position, bool constantZero, EvaluationError& error, double& result)
{
	regression_t_crit_helper(y, b, stdErrorVector, confidenceLevel, position, constantZero, true, error, result);
}

void dafunc_regression_t_crit_upper(FloatVector& y, FloatVector& b, FloatVector& stdErrorVector, double confidenceLevel, uint position, bool constantZero, EvaluationError& error, double& result)
{
	regression_t_crit_helper(y, b, stdErrorVector, confidenceLevel, position, constantZero, false, error, result);
}

void dafunc_regression_total_df(FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double count;
	dafunc_count(y, error, count);
	if (error != NoError) return;
	result = constantZero ? count : count - 1;
}

void dafunc_regression_total_ss(FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (!constantZero)
		sum_of_squares_helper(y, error, result);
	else
		sum_of_product_helper(y, y, error, result);
}

void dafunc_regression_residuals_df(FloatVector& b, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double total_df;
	dafunc_regression_total_df(y, constantZero, error, total_df);
	if (error != NoError) return;
	double regression_df;
	dafunc_regression_regression_df(b, error, regression_df);
	if (error != NoError) return;
	result = total_df - regression_df;
}

void dafunc_regression_residuals_ss(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	if (!constantZero)
	{
		double total_ss;
		dafunc_regression_total_ss(y, constantZero, error, total_ss);
		if (error != NoError) return;
		double regression_ss;
		dafunc_regression_regression_ss(b, X, y, constantZero, error, regression_ss);
		if (error != NoError) return;
		result = total_ss - regression_ss;
	}
	else
	{
		double y_count;
		dafunc_count(y, error, y_count);
		if (error != NoError) return;
		double sum = 0;
		for (uint i = 0; i < y_count; i++)
		{
			double temp;
			dafunc_regression_residuals(b, X, y, i, error, temp);
			if (error != NoError) return;
			sum += power(temp, 2);
		}
		result = sum;
	}
}

void dafunc_regression_residuals_ms(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double residuals_ss;
	dafunc_regression_residuals_ss(b, X, y, constantZero, error, residuals_ss);
	if (error != NoError) return;
	double residuals_df;
	dafunc_regression_residuals_df(b, y, constantZero, error, residuals_df);
	if (error != NoError) return;
	if (residuals_df == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = residuals_ss / residuals_df;
}

void dafunc_regression_regression_df(FloatVector& b, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	dafunc_anova_df_total(b, error, result);
}

void dafunc_regression_regression_ss(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double y_count;
	dafunc_count(y, error, y_count);
	if (error != NoError) return;
	if (y_count < 2)
	{
		error = Impl;
		return;
	}
	double sum = 0;
	if (!constantZero)
		for (uint i = 0; i < X.size(); i++)
		{
			double temp;
			cov_numerator_helper(X[i], y, error, temp);
			if (error != NoError) return;
	 		sum += b[i + 1] * temp;
		}
	else
	{
		double total_ss;
		dafunc_regression_total_ss(y, constantZero, error, total_ss);
		if (error != NoError) return;
		double residuals_ss;
		dafunc_regression_residuals_ss(b, X, y, constantZero, error, residuals_ss);
		if (error != NoError) return;
		sum = total_ss - residuals_ss;
	}
	result = sum;

}

void dafunc_regression_regression_ms(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double regression_ss;
	dafunc_regression_regression_ss(b, X, y, constantZero, error, regression_ss);
	if (error != NoError) return;
	double regression_df;
	dafunc_regression_regression_df(b, error, regression_df);
	if (error != NoError) return;
	if (regression_df == 0)
	{
		error = DivisionByZero;
		return;
	} 
	result = regression_ss / regression_df;
}

void dafunc_regression_f_stat(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double regression_ms;
	dafunc_regression_regression_ms(b, X, y, constantZero, error, regression_ms);
	if (error != NoError) return;
	double residuals_ms;
	dafunc_regression_residuals_ms(b, X, y, constantZero, error, residuals_ms);
	if (error != NoError) return;
	if (residuals_ms == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = regression_ms / residuals_ms;
}

void dafunc_regression_f_dist(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double df1;
	dafunc_regression_regression_df(b, error, df1);
	if (error != NoError) return;
	double df2;
	dafunc_regression_residuals_df(b, y, constantZero, error, df2);
	if (error != NoError) return;
	df2 -= constantZero;
	if (df2 < 0)
	{
		error = Impl;
		return;
	}
	double f;
	dafunc_regression_f_stat(b, X, y, constantZero, error, f);
	if (error != NoError) return;
	f_dist_helper(f, df1, df2, error, result);
}

void dafunc_regression_predicted_y(FloatVector& b, FloatVectorVector& X, uint position, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double res = b[0];
	for (uint i = 0; i < X.size(); i++)
		res += X[i][position] * b[i + 1];
	result = res;
}

void dafunc_regression_residuals(FloatVector& b, FloatVectorVector& X, FloatVector& y, uint position, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double predicted_y;
	dafunc_regression_predicted_y(b, X, position, error, predicted_y);
	if (error != NoError) return;
	result = y[position] - predicted_y;
}

void dafunc_regression_standarized_residuals(FloatVector& b, FloatVectorVector& X, FloatVector& y, uint position, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double y_count;
	dafunc_count(y, error, y_count);
	if (error != NoError) return;
	FloatVector residualsVector;
	double residual = 0;
	for (uint i = 0; i < y_count; i++)
	{
		double temp_residual;
		dafunc_regression_residuals(b, X, y, i, error, temp_residual);
		if (error != NoError) return;
		residualsVector[i] = temp_residual;
		if (i == position)
			residual = residualsVector[i];
	}
	double std_dev;
	dafunc_standard_deviation(residualsVector, error, std_dev);
	if (error != NoError) return;
	if (std_dev == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = residual / std_dev;
}

void dafunc_regression_r_square(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double numerator;
	dafunc_regression_regression_ss(b, X, y, constantZero, error , numerator);
	if (error != NoError) return;
	double denominator;
	dafunc_regression_total_ss(y, constantZero, error, denominator);
	if (error != NoError) return;
	if (denominator == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = numerator / denominator;
}

void dafunc_regression_multiple_r(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double r_square;
	dafunc_regression_r_square(b, X, y, constantZero, error, r_square);
	if (error != NoError) return;
	result = sqrt(r_square);
}
void dafunc_regression_adjusted_r_square(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	double r_square;
	dafunc_regression_r_square(b, X, y, constantZero, error, r_square);
	if (error != NoError) return;
	double total_df;
	dafunc_regression_total_df(y, constantZero, error, total_df);
	if (error != NoError) return;
	double residuals_df;
	dafunc_regression_residuals_df(b, y, constantZero, error, residuals_df);
	if (error != NoError) return;
	if (residuals_df == 0)
	{
		error = DivisionByZero;
		return;
	}
	result = 1 - ((1 - r_square) * (total_df / residuals_df));
}

void dafunc_regression_standard_error(FloatVector& b, FloatVectorVector& X, FloatVector& y, bool constantZero, EvaluationError& error, double& result)
{
	if (error != NoError) return;
	double residuals_ms;
	dafunc_regression_residuals_ms(b, X, y, constantZero, error, residuals_ms);
	if (error != NoError) return;
	result = sqrt(residuals_ms);
}

void dafunc_random_numbers(FloatVector& vector, uint maxPos, uint numberOfElements, EvaluationError& error, FloatVector& result)
{
	if (error != NoError) return;
	random_init();
	for (uint i = 0; i < numberOfElements; i++)
	{
		uint pos = (uint) random_int(0, maxPos);
		if (vector.contains(pos))
			result[i] = vector[pos];
	}
}
