#include "kspread_functions_hlp.h"
#include <math.h>

double almostZero(double& value)
{
	if (fabs(value) < 1.0e-14)
		return 0;
	else
		return value;
}

QString hlpfunc_result_string(EvaluationError error, double result, uint precision)
{
	switch (error)
	{
		case DivisionByZero:
			return "#DIV/0!";
		break;
		case Number:
			return "#NUM!";
		break;
		case NA:
			return "#N/A";
		break;
		case Impl:
			return "!IMPLEMENTATION!";
		case NoError:
			return QString::number(almostZero(result), 'g', precision);
		break;
	}
	return "";
}

QString hlpfunc_result_string(double& result, uint precision)
{
	EvaluationError error = NoError;
	return hlpfunc_result_string(error, result, precision);
}

FloatVector* hlpfunc_merge_vectors_content(FloatVector& vector1, FloatVector& vector2)
{
	FloatVector* retVector = new FloatVector();
	FloatVector::iterator vectorIt;
	int j = 0;
	for (vectorIt = vector1.begin(); vectorIt != vector1.end(); vectorIt++, j++)
		(*retVector)[j] = *vectorIt;
	for (vectorIt = vector2.begin(); vectorIt != vector2.end(); vectorIt++, j++)
		(*retVector)[j] = *vectorIt;
	return retVector;
}

QStringVector* hlpfunc_merge_labels(QStringVector& vector1, QStringVector& vector2)
{
	QStringVector* retVector = new QStringVector();
	QStringVector::iterator vectorIt;
	for (vectorIt = vector1.begin(); vectorIt != vector1.end(); vectorIt++)
		retVector->push_back(*vectorIt);
	for (vectorIt = vector2.begin(); vectorIt != vector2.end(); vectorIt++)
		retVector->push_back(*vectorIt);
	return retVector;
}

FloatVectorVector* hlpfunc_merge_vectors_vectors(FloatVectorVector& vectorVector1, FloatVectorVector& vectorVector2)
{
	FloatVectorVector* retVectorVector = new FloatVectorVector();
	for (uint i = 0; i < vectorVector1.size(); i++)
		retVectorVector->push_back(vectorVector1[i]);
	for (uint i = 0; i < vectorVector2.size(); i++)
		retVectorVector->push_back(vectorVector2[i]);
	return retVectorVector;
}

uint hlpfunc_max_vector_size(FloatVectorVector& vectorVector)
{
	uint ret = 0;
	for (uint i = 0; i < vectorVector.size(); i++)
		if (vectorVector[i].size() > ret)
			ret = vectorVector[i].size();
	return ret;
}


uint* hlpfunc_find_key(FloatVector& vector, double value)
{
	FloatVector::iterator vectorIt;
	for (vectorIt = vector.begin(); vectorIt != vector.end(); vectorIt++)
		if (value == vectorIt.data())
			return new uint(vectorIt.key());
	return NULL;
}

void sort_reverse_helper(FloatVectorVector& vectorVector, uint i, uint j)
{
	double temp;
	for (uint k = 0; k < vectorVector.size(); k++)
	{
		temp = vectorVector[k][j];
		vectorVector[k][j] = vectorVector[k][i];
		vectorVector[k][i] = temp;
	}
}

void hlpfunc_sort(FloatVectorVector& vectorVectorIn, uint position, FloatVectorVector& vectorVectorOut, SortType sort)
{
	FloatVectorVector tempVectorVector;
	
	for (uint i = 0; i < vectorVectorIn.size(); i++)
	{
		FloatVector vector = vectorVectorIn[i];
		tempVectorVector.push_back(vector);
	}
	vectorVectorOut = tempVectorVector;
	uint n = vectorVectorOut[0].size();
	if (sort == SortAscending)
	{
		for (uint i = 0; i < n - 1; i++)
			for (uint j = i + 1; j < n; j++)
				if (vectorVectorOut[position][j] < vectorVectorOut[position][i])
					 sort_reverse_helper(vectorVectorOut, i, j);
	}
	else
	{
		for (uint i = 0; i < n - 1; i++)
			for (uint j = i + 1; j < n; j++)
				if (vectorVectorOut[position][j] > vectorVectorOut[position][i])
					 sort_reverse_helper(vectorVectorOut, i, j);
	}
}

void hlpfunc_sort(FloatVector& vectorIn, FloatVector& vectorOut, SortType sort)
{
	FloatVectorVector vectorVector;
	vectorVector.push_back(vectorIn);
	hlpfunc_sort(vectorVector, 0, vectorVector, sort);
	vectorOut = vectorVector[0];
}

bool hlpfunc_vectors_equals(FloatVector& vector1, FloatVector& vector2)
{
	if (vector1.size() != vector2.size())
		return false;
	FloatVector::iterator vector1it;
	FloatVector::iterator vector2it;
	for (vector1it = vector1.begin(), vector2it = vector2.begin(); vector1it != vector1.end(); vector1it++, vector2it++)
	{
		if (*vector1it != *vector2it)
			return false;
	}
	return true;
	
}

void remove_not_contained_helper(FloatVector& vector1, FloatVector& vector2)
{
	typedef QValueList<uint> QIntList;
	QIntList v1_keys = vector1.keys();
	QIntList::iterator v_keysIt;
	for (v_keysIt = v1_keys.begin(); v_keysIt != v1_keys.end(); v_keysIt++)
	{
		int key = *v_keysIt;
		if (!vector2.contains(key))
			vector1.remove(key);
	}
}

void vector_order_helper(FloatVector& vector)
{
	typedef QValueList<double> QDoubleList;
	QDoubleList::iterator v_valsIt;
	int count;
	QDoubleList v_values = vector.values();
	vector.clear();
	count = 0;
	for (v_valsIt = v_values.begin(); v_valsIt != v_values.end(); v_valsIt++, count++)
		vector[count] = *v_valsIt;
}

void hlpfunc_remove_not_paired_elements(FloatVector& vector1, FloatVector& vector2)
{
	remove_not_contained_helper(vector1, vector2);
	remove_not_contained_helper(vector2, vector1);
	vector_order_helper(vector1);
	vector_order_helper(vector2);
}

void hlpfunc_remove_empty_vectors(FloatVectorVector& floatVV, QStringVector& qStringV)
{
	if (floatVV.size() == 0 || qStringV.size() == 0)
		return;
	FloatVectorVector::iterator floatVVit = floatVV.begin();
	QStringVector::iterator qStringVit = qStringV.begin();
	do
	{
		if (floatVVit->size() == 0)
		{
			floatVV.erase(floatVVit);
			qStringV.erase(qStringVit);
		}
		else
		{
			floatVVit++;
			qStringVit++;
		}
	}
	while (floatVVit != floatVV.end());
}
