
#include "kspread_plugin_data_analysis.h"

#include "kspread_dlg_data_analysis_main.h"

#include "../../kspread_view.h"
#include "../../kspread_doc.h"
#include "../../selection.h"
#include "../../kspread_sheet.h"

#include <kaboutdata.h>
#include <kgenericfactory.h>
#include <klocale.h>
#include <kofficeversion.h>
#include <kmessagebox.h>
	
	// make the plugin available
typedef KGenericFactory<PluginDataAnalysis> DataAnalysisFactory;
K_EXPORT_COMPONENT_FACTORY( libkspread_data_analysis,  DataAnalysisFactory("kspreaddataanalysis"))
	
	
PluginDataAnalysis::PluginDataAnalysis(QObject *parent, const char *name, const QStringList& args) : Plugin(parent,name)
{
	this->m_kspreadView = NULL;
	if (parent)
	{
		if (parent->inherits("KSpread::View"))
		{
			this->m_kspreadView = (KSpread::View*)parent;
		}
		else
		{
			kdWarning() << "Parent does not inherit KSpread::View!!!" << endl;
		}
	}
	else
	{
		kdWarning() << "Plugin created without a parent!!!" << endl;
	}
	
	
	this->m_dialog = NULL;
	
	new KAction( i18n("Data Analysis..."), KShortcut::null(),
		this,SLOT( slotDlgDataAnalysisShow()), actionCollection(), "kspreaddataanalysis");
	
}
	
PluginDataAnalysis::~PluginDataAnalysis()
{
}
	
KAboutData* PluginDataAnalysis::createAboutData()
{
	KAboutData * aboutData = new KAboutData(
	"kspreaddataanalysis",  //app name
	I18N_NOOP("Data Analysis"),  //program name
	KOFFICE_VERSION_STRING,  //version
	I18N_NOOP("KSpread Data Analysis Plugin"),  //short description
	KAboutData::License_BSD,  //license type
	I18N_NOOP("(c) 2006, Tomasz Maliszewski"),  //copyright
	0,
	"http://www.koffice.org/kspread/");
	aboutData->addAuthor("Tomasz Maliszewski", 0, "tmaliszewski@wp.pl");
	
	return aboutData;
}
	
void PluginDataAnalysis::slotDlgDataAnalysisShow()
{
	kdDebug() << "PluginDataAnalysis::slotDlgDataAnalysisShow(): start" << endl;
	if (this->m_dialog == NULL)
	{
		this->m_dialog = new KSpreadDlgDataAnalysisMain(m_kspreadView, "Data Analysis");
		connect(m_dialog, SIGNAL(destroyed()), this, SLOT(slotDlgDataAnalysisDestroyed()));

		this->m_dialog->show();
	}
	else 
		kdDebug() << "PluginDataAnalysis::slotDlgDataAnalysisShow(): error" << endl;
}
	
void PluginDataAnalysis::slotDlgDataAnalysisDestroyed()
{
	m_dialog = NULL;
	kdDebug() << "PluginDataAnalysis::slotDlgDataAnalysisDestroyed()" << endl;
}


#include "kspread_plugin_data_analysis.moc"
