#include "kdialogeditbutton.h"
#include "qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../kspread_cell.h"
#include "../../selection.h"
#include "../../kspread_canvas.h"
#include "../../kspread_doc.h"

#include <qevent.h>
#include <qbutton.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qwidgetstack.h>
#include <kdebug.h>
#include <qobjectlist.h>
#include <kmessagebox.h>

KDialogEditButton::KDialogEditButton(KSpread::View* parent, const char* name, bool modal, const char* caption, int buttonMask) : KDialogBase(parent, name, modal, caption, buttonMask)
{
	kspreadView = parent;
	mainWidgetStack = new QWidgetStack(this);
	KDialogBase::setMainWidget(mainWidgetStack);
	tempEditButton = new QEditButton(mainWidgetStack, "");
	connect( kspreadView->choice(), SIGNAL(changed(const Region&)), this, SLOT( slotSelectionChanged()) );
	connect(this, SIGNAL(cancelClicked()), SLOT(reject()));
	wereDescendantsPrepared = false;
	mainWidgetFocus = tempEditButton;
}

void KDialogEditButton::removeSelection()
{
	kspreadView->canvasWidget()->endChoose();
/*
	typedef QValueList<KSpread::Sheet*> SheetList;
	SheetList list = selectionMap.keys();
	for (SheetList::iterator it = list.begin(); it != list.end(); it++)
	{
		KSpread::Sheet* sheet = (KSpread::Sheet*) *it;
		kspreadView->selectionInfo()->initialize(selectionMap[sheet]);
		kspreadView->selectionInfo()->initialize(QRect(1,1, 3, 3) , sheet);
		sheet->update();
		kspreadView->slotChangeSelection(selectionMap[sheet]);
		kspreadView->selectionInfo()->update();
		kspreadView->slotRefreshView();
	}*/
}

void  KDialogEditButton::accept()
{
	removeSelection();
	kdDebug() <<"KDialogEditButton::accept()" << endl;
	emit accepted();
	delayedDestruct();
}

void  KDialogEditButton::reject()
{
	removeSelection();
	kdDebug() <<"KDialogEditButton::reject()" << endl;
	emit rejected();
	delayedDestruct();
}

void KDialogEditButton::show()
{
	if (!wereDescendantsPrepared)
	{
		prepareDescendants();
		wereDescendantsPrepared = true;
	}
	KDialogBase::show();
}

void KDialogEditButton::exec()
{
	if (!wereDescendantsPrepared)
	{
		prepareDescendants();
		wereDescendantsPrepared = true;
	}
	KDialogBase::exec();
}

void KDialogEditButton::prepareDescendants()
{
	std::vector<const char*> focusVector;
	focusVector.push_back("QEditButton");
	focusVector.push_back("QButton");
	focusVector.push_back("QLineEdit");

	QObjectList* children = this->queryList("QWidget");
	QObjectListIt* it = new QObjectListIt(*children);
	QObject* obj;
	bool inherits = false;
	while ( (obj = it->current()) != NULL)
	{
		++(*it);
		for (unsigned int i = 0; i < focusVector.size(); i++)
			if (obj->inherits(focusVector[i]))
			{
				inherits = true;
				break;
			}
		if (inherits && !obj->parent()->inherits("QEditButton"))
		{
			obj->installEventFilter(this);
			if (obj->inherits("QEditButton"))
			{
				connect(obj, SIGNAL(clicked()), this, SLOT(slotChangeWindowMode()));
				if (obj != tempEditButton)
					validationVector.push_back((QEditButton*)obj);
			}
		}
	}
}


bool KDialogEditButton::eventFilter(QObject* watched, QEvent* e)
{
	if (e->type() == QEvent::FocusIn )
	{
		if (watched->inherits("QEditButton"))
		{
			if (watched != focusEditButton)
			{
				if (watched  != tempEditButton &&
							focusEditButton != tempEditButton)
					kspreadView->canvasWidget()->endChoose();
				focusEditButton = (QEditButton*)watched;
	 	      		kspreadView->canvasWidget()->startChoose();
			}
		} 
		else 
		{
			kspreadView->canvasWidget()->endChoose();
			kspreadView->canvasWidget()->startChoose();
			focusEditButton = NULL;
		}
	}
	return false;
}


void KDialogEditButton::setMainWidget(QWidget* widget)
{
    QWidget* oldFirstWidget = mainWidgetStack->visibleWidget();
    if (oldFirstWidget != widget)
    {
	if (widget != tempEditButton)
		mainWidgetFrame = widget;
	mainWidgetStack->addWidget(widget);
	mainWidgetStack->raiseWidget(widget);
	mainWidgetStack->removeWidget(oldFirstWidget);
    }
}

QWidget* KDialogEditButton::mainWidget()
{
	return mainWidgetStack->visibleWidget();
}

void KDialogEditButton::slotChangeEditStatus(bool on)
{
	QButton* triggerButton = (QButton*)sender();
	QWidget* widget = focusConnections[triggerButton];
	if (widget  !=  NULL)
		widget->setEnabled(on);
}

void KDialogEditButton::slotSetEditFocus()
{
	QButton* triggerButton = (QButton*)sender();
	QWidget* widget = focusConnections[triggerButton];
	if (widget  !=  NULL)
		widget->setFocus();
	else
		triggerButton->setFocus();
}

void KDialogEditButton::connectFocus(QButton* button, QWidget* widget)
{
	if (widget != NULL)
		focusConnections[button] = widget;
	connect(button, SIGNAL(toggled(bool)), this, SLOT(slotChangeEditStatus(bool)));
	connect(button, SIGNAL(clicked()), this, SLOT(slotSetEditFocus()));

}

void KDialogEditButton::slotChangeWindowMode()
{
	if (mainWidgetStack->visibleWidget() == mainWidgetFrame)
	{
		mainWidgetFocus = focusEditButton;
		tempEditButton->setEditText(mainWidgetFocus->editText());
		frameMaximumHeight = maximumHeight();
		showButton(Help, false);
		showButton(Default, false);
		showButton(Details, false);
		showButton(Ok, false);
		showButton(Apply, false);
		showButton(Try, false);
		showButton(Cancel, false);
		showButton(Close, false);
		showButton(User1, false);
		showButton(User2, false);
		showButton(User3, false);
		showButton(No, false);
		showButton(Yes, false);
		setMaximumHeight(0);
		setMainWidget(tempEditButton);
		tempEditButton->setFocus();
		adjustSize();
	}
	else
	{
		mainWidgetFocus->setEditText(tempEditButton->editText());
		setMaximumHeight(frameMaximumHeight);
		showButton(Help, true);
		showButton(Default, true);
		showButton(Details, true);
		showButton(Ok, true);
		showButton(Apply, true);
		showButton(Try, true);
		showButton(Cancel, true);
		showButton(Close, true);
		showButton(User1, true);
		showButton(User2, true);
		showButton(User3, true);
		showButton(No, true);
		showButton(Yes, true);
		setMainWidget(mainWidgetFrame);
		mainWidgetFocus->setFocus();
		adjustSize();
	}
}

void KDialogEditButton::slotSelectionChanged()
{
	if ( !focusEditButton )
		return;

	if (kspreadView->choice()->isValid())
	{
/*		if (!selectionMap.contains(kspreadView->choice()->sheet()))
		{
			selectionMap[kspreadView->choice()->sheet()] = *kspreadView->selectionInfo();
		}
		kspreadView->selectionInfo()->initialize(*kspreadView->choice());
		kspreadView->slotChangeSelection(*kspreadView->choice());*/
		QString area = kspreadView->choice()->name();
		focusEditButton->setEditText(area);
	}
}

 QWidgetMap KDialogEditButton::allWidgets()
{
 	QWidgetMap ret;
	QObjectList* children = this->queryList("QWidget");
	QObjectListIt* it = new QObjectListIt(*children);
	QWidget* obj;
	while ( (obj = (QWidget*)it->current()) != NULL)
	{
		++(*it);
		if (obj->isEnabled())
		{
			QString name(obj->name());
			if (obj->inherits("QLineEdit"))
			{
				QLineEdit* edit = (QLineEdit*)obj;
				ret[name] = edit->text();
				kdDebug() << "lineEdit: " << name << " => " << ret[name] << endl;
			}
			else
			if (obj->inherits("QButtonGroup"))
			{
				QButtonGroup* buttonGroup = (QButtonGroup*)obj;
				ret[name] = buttonGroup->selected()->name();
				kdDebug() << "buttonGroup: " << name << " => " << ret[name] << endl;
			}
			else
			if (obj->inherits("QCheckBox"))
			{
				QCheckBox* checkBox = (QCheckBox*)obj;
				ret[name] = checkBox->isChecked() ? "1" : "0";
				kdDebug() << "checkBox: " << name << " => " << ret[name] << endl;
			}
		}
	}
	return ret;
}

KSpread::Sheet* KDialogEditButton::getSheet(QString str)
{
	if (str.contains('!'))
	{
		QString sheetName = str.section('!', 0, 0);
		str = str.section('!', 1, 1);
		return kspreadView->canvasWidget()->findSheet(sheetName);
	}
	return kspreadView->canvasWidget()->activeSheet();
}

QString KDialogEditButton::getAddress(QString str)
{
	str = str.upper();
	if (str.contains('!'))
		return str.section('!', 1, 1);
	return str;
}
RefErrorCode KDialogEditButton::validateReference(QEditButton* editButton)
{
	if (!editButton->isEnabled())
		return RefNoError;
	QString str = editButton->editText();
	if (str == "")
	{
 		if (editButton->canBeEmpty())
			return RefNoError;
		return StringIsEmpty;
	}
	KSpread::Sheet* sheet = getSheet(str);
	if (!sheet)
		return WrongExpression;
	else
	{
		if (str.contains(':'))
		{
			KSpread::Range range(getAddress(str), kspreadView->doc()->map(), sheet);
			if (!range.isValid())
				return WrongExpression;
		}
		else
		{
			KSpread::Point point(getAddress(str), kspreadView->doc()->map(), sheet);
			if (!point.isValid())
				return WrongExpression;
		}
	}
	return RefNoError;
}

bool KDialogEditButton::validateDataPoints(QString name, QString value, uint max)
{
	QString errorStr = "%1 must be a positive integer greater than zero";
	errorStr += (max == 0) ? "." : " and cannot exceed the number of data points.";
	bool ok;
	uint number = value.toUInt(&ok); 
	if (!ok || number < 1 || (max > 0 && number > max))
	{
		KMessageBox::information(this, errorStr.arg(name));
		return false;
	}
	return true;
}

bool KDialogEditButton::validatePositiveNumber(QString name, QString value)
{
	bool ok;
	double number =value.toDouble(&ok);
	if (!ok || number < 0)
	{
		QString errorStr = "%1 must be a positive number.";
		KMessageBox::information(this, errorStr.arg(name));
		return false;
	}
	return true;
}


bool KDialogEditButton::validateNumber(QString name, QString value)
{
	bool ok;
	value.toDouble(&ok);
	if (!ok)
	{
		QString errorStr = "%1 must be a number.";
		KMessageBox::information(this, errorStr.arg(name));
		return false;
	}
	return true;
}

bool KDialogEditButton::validateNumber(QString name, QString value, double min, double max)
{
	bool ok;
	double number = value.toDouble(&ok);
	if (!ok || number < min || number > max)
	{
		QString errorStr = "%1 must be a number between %2 and %3.";
		KMessageBox::information(this, errorStr.arg(name).arg(QString::number(min)).arg(QString::number(max)));
		return false;
	}
	return true;
}


QRect KDialogEditButton::getRange(QString input)
{
	if (input.contains(':'))
	{
		KSpread::Range range(getAddress(input), kspreadView->doc()->map(), getSheet(input));
		int startCol, startRow, endCol, endRow;
		if (range.startCol() <= range.endCol())
		{
			startCol = range.startCol();
			endCol = range.endCol();
		}
		else
		{
			startCol = range.endCol();
			endCol = range.startCol();
		}
		if (range.startRow() <= range.endRow())
		{
			startRow = range.startRow();
			endRow = range.endRow();
		}
		else
		{
			endRow = range.startRow();
			startRow = range.endRow();
		}
		QPoint startPoint(startCol, startRow);
		QPoint endPoint(endCol, endRow);
		QRect rect(startPoint, endPoint);
		return rect;
	}
	else
	{
		KSpread::Point point(getAddress(input), kspreadView->doc()->map(), getSheet(input));
		QPoint startEndPoint(point.column(), point.row());
		QRect rect(startEndPoint, startEndPoint);
		return rect;
	}
}


#include "kdialogeditbutton.moc"
