#ifndef __kdialogeditbutton__
#define __kdialogeditbutton__

#include "kspread_functions_common.h"

#include <kdialogbase.h>
#include <qmap.h>
#include <vector>

class QButton;
class QEditButton;
class QWidgetStack;
class QWidget;

namespace KSpread
{
	class View;
	class Sheet;
	class Cell;
}

typedef QMap<QString, QString> QWidgetMap;
typedef std::vector<QEditButton*> QEditButtonVector;
enum SheetOrientation {OrientationColumn, OrientationRow, OrientationAll};
enum ValuePolice {IgnoreEmpty, EmptyShift, NoEmpty};
enum LineStyle {NoLine, ThinLine, FatLine};
enum FontType {FontNormal, FontItalic};
enum CellAlign {StandardAlign, CenterAlign};
enum RefErrorCode {RefNoError, StringIsEmpty, WrongExpression};

//typedef QMap<KSpread::Sheet*, KSpread::Region> SelectionMap;

class KDialogEditButton: public KDialogBase
{
	Q_OBJECT
public:
	KDialogEditButton(KSpread::View* parent, const char* name=0, bool modal=true, const char* caption=0, int buttonMask=Ok|Apply|Cancel);
	virtual void setMainWidget(QWidget* widget);
	virtual QWidget* mainWidget();
	virtual bool eventFilter(QObject* watched, QEvent* e);
	virtual void show();
	virtual void exec();
	void connectFocus(QButton* button, QWidget* widget);
protected:
	KSpread::View* kspreadView;
	QEditButtonVector validationVector;
	RefErrorCode validateReference(QEditButton* editButton);
	bool validateDataPoints(QString name, QString value, uint max = 0);
	bool validateNumber(QString name, QString value);
	bool validateNumber(QString name, QString value, double min, double max);
	bool validatePositiveNumber(QString name, QString value);
 	QWidgetMap allWidgets();
	QString getAddress(QString str);
	KSpread::Sheet* getSheet(QString str);
	QRect getRange(QString input);
	void removeSelection();
private:
	void prepareDescendants();
	int frameMaximumHeight;
	QEditButton* focusEditButton;
	QEditButton* mainWidgetFocus;
	QEditButton* tempEditButton;
	QWidget* mainWidgetFrame;
	QWidgetStack* mainWidgetStack;
	bool wereDescendantsPrepared;
	bool isSelection;
	QMap<QButton*, QWidget*> focusConnections;
private slots:
	void slotChangeEditStatus(bool on);
	void slotSetEditFocus();
	void slotChangeWindowMode();
	void slotSelectionChanged();
protected slots:
	virtual void accept();
	virtual void reject();
signals:
	void accepted();
	void rejected();
};

#endif
