#include "kspread_dlg_solv_add_change_condition.h"
#include "kspread_dlg_solver_main.h"
#include "kspread_dlg_solver_template.h"

#include "kspread_functions_solver.h"

#include "../../kspread_view.h"
#include "../../kspread_canvas.h"
#include "../../kspread_map.h"
#include "../../kspread_doc.h"
#include "../../kspread_sheet.h"
#include "../../kspread_cell.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include <qpushbutton.h>
#include <kdebug.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <kmessagebox.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>


KSpreadDlgSolverAddChangeCondition::KSpreadDlgSolverAddChangeCondition( KSpread::View* parent, const char* name, KSpreadDlgSolverMain* mainDialog, bool add) : KSpreadDlgSolverTemplate(parent, name, false, name, Ok | Cancel | User1)
{
	kdDebug() <<"KSpreadDlgSolverAddChangeCondition::KSpreadDlgSolverAddChangeCondition() " << endl;
	
	kspreadView = parent;
	this->mainDialog = mainDialog;
	this->add = add;

	setButtonText(User1, "Add");
	QWidget* mainGrid = new QWidget(this);
	setMainWidget(mainGrid);
	mainGrid->setMinimumWidth(350);
	QGridLayout* layout = new QGridLayout(mainGrid, 3, 2, marginHint(), spacingHint());

	layout->addWidget(new QLabel("Cell Reference: ", mainGrid), 0, 0);
	cellReferenceEB = new QEditButton(mainGrid, "Cell Reference");
	layout->addWidget(cellReferenceEB, 1, 0);

	comparatorBox = new QComboBox(mainGrid);
	layout->addWidget(comparatorBox, 1, 1);
	comparatorBox->insertItem("<=");
	comparatorBox->insertItem("=");
	comparatorBox->insertItem(">=");
	comparatorBox->insertItem("int");

	layout->addWidget(new QLabel("Constraint: ", mainGrid), 0, 2);
	constraintEB = new QEditButton(mainGrid, "Constraint");
	layout->addWidget(constraintEB, 1, 2);

	if (!add)
	{
		QString str = mainDialog->conditionTextList()->currentText();
		cellReferenceEB->setEditText(str.section(' ', 0, 0));
		QString constraintStr = str.section(' ', 2, 2);
		constraintEB->setEditText(constraintStr);
		QString comparatorStr;
		if (constraintStr == "integer")
			comparatorStr = "int";
		else
		if (constraintStr == "binary")
			comparatorStr = "bin";
		else
			comparatorStr = str.section(' ', 1, 1);
		comparatorBox->setCurrentText(comparatorStr);
	}
	connect(comparatorBox, SIGNAL(activated(const QString&)), this, SLOT(slotComparatorClicked(const QString&)));
	connect(this, SIGNAL(user1Clicked()), SLOT(slotAddClicked()));
}
	
bool KSpreadDlgSolverAddChangeCondition::checkComparator(QString shortStr, QString str)
{
	if (comparatorBox->currentText() == shortStr)
	{
		if (constraintEB->editText() != str)
		{
			QString errorStr = "The constraint for \"%1\" can be \"%2\" only!";
			KMessageBox::error(this, errorStr.arg(shortStr).arg(str), "Error");
			return false;
		}
	}
	return true;
}

bool KSpreadDlgSolverAddChangeCondition::addCondition()
{
	ConditionType conditionType;
	uint cellReferenceNum, constraintNum = 1;
	if (validateReference(cellReferenceEB) == RefNoError)
	{
		QRect range;
		range = getRange(cellReferenceEB->editText());
		cellReferenceNum = range.width() * range.height();
	}
	else
	{
		QString errorStr = "Cell Reference box is empty or contents are not valid.";
		KMessageBox::information(this, errorStr);
		return false;
	}
	QString comparatorStr = comparatorBox->currentText();
	bool isInt = (comparatorStr == "int");
	if (isInt)
	{
		if (!checkComparator("int", "integer"))
			return false;
		conditionType = Cond_ValueTypeInt;
		comparatorStr = "=";
		QString encodedAddress = mainDialog->getEncodedAddress(cellReferenceEB->editText());
		QStringVector singleCellsVector = solvfunc_get_single_cells(encodedAddress);
		QStringVector& changingCellsVector = *mainDialog->getChangingCells();
		for (uint i = 0; i < singleCellsVector.size(); i++)
		{
			bool found = false;
			for (uint j = 0; j < changingCellsVector.size(); j++)
				if (singleCellsVector[i] == changingCellsVector[j])
					found = true;
			if (!found)
			{
				QString errorStr = "Integer constraint Cell Reference must include only Adjustable Cells.";
				KMessageBox::information(this, errorStr);
				return false;
			}
		}
	}
	else
	{
		if (validateReference(constraintEB) == RefNoError)
		{
			QRect range = getRange(constraintEB->editText());
			constraintNum = range.width() * range.height();
			if (constraintNum > 1 && constraintNum != cellReferenceNum)
			{
				QString errorStr = "Unequal number of cells in Cell Refrence and Constraint.";
				KMessageBox::information(this, errorStr);
				return false;
			}
			conditionType = constraintNum == 1 ? Cond_SingleValue : Cond_ValueRange;
		}
		else
		{
			bool ok;
			mainDialog->getFormulaValue(ok, constraintEB->editText());
			if (!ok)
			{
				QString errorStr = "Constraint must be a number, simple reference or formula with a numeric value.";
				KMessageBox::information(this, errorStr);
				return false;
			}
			conditionType = Cond_FormulaValue;
		}
	}
	QString condition = cellReferenceEB->editText()  + " " + comparatorStr + " " + constraintEB->editText();
	QListBox* conditionList = mainDialog->conditionTextList();
	if (add)
	{
		conditionList->insertItem(condition);
		mainDialog->addConditionType(conditionType);
	}
	else
	{
		conditionList->changeItem(condition, conditionList->currentItem());
		mainDialog->changeConditionType(conditionType, conditionList->currentItem());
	}
	return true;
}

void KSpreadDlgSolverAddChangeCondition::slotComparatorClicked(const QString& comparatorStr)
{
	if (comparatorStr == "int")
		constraintEB->setEditText("integer");
}

void KSpreadDlgSolverAddChangeCondition::accept()
{
	if (!addCondition())
		return;
	KSpreadDlgSolverTemplate::accept();
}

void KSpreadDlgSolverAddChangeCondition::slotAddClicked()
{
	if (!addCondition())
		return;
	cellReferenceEB->setEditText("");
	comparatorBox->setCurrentItem(0);
	constraintEB->setEditText("");
	add = true;
}

KSpreadDlgSolverAddChangeCondition::~KSpreadDlgSolverAddChangeCondition()
{
	kdDebug() <<"KSpreadDlgSolverAddChangeCondition::~KSpreadDlgSolverAddChangeCondition() " << endl;
}
	
#include "kspread_dlg_solv_add_change_condition.moc"
