#include "kspread_dlg_solv_options.h"
#include "kspread_dlg_solver_main.h"
#include "kspread_dlg_solver_template.h"

#include "kspread_functions_solver.h"

#include "../../kspread_view.h"
#include "../../kspread_canvas.h"
#include "../../kspread_map.h"
#include "../../kspread_doc.h"
#include "../../kspread_sheet.h"
#include "../../kspread_cell.h"

#include <qpushbutton.h>
#include <kdebug.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <kmessagebox.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qvbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>


KSpreadDlgSolverOptions::KSpreadDlgSolverOptions( KSpread::View* parent, const char* name, KSpreadDlgSolverMain* mainDialog) : KSpreadDlgSolverTemplate(parent, name, false, name, Ok | Cancel)
{
	kdDebug() <<"KSpreadDlgSolverOptions::KSpreadDlgSolverOptions() " << endl;
	
	kspreadView = parent;
	this->mainDialog = mainDialog;

	QVBox* box = new QVBox(this);

	setMainWidget(box);
	box->setMinimumHeight(130);

	QGrid* grid = new QGrid(2, box);
		
	QCheckBox* lowerBoundBox = new QCheckBox("Objective function lower bound: ", grid, "lowerBoundBox");
	QLineEdit* lowerBoundEdit = new QLineEdit(grid, "lowerBoundEdit");
	QCheckBox* epsilonBox = new QCheckBox("Epsilon: ", grid, "epsilonBox");
	QLineEdit* epsilonEdit = new QLineEdit(grid, "epsilonEdit");
	QCheckBox* floorFirstBox = new QCheckBox("during branch-and-bound, take the ceiling branch first", box, "floorFirstBox");
	QCheckBox* automaticScalingBox = new QCheckBox("use automatic scaling", box, "automaticScalingBox");
	QCheckBox* antiDegenBox = new QCheckBox("use perturbations to reduce degeneracy", box, "antiDegenBox");

	OptionsRec* optionsRecord = mainDialog->getOptionsRecord();

	if (optionsRecord->obj_bound)
	{
		lowerBoundBox->toggle();
		lowerBoundEdit->setText(QString::number(*optionsRecord->obj_bound));
	}
	else
		lowerBoundEdit->setEnabled(false);

	if (optionsRecord->epsilon)
	{
		epsilonBox->toggle();
		epsilonEdit->setText(QString::number(*optionsRecord->epsilon));
	}
	else
		epsilonEdit->setEnabled(false);

	if (optionsRecord->auto_scale)
		automaticScalingBox->toggle();
	if (optionsRecord->anti_degen)
		antiDegenBox->toggle();
	if (optionsRecord->floor_first)
		floorFirstBox->toggle();

	connectFocus(lowerBoundBox, lowerBoundEdit);
	connectFocus(epsilonBox, epsilonEdit);
}

void KSpreadDlgSolverOptions::accept()
{
	QWidgetMap map = allWidgets();

	OptionsRec* optionsRecord = mainDialog->getOptionsRecord();
	
	QString lowerBound;
	if (map["lowerBoundBox"] == "1")
	{
 		lowerBound = map["lowerBoundEdit"];
		if (!validateNumber("Lower Bound", lowerBound))
			return;
		optionsRecord->obj_bound = new double(lowerBound.toDouble());
	}
	else
		optionsRecord->obj_bound = NULL;

	QString epsilon;
	if (map["epsilonBox"] == "1")
	{
		epsilon = map["epsilonEdit"];
		if (!validateNumber("Epsilon", epsilon, 0.0, 0.5))
			return;
		optionsRecord->epsilon = new double(epsilon.toDouble());
	}
	else
		optionsRecord->epsilon = NULL;

	optionsRecord->auto_scale = (map["automaticScalingBox"] == "1");
	optionsRecord->anti_degen = (map["antiDegenBox"] == "1");
	optionsRecord->floor_first = (map["floorFirstBox"] == "1");

	KSpreadDlgSolverTemplate::accept();
}

KSpreadDlgSolverOptions::~KSpreadDlgSolverOptions()
{
	kdDebug() <<"KSpreadDlgSolverOptions::~KSpreadDlgSolverOptions() " << endl;
}

#include "kspread_dlg_solv_options.moc"
