#include "kspread_dlg_solv_result.h"
#include <kdialogbase.h>

#include "kspread_functions_solver.h"
#include "lp_kit.h"

#include "../../kspread_view.h"
#include "../../kspread_canvas.h"
#include "../../kspread_map.h"
#include "../../kspread_doc.h"
#include "../../kspread_sheet.h"
#include "../../kspread_cell.h"

#include <qpushbutton.h>
#include <kdebug.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <kmessagebox.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qvbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>


KSpreadDlgSolverResult::KSpreadDlgSolverResult(KSpread::View* parent, const char* name, QStringVector* variablesVector, double* resultRow, int resultStatus) : KDialogBase(parent, name, false, name, Ok | Cancel)
{
	kdDebug() <<"KSpreadDlgSolverResult::KSpreadDlgSolverResult() " << endl;
	
	QVBox* box = new QVBox(this);
	setMainWidget(box);
	setMinimumSize(300, 180);
	kspreadView = parent;
	this->variablesVectorPtr = variablesVector;
	this->resultRow = resultRow;
	QString resultStr;
	switch (resultStatus)
	{
		case OPTIMAL:
			resultStr = "Solver found optimal solution.";
		break;
		case INFEASIBLE:
			resultStr = "This problem is infeasible.";
		break;
		case UNBOUNDED:
			resultStr = "This problem is unbounded.";
		break;
		case FAILURE:
			resultStr = "Solver failed.";
		break;
	}
	new QLabel(resultStr, box);
	QButtonGroup* buttonGroup = new QButtonGroup("", box);
	QVBoxLayout* layout = new QVBoxLayout(buttonGroup, marginHint(), spacingHint());
	keepButton = new QRadioButton("Keep Solver Solution", buttonGroup);
	layout->addWidget(keepButton);
	restoreButton = new QRadioButton("Restore Original Values", buttonGroup);
	layout->addWidget(restoreButton);

	buttonGroup->insert(keepButton);
	buttonGroup->insert(restoreButton);

	keepButton->toggle();

	connect(this, SIGNAL(cancelClicked()), SLOT(delayedDestruct()));
}

void KSpreadDlgSolverResult::accept()
{
	QStringVector& previousValues = *previousValuesPtr;
	QStringVector& variablesVector = *variablesVectorPtr;
	if (restoreButton->isOn())
		for (uint i = 0; i < variablesVector.size(); i++)
		{
			QString sheetName = variablesVector[i].section('!', 0, 0);
			KSpread::Sheet* sheet = kspreadView->canvasWidget()->findSheet(sheetName);
 			QPoint point = solvfunc_get_cell_point(variablesVector[i]);
			KSpread::Cell* cell = sheet->cellAt(point.x(), point.y());
			cell->setCellText(previousValues[i]);
		}
	delayedDestruct();
}

void KSpreadDlgSolverResult::show()
{
	QStringVector& variablesVector = *variablesVectorPtr;
	previousValuesPtr = new QStringVector();
	for (uint i = 0; i < variablesVector.size(); i++)
	{
		QString sheetName = variablesVector[i].section('!', 0, 0);
		KSpread::Sheet* sheet = kspreadView->canvasWidget()->findSheet(sheetName);
 		QPoint point = solvfunc_get_cell_point(variablesVector[i]);
		KSpread::Cell* cell = sheet->cellAt(point.x(), point.y());
		previousValuesPtr->push_back(cell->strOutText());
		cell->setCellText(QString::number(resultRow[i]));
	}
	KDialogBase::show();
}

KSpreadDlgSolverResult::~KSpreadDlgSolverResult()
{
	kdDebug() <<"KSpreadDlgSolverResult::~KSpreadDlgSolverResult() " << endl;
}
	
#include "kspread_dlg_solv_result.moc"
