#include "kspread_dlg_solver_main.h"

#include "kspread_dlg_solver_template.h"
#include "kspread_dlg_solv_add_change_condition.h"
#include "kspread_dlg_solv_options.h"
#include "kspread_dlg_solv_result.h"

#include "kspread_functions_solver.h"
#include "lp_kit.h"

#include "../../kspread_view.h"
#include "../../kspread_canvas.h"
#include "../../kspread_doc.h"
#include "../../kspread_sheet.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include <qpushbutton.h>
#include <kdebug.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <kmessagebox.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qevent.h>

KSpreadDlgSolverMain::KSpreadDlgSolverMain( KSpread::View* parent, const char* name) : KSpreadDlgSolverTemplate(parent, name, false, name, Stretch)
{
	kdDebug() <<"KSpreadDlgSolverMain::KSpreadDlgSolverMain() " << endl;
	
	kspreadView = parent;

	mainGrid = new QGrid(2, this);
	setMainWidget(mainGrid);
	mainGrid->setMinimumSize(480, 250);

	QButtonGroup* editGroup = new QButtonGroup(mainGrid, "targetEqualGroup");
	editGroup->setFrameShape(QFrame::NoFrame);
	QVBoxLayout* editLayout = new QVBoxLayout(editGroup, marginHint(), spacingHint());
	QGrid* grid1 = new QGrid(2, editGroup);
	editLayout->addWidget(grid1);
	new QLabel("Set Target Cell: ", grid1);
	targetCellEB = new QEditButton(grid1, "Target Cell");
	QGrid* grid2 = new QGrid(5, editGroup);
	editLayout->addWidget(grid2);
	new QLabel("Problem type: ", grid2);
	QRadioButton* maxButton = new QRadioButton("Max", grid2, "maxButton");
	QRadioButton* minButton = new QRadioButton("Min", grid2, "minButton");
	QGroupBox* changingCellsBox = new QGroupBox("By Changing Cells:", editGroup);
	editLayout->addWidget(changingCellsBox);
	QVBoxLayout* changingCellsLayout = new QVBoxLayout(changingCellsBox, marginHint(), spacingHint());
	QGrid* grid3 = new QGrid(2, changingCellsBox);
	changingCellsLayout->addWidget(grid3);
	grid3->setMargin(5);
	grid3->setSpacing(15);
	changingCellsEB = new QEditButton(grid3, "Changing Cells");
	QPushButton* guessButton = new QPushButton("Guess", grid3);

	QGroupBox* subjectBox = new QGroupBox("Subject to the Constraints:", editGroup);
	editLayout->addWidget(subjectBox);
	QVBoxLayout* subjectLayout = new QVBoxLayout(subjectBox, marginHint(), spacingHint());
	QGrid* grid4 = new QGrid(2, subjectBox);
	subjectLayout->addWidget(grid4);
	grid4->setMargin(5);
	grid4->setSpacing(15);
	subjectList = new QListBox(grid4);
	QGrid* grid4a = new QGrid(1, grid4);
	QPushButton* addButton = new QPushButton("Add", grid4a);
	QPushButton* changeButton = new QPushButton("Change", grid4a);
	QPushButton* deleteButton = new QPushButton("Delete", grid4a);

	QGrid* buttonGrid = new QGrid(1, mainGrid);
	QPushButton* solveButton = new QPushButton("Solve", buttonGrid);
	QPushButton* closeButton = new QPushButton("Close", buttonGrid);
	QPushButton* optionsButton = new QPushButton("Options", buttonGrid);
	QPushButton* resetAllButton = new QPushButton("Reset All", buttonGrid);

	editGroup->insert(minButton);
	editGroup->insert(maxButton);
	minButton->toggle();

	connect(solveButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(optionsButton, SIGNAL(clicked()), this, SLOT(slotOptionsClicked()));
	connect(closeButton, SIGNAL(clicked()), this, SLOT(reject()));
	connect(guessButton, SIGNAL(clicked()), this, SLOT(slotGuessClicked()));
	connect(addButton, SIGNAL(clicked()), this, SLOT(slotAddClicked()));
	connect(changeButton, SIGNAL(clicked()), this, SLOT(slotChangeClicked()));
	connect(deleteButton, SIGNAL(clicked()), this, SLOT(slotDeleteClicked()));
	connect(resetAllButton, SIGNAL(clicked()), this, SLOT(slotResetClicked()));

	tempSheet = new KSpread::Sheet(kspreadView->doc()->map(), "tempSheet");
	tempCell = new KSpread::Cell(tempSheet, 1, 1);
	optionsRecord = new OptionsRec();
	resetOptionsRecord();
}

double KSpreadDlgSolverMain::getFormulaValue(bool& ok, QString formula)
{
	if (formula.length() < 1)
	{
		ok = false;
		return 0;
	}
	if (formula.at(0) != '=')
		formula = "=" + formula;
	tempCell->setCellText(formula);
	formula = tempCell->encodeFormula(true);
	kdDebug() << formula << endl;
	QString sheetName = kspreadView->canvasWidget()->activeSheet()->sheetName();
	formula = solvfunc_add_sheetname(formula, sheetName);
	tempCell->setCellText(tempCell->decodeFormula(formula));
	if (!tempCell->calc(false))
	{
		ok = false;
		return 0;
	}
	if (tempCell->testFlag(tempCell->Flag_ParseError) || !tempCell->value().isNumber())
	{
		ok = false;
		return 0;
	}
	return tempCell->value().asFloat();
}

QString* KSpreadDlgSolverMain::getCellText(QString cellAddress)
{
	KSpread::Sheet* sheet = getSheet(cellAddress);
	QRect rect = getRange(cellAddress);
	KSpread::Cell* cell = sheet->cellAt(rect.x(), rect.y());
	if (cell->testFlag(cell->Flag_ParseError))
		return NULL;
	return new QString(cell->text());
}


QString KSpreadDlgSolverMain::getEncodedAddress(QString address)
{
	KSpread::Sheet* sheet = getSheet(address);
	tempCell->setCellText(address);
	QString encodedFormula = "=" + tempCell->encodeFormula(true);
	encodedFormula = solvfunc_add_sheetname(encodedFormula, sheet->sheetName());
	return encodedFormula.remove(0, 1);
}


double* KSpreadDlgSolverMain::getFactors(QString formula, QString sheetName, QStringVector variableVector)
{
	tempCell->setCellText(formula);
	QString encodedFormula = tempCell->encodeFormula(true);
	encodedFormula = solvfunc_add_sheetname(encodedFormula, sheetName);
	if (!solvfunc_check_formula_linear(encodedFormula, variableVector, getEncodedAddress(targetCellEB->editText())))
		return NULL;
	double* double_table = new double[variableVector.size() + 1];
	QString freeFactorStr;
	for (uint i = 0; i <= variableVector.size(); i++)
	{
		double_table[i] = 0;
		QString factorStr;
		if (i == 0)
		{
			factorStr = solvfunc_get_free_factor(encodedFormula, variableVector);
			freeFactorStr = factorStr;
		}
		else
			factorStr = solvfunc_get_factors(encodedFormula, variableVector, variableVector[i - 1], freeFactorStr);
		tempCell->setCellText(tempCell->decodeFormula(factorStr));
		if (!tempCell->calc(false))
			return NULL;
		double_table[i] = tempCell->value().asFloat();
	}
	return double_table;
}

short KSpreadDlgSolverMain::getConstraintType(QString str)
{
	if (str == ">" || str == ">=")
		return GE;
	else
	if (str == "<" || str == "<=")
		return LE;
	return EQ;
}

double* KSpreadDlgSolverMain::getConstraintFormula(QString rightCell, QString leftFormula, QStringVector variableVector, QString leftSheetName)
{
	double* left_double_table = NULL;
	double* right_double_table = NULL;
	QString* rightFormula = NULL;
	if (!(rightFormula = getCellText(rightCell)))
	{
		return NULL;
	}
	else
	{
		QString rightSheetName = getSheet(rightCell)->sheetName();
		 if (leftSheetName == "")
			leftSheetName = kspreadView->canvasWidget()->activeSheet()->sheetName();
		if (!(left_double_table = getFactors(*rightFormula, leftSheetName, variableVector)) ||
			!(right_double_table = getFactors(leftFormula, rightSheetName, variableVector)))
		{
			return NULL;
		}
	}
	for (uint i = 0; i <= variableVector.size(); i++)
		left_double_table[i] -= right_double_table[i];
	left_double_table[0] *= -1;
	return left_double_table;
}

double* KSpreadDlgSolverMain::getConstraintCell(QString rightCell, QString leftCell, QStringVector variableVector)
{
	QString* leftFormula;
	if (!(leftFormula = getCellText(leftCell)))
	{
		return NULL;
	}
	QString leftSheetName = getSheet(leftCell)->sheetName();
	return getConstraintFormula(rightCell, *leftFormula, variableVector, leftSheetName);
}

void KSpreadDlgSolverMain::accept()
{
	if (!validateTargetCell())
		return;
	QWidgetMap map = allWidgets();
	QStringVector* variableVectorPtr;
	if (!(variableVectorPtr = getChangingCells()))
		return;
	if (variableVectorPtr->size() == 0)
	{
		QString errorStr = "Problem to solve not specified.";
		informationError(errorStr);
		return;
	}
	QStringVector& variableVector = *variableVectorPtr;
	double* double_table = NULL;
	lprec* lp = make_lp(0, variableVector.size());
	QString sharedErrorStr = "%1 is invalid. Check if content of cells included in %2 is defined properly and arguments of functions within do not contain Changing Cells or Target Cell.";
	for (uint i = 0; i < subjectList->count(); i++)
	{
		QString conditionStr = subjectList->text(i);
		short constraintType = getConstraintType(conditionStr.section(' ', 1, 1));
		QString leftSide = conditionStr.section(' ', 0, 0);
		QString rightSide = conditionStr.section(' ', 2, 2);
		ConditionType condition = getConditionType(i);
		QString encodedAddress;
		QStringVector cellsVectorLeft;
		QStringVector cellsVectorRight;
		bool constraintError = false;
		switch (condition)
		{
			case Cond_FormulaValue:
			case Cond_SingleValue:
				encodedAddress = getEncodedAddress(leftSide);
				cellsVectorLeft = solvfunc_get_single_cells(encodedAddress);
				for (uint i = 0; i < cellsVectorLeft.size(); i++)
				{
					QString cellAddressLeft = tempCell->decodeFormula(cellsVectorLeft[i]);
					if (condition == Cond_SingleValue)
					{
						if (!(double_table = getConstraintCell(cellAddressLeft, rightSide, variableVector)))
						{
							constraintError = true;
							break;
						}
					}
					else
					{
						if (!(double_table = getConstraintFormula(cellAddressLeft, rightSide, variableVector)))
						{
							constraintError = true;
							break;
						}
					}
					double value = double_table[0];
					double_table[0] = 0;
					add_constraint(lp, double_table, constraintType, value);
				}
			break;
			case Cond_ValueRange:
				encodedAddress = getEncodedAddress(leftSide);
				cellsVectorLeft = solvfunc_get_single_cells(encodedAddress);
				encodedAddress = getEncodedAddress(rightSide);
				cellsVectorRight = solvfunc_get_single_cells(encodedAddress);
				for (uint i = 0; i < cellsVectorLeft.size(); i++)
				{
					QString cellAddressLeft = tempCell->decodeFormula(cellsVectorLeft[i]);
					QString cellAddressRight = tempCell->decodeFormula(cellsVectorRight[i]);
					if (!(double_table = getConstraintCell(cellAddressLeft, cellAddressRight, variableVector)))
					{
						constraintError = true;
						break;
					}
					double value = double_table[0];
					double_table[0] = 0;
					add_constraint(lp, double_table, constraintType, value);
				}
			break;
			case Cond_ValueTypeInt:
				encodedAddress = getEncodedAddress(leftSide);
				cellsVectorLeft = solvfunc_get_single_cells(encodedAddress);
				for (uint i = 0; i <cellsVectorLeft.size(); i++)
				{
					int pos = -1;
					for (uint j = 0; j < variableVector.size(); j++)
						if (cellsVectorLeft[i] == variableVector[j])
							pos = j + 1;
					if (pos == -1)
					{
						QString errorStr = "One of the integer constraint does not include only Adjustable Cells.";
						informationError(errorStr);
						return;
					}
					set_int(lp, pos, TRUE);
				}
			break;
		}
		if (constraintError)
		{
			QString errorStr = sharedErrorStr.arg("Constraint " + QString::number(i + 1)).arg("constraint");
			informationError(errorStr);
			return;
		}
	}
	QString cellAddressTarget = getEncodedAddress(targetCellEB->editText());
 	cellAddressTarget = tempCell->decodeFormula(cellAddressTarget);
	if (!(double_table = getConstraintFormula(cellAddressTarget, "=0", variableVector)))
	{
		QString errorStr = sharedErrorStr.arg("Target cell").arg("target cell");
		informationError(errorStr);
		return;
	}
	set_obj_fn(lp, double_table);

	if (optionsRecord->obj_bound)
		lp->obj_bound = *optionsRecord->obj_bound;
	if (optionsRecord->epsilon)
		lp->epsilon = *optionsRecord->epsilon;
	if (optionsRecord->auto_scale)
		auto_scale(lp);
	lp->anti_degen = optionsRecord->anti_degen;
	lp->floor_first = optionsRecord->floor_first;

	if (map["targetEqualGroup"] == "maxButton")
		set_maxim(lp);
	print_lp(lp);
	int resultStatus = solve(lp);

	print_solution(lp);

	double* resultRow = new double[lp->columns];
	for(int i = 1; i <= lp->columns; i++)
		resultRow[i - 1] = (double) lp->best_solution[lp->rows + i];

	KSpreadDlgSolverResult* resultDialog = new KSpreadDlgSolverResult(kspreadView, "Solver Results", &variableVector, resultRow, resultStatus);
	removeSelection();
	hide();
	resultDialog->show();
	emit accepted();
}

void KSpreadDlgSolverMain::reject()
{
	removeSelection();
	hide();
	emit rejected();
}

ConditionType KSpreadDlgSolverMain::getConditionType(uint position)
{
	return conditionVector[position];
}

void KSpreadDlgSolverMain::addConditionType(ConditionType type)
{
	conditionVector.push_back(type);
}

void KSpreadDlgSolverMain::changeConditionType(ConditionType type, uint position)
{
	conditionVector[position] = type;
}

void KSpreadDlgSolverMain::removeConditionType(uint position)
{
	ConditionTypeVector::iterator it = conditionVector.begin();
	for (uint i = 0; i < position; i++)
		it++;
	conditionVector.erase(it);
}

void KSpreadDlgSolverMain::addChangeCondition(bool add)
{
	QString titleStr = add ? "Add Constraint" : "Change Constraint";
	KSpreadDlgSolverAddChangeCondition* dialog = new KSpreadDlgSolverAddChangeCondition(kspreadView, titleStr.ascii(), this, add);

	connect(dialog, SIGNAL(rejected()), this, SLOT(slotWindowClosed()));
	connect(dialog, SIGNAL(accepted()), this, SLOT(slotWindowClosed()));

	subjectList->setFocus();
	this->hide();
	dialog->show();
}
bool KSpreadDlgSolverMain::checkConditionList(QString errorStr)
{
	if (subjectList->currentItem() == -1 || subjectList->count() == 0)
	{
		informationError(errorStr);
		return FALSE;
	}
	return TRUE;
}

bool KSpreadDlgSolverMain::validateTargetCell()
{
	RefErrorCode errorCode;
	QString errorStr = "";
	if ((errorCode = validateReference(targetCellEB)) != RefNoError)
		switch (errorCode)
		{
			case StringIsEmpty:
				errorStr = "Target Cell reference box is empty.";
			break;
			case WrongExpression:
				errorStr = "Target Cell reference box is invalid.";
			break;
			default: break;
		}
	else
	{
		QString targetCellAddress = targetCellEB->editText();
		KSpread::Sheet* sheet = getSheet(targetCellAddress);
		QRect rect = getRange(targetCellAddress);
		KSpread::Cell* cell = sheet->cellAt(rect.x(), rect.y());
		if (cell->testFlag(cell->Flag_ParseError))
			errorStr = "Target Cell is invalid.";
		else
		if (!cell->isFormula())
			errorStr = "Target Cell is not a formula.";
	}
	if (errorStr == "")
		return true;
	else
	{
		KMessageBox::information(this, errorStr);
		return false;
	}
}

void KSpreadDlgSolverMain::slotGuessClicked()
{
	if (!validateTargetCell())
		return;
	QString targetCellAddress = targetCellEB->editText();
	KSpread::Sheet* sheet = getSheet(targetCellAddress);
	QRect rect = getRange(targetCellAddress);
	KSpread::Cell* cell = sheet->cellAt(rect.x(), rect.y());
	tempCell->setCellText(cell->text());
	QString encodedFormula = tempCell->encodeFormula(true);
	encodedFormula = solvfunc_add_sheetname(encodedFormula, sheet->sheetName());
	QStringVector* variableVector = solvfunc_get_encoded_addresses(encodedFormula, true);
	QString defSheet = kspreadView->canvasWidget()->activeSheet()->sheetName();
	QString exp = solvfunc_get_changing_cells_expression(variableVector, defSheet);
	changingCellsEB->setEditText(tempCell->decodeFormula(exp));
}

QStringVector* KSpreadDlgSolverMain::getChangingCells()
{
	QString defSheet = kspreadView->canvasWidget()->activeSheet()->sheetName();
	tempCell->setCellText(changingCellsEB->editText());
	QString encodedFormula = tempCell->encodeFormula(true);
	encodedFormula = solvfunc_add_sheetname("=" + encodedFormula, defSheet);
	encodedFormula = encodedFormula.remove(0, 1);
	QStringVector* variableVectorPtr =  solvfunc_get_changing_cells_vector(encodedFormula);
	if (variableVectorPtr == NULL)
	{
		QString errorStr = "The formula in \"Changing Cells\" has an error.";
		informationError(errorStr);
		return NULL;
	}
	return variableVectorPtr;
}

void KSpreadDlgSolverMain::resetOptionsRecord()
{
	optionsRecord->obj_bound = NULL;
	optionsRecord->epsilon = NULL;
	optionsRecord->auto_scale = FALSE;
	optionsRecord->anti_degen = FALSE;
	optionsRecord->floor_first = FALSE;
}

OptionsRec* KSpreadDlgSolverMain::getOptionsRecord()
{
	return optionsRecord;
}

void KSpreadDlgSolverMain::slotAddClicked()
{
	if (!getChangingCells())
		return;
	addChangeCondition(true);
}

void KSpreadDlgSolverMain::slotChangeClicked()
{
	if (!getChangingCells())
		return;
	QString errorStr = "No constraint to change.";
	if (checkConditionList(errorStr))
	{
		kspreadView->canvasWidget()->endChoose();
		addChangeCondition(false);
	}
}

void KSpreadDlgSolverMain::slotDeleteClicked()
{
	if (!getChangingCells())
		return;
	QString errorStr = "No constraint to delete.";
	if (checkConditionList(errorStr))
		subjectList->removeItem(subjectList->currentItem());
}

void KSpreadDlgSolverMain::slotOptionsClicked()
{
	
	KSpreadDlgSolverOptions* dialog = new KSpreadDlgSolverOptions(kspreadView, "Solver Options", this);

	connect(dialog, SIGNAL(rejected()), this, SLOT(slotWindowClosed()));
	connect(dialog, SIGNAL(accepted()), this, SLOT(slotWindowClosed()));

	this->hide();
	dialog->show();
}

void KSpreadDlgSolverMain::slotResetClicked()
{
	QString errorStr = "Reset all Solver options and cell selections?";
	if (KMessageBox::Yes == KMessageBox::warningYesNo(this, errorStr, QString::null, KStdGuiItem::ok(), KStdGuiItem::cancel()))
	{
		subjectList->clear();
		conditionVector.clear();
		targetCellEB->setEditText("");
		changingCellsEB->setEditText("");
		resetOptionsRecord();
	}
}

QListBox* KSpreadDlgSolverMain::conditionTextList()
{
	return subjectList;
}

void KSpreadDlgSolverMain::slotWindowClosed()
{
	this->show();
	kdDebug() <<"KSpreadDlgSolverMain::slotWindowClosed()" << endl;
}

KSpreadDlgSolverMain::~KSpreadDlgSolverMain()
{
	delete tempCell;
	delete tempSheet;
	kdDebug() <<"KSpreadDlgSolverMain::~KSpreadDlgSolverMain() " << endl;
}
	
#include "kspread_dlg_solver_main.moc"
