#ifndef __kspread_dlg_solver_main__
#define __kspread_dlg_solver_main__

#include "kspread_dlg_solver_template.h"

enum ConditionType
{
	Cond_ValueTypeInt,
	Cond_SingleValue,
	Cond_ValueRange,
	Cond_FormulaValue
};

typedef std::vector<ConditionType> ConditionTypeVector;

typedef struct
{
	double* obj_bound;
	double* epsilon;
	bool auto_scale;
	bool anti_degen;
	bool floor_first;
}
OptionsRec;

class KSpread::View;
class QListBox;

class KSpreadDlgSolverMain: public KSpreadDlgSolverTemplate
{
	Q_OBJECT
public:
	KSpreadDlgSolverMain( KSpread::View* parent, const char* name);
	~KSpreadDlgSolverMain();
	QListBox* conditionTextList();
	double getFormulaValue(bool& ok, QString formula);
	short getConstraintType(QString str);
	void addConditionType(ConditionType type);
	void changeConditionType(ConditionType type, uint position);
	void removeConditionType(uint position);
	QString getEncodedAddress(QString address);
	QStringVector* getChangingCells();
	void resetOptionsRecord();
	OptionsRec* getOptionsRecord();
private slots:
	void accept();
	void reject();
	void slotOptionsClicked();
	void slotAddClicked();
	void slotChangeClicked();
	void slotDeleteClicked();
	void slotWindowClosed();
	void slotGuessClicked();
	void slotResetClicked();
private:
	QGrid* mainGrid;
	OptionsRec* optionsRecord;
	KSpread::Sheet* tempSheet;
	KSpread::Cell* tempCell;
	QEditButton* targetCellEB;
	QEditButton* changingCellsEB;
	ConditionTypeVector conditionVector;
	KSpread::View* kspreadView;
	QListBox* subjectList;
	bool checkConditionList(QString errorStr);
	void addChangeCondition(bool add);
	ConditionType getConditionType(uint position);
	bool validateTargetCell();
	QString* getCellText(QString cellAddress);
	double* getFactors(QString formula, QString sheetName, QStringVector variableVector);
	double* getConstraintFormula(QString rightCell, QString leftFormula, QStringVector variableVector, QString leftSheetName = "");
	double* getConstraintCell(QString rightCell, QString leftCell, QStringVector variableVector);
};

#endif
