#include "kspread_functions_solver.h"

#include <kdebug.h>
#include <math.h>
#include <qstringlist.h>
#include <qregexp.h>
#include <qpoint.h>

QString standarize_formula_helper(QString formula)
{
	 return formula.replace(QChar(0xA7), "$");
}

#define SHEET "[A-Za-z0-9]+\\!"
#define SINGLE_CELL_ADDRESS "\\$[0-9]+\\$[0-9]+\\#"
#define SINGLE_CELL_ADDRESS_WITH_SHEET SHEET SINGLE_CELL_ADDRESS
#define MULTI_CELL_ADDRESS SINGLE_CELL_ADDRESS"(\\:"SINGLE_CELL_ADDRESS")?"
#define MULTI_CELL_ADDRESS_WITH_SHEET SHEET MULTI_CELL_ADDRESS


QPoint solvfunc_get_cell_point(QString address)
{
	QRegExp pointExp("\\$[0-9]+");
	int pos = pointExp.search(address, 0);
	QString xStr = pointExp.cap(0).remove(0, 1);
	pos += pointExp.matchedLength();
	pos = pointExp.search(address, pos);
	QString yStr = pointExp.cap(0).remove(0, 1);
	bool ok;
	int x = xStr.toUInt(&ok);
	int y = yStr.toUInt(&ok);
	return QPoint(x, y);
}

#define MAX_CELL_REF 32767

bool check_cell_point_helper(QPoint point)
{
	return !(point.x() > MAX_CELL_REF || point.y() > MAX_CELL_REF);
}

QString solvfunc_get_changing_cells_expression(QStringVector* vectorPtr, QString defaultSheet)
{
	if (vectorPtr == NULL || vectorPtr->size() == 0)
		return "";
	QStringVector& vector = *vectorPtr;
	QString ret = "";
	for (uint i = 0; i < vector.size(); i++)
	{
		QString cell = vector[i];
		QString sheet = cell.section('!', 0, 0);
		if (sheet == defaultSheet)
			cell = cell.remove(0, sheet.length() + 1);
		ret += ";" + cell;
	}
	return ret.remove(0, 1);
}

bool string_exists_helper(QString str, QStringVector* vector)
{
	for (QStringVector::iterator it = vector->begin(); it != vector->end(); it++)
		if (*it == str)
			return true;
	return false;
}

QStringVector* solvfunc_get_changing_cells_vector(QString expression)
{
	if (expression != "")
	{
		expression = standarize_formula_helper(expression);
		expression = ";" + expression;
		QString regExp = "(\\;";
		regExp.append(MULTI_CELL_ADDRESS_WITH_SHEET);
		regExp.append(")+");
		QRegExp exp(regExp);
		int pos = exp.search(expression);
		if (pos > -1 && exp.matchedLength() == (int) expression.length())
		{
			QStringVector* variableVectorPtr = solvfunc_get_encoded_addresses(expression, true);
			if (variableVectorPtr == NULL)
				return NULL;
			QStringVector& variableVector = *variableVectorPtr;
			QStringVector* vector = new QStringVector();
			for (uint i = 0; i < variableVector.size(); i++)
				if (variableVector[i].contains(':'))
				{
					QStringVector cellsVector = solvfunc_get_single_cells(variableVector[i]);
					for (uint j = 0; j < cellsVector.size(); j++)
					{
						QPoint cellPoint = solvfunc_get_cell_point(cellsVector[j]);
						if (!check_cell_point_helper(cellPoint))
							return NULL;
						if (!string_exists_helper(cellsVector[j],  vector))
							vector->push_back(cellsVector[j]);
					}
				}
				else
				{
					QPoint cellPoint = solvfunc_get_cell_point(variableVector[i]);
					if (!check_cell_point_helper(cellPoint))
						return NULL;
					if (!string_exists_helper(variableVector[i],  vector))
						vector->push_back(variableVector[i]);
				}
			return vector;
		}
		else
			return NULL;
	}
	return new QStringVector();
}

QStringVector* solvfunc_get_encoded_addresses(QString encodedFormula, bool withSheetNames)
{
	encodedFormula = standarize_formula_helper(encodedFormula);
	QStringVector* vector = new QStringVector();
	QString pattern = "";
	if (withSheetNames)
		pattern.append(MULTI_CELL_ADDRESS_WITH_SHEET);
	else
		pattern.append(MULTI_CELL_ADDRESS);
	QRegExp rx(pattern);
	int pos = 0;
	while (pos >= 0)
	{
		pos = rx.search(encodedFormula, pos);
		if (pos > -1)
		{
			QString addr = rx.cap(0);
			if (!string_exists_helper(addr, vector))
				vector->push_back(addr);
			pos += rx.matchedLength();
		}
	}
	return vector;
}

bool solvfunc_check_formula_linear(QString encodedFormula, QStringVector& variableVector, QString targetCellAddress)
{
	encodedFormula = standarize_formula_helper(encodedFormula);
	QString functionExp("[A-Za-z0-9\\_]+\\(.*\\)");
	QRegExp rx(functionExp);
	int pos = 0;
	while (pos >= 0)
	{
		pos = rx.search(encodedFormula, pos);
		if (pos > -1)
		{
			QString function = "=" + rx.cap(0);
			QStringVector& addrVector = *solvfunc_get_encoded_addresses(function, true);
			for (uint i = 0; i < addrVector.size(); i++)
			{
				QStringVector singleCellsVector = solvfunc_get_single_cells(addrVector[i]);
				for (uint j = 0; j < singleCellsVector.size(); j++)
				{
					for (uint k = 0; k < variableVector.size(); k++)
						if (singleCellsVector[j] == variableVector[k])
							return false;
					if (singleCellsVector[j] == targetCellAddress)
						return false;
				}
			}
			pos += rx.matchedLength();
		}
	}
	return true;
}

QString solvfunc_get_factors(QString encodedFormula, QStringVector& variableVector, QString variable, QString freeFactorStr, QString sheetName)
{
	encodedFormula = standarize_formula_helper(encodedFormula);
	if (sheetName != "")
		encodedFormula = solvfunc_add_sheetname(encodedFormula, sheetName);
	for (uint i = 0; i < variableVector.size(); i++)
	{
		QString replaceStr = "0";
		if (variableVector[i] == variable)
			replaceStr = "1";
		encodedFormula = encodedFormula.replace(variableVector[i], replaceStr);
	}
	if (encodedFormula.at(0) == '=')
		encodedFormula = encodedFormula.remove(0, 1);
	if (freeFactorStr.at(0) == '=')
		freeFactorStr = freeFactorStr.remove(0, 1);
	encodedFormula = "=(" + encodedFormula + ")-(" + freeFactorStr + ")";
 	return encodedFormula;
}

QString solvfunc_get_free_factor(QString encodedFormula, QStringVector& variableVector, QString sheetName)
{
	return solvfunc_get_factors(encodedFormula, variableVector, "", "0", sheetName);
}


QString solvfunc_add_sheetname(QString encodedFormula, QString sheetName)
{
	encodedFormula = standarize_formula_helper(encodedFormula);
	QStringVector& vector = *solvfunc_get_encoded_addresses(encodedFormula);
	for (uint i = 0; i < vector.size(); i++)
	{
		QString newStr = "";
		QStringList strList = QStringList::split(vector[i], encodedFormula);
		uint listSize = strList.size();
		QString addrStr = vector[i];
		uint i = 1;
		QString tail = encodedFormula.right(addrStr.length());
		bool omitLastPart = (tail != addrStr);
		for (QStringList::Iterator it = strList.begin(); it != strList.end(); it++, i++)
		{
			QString strPart = *it;
			if (i == listSize && omitLastPart)
			{
				newStr += strPart;
				break;
			}
			if (strPart.at(strPart.length() - 1) != '!')
				newStr += strPart + sheetName + "!" + addrStr;
			else
				newStr += strPart +  addrStr;
		}
		encodedFormula = newStr;
	}
	return encodedFormula;
}

QStringVector solvfunc_get_single_cells(QString address)
{
	address = standarize_formula_helper(address);
	QStringVector vector;
	if (!address.contains(':'))
		vector.push_back(address);
	else
	{
		int pos = 0;
		QRegExp sheetExp(SHEET);
		QString sheetName = "";
		pos = sheetExp.search(address, pos);
		if (pos > -1)
			sheetName = sheetExp.cap(0);
		QPoint point1 = solvfunc_get_cell_point(address.section(':', 0, 0));
		QPoint point2 = solvfunc_get_cell_point(address.section(':', 1, 1));
		int start1 = point1.x();
		uint start2 = point1.y();
		uint end1 = point2.x();
		uint end2 = point2.y();
		for (uint i = start1; i <= end1; i++)
			for (uint j = start2; j <= end2; j++)
			{
				QString singleCell = "$" + QString::number(i) + "$" + QString::number(j) + "#";
				singleCell = sheetName + singleCell;
				vector.push_back(singleCell);
			}
	}
	return vector;
}

