
#include "kspread_plugin_solver.h"

#include "kspread_dlg_solver_main.h"

#include "../../kspread_view.h"

#include <kaboutdata.h>
#include <kgenericfactory.h>
#include <kofficeversion.h>

	
// make the plugin available
typedef KGenericFactory<PluginSolver> SolverFactory;
K_EXPORT_COMPONENT_FACTORY( libkspread_solver,  SolverFactory("kspreadsolver"))
	
	
PluginSolver::PluginSolver(QObject *parent, const char *name, const QStringList& args) : Plugin(parent,name)
{
	this->kspreadView = NULL;
	if (parent)
	{
		if (parent->inherits("KSpread::View"))
		{
			this->kspreadView = (KSpread::View*)parent;
		}
		else
		{
			kdWarning() << "Parent does not inherit KSpread::View!!!" << endl;
		}
	}
	else
	{
		kdWarning() << "Plugin created without a parent!!!" << endl;
	}
	
	
	this->m_dialog = NULL;
	dialogIsVisible = false;

	new KAction( i18n("Solver..."), KShortcut::null(),
		this,SLOT( slotDlgSolverShow()), actionCollection(), "kspreadsolver");
	
}
	
PluginSolver::~PluginSolver()
{
}
	
KAboutData* PluginSolver::createAboutData()
{
	KAboutData * aboutData = new KAboutData(
	"kspreadsolver",  //app name
	I18N_NOOP("Solver"),  //program name
	KOFFICE_VERSION_STRING,  //version
	I18N_NOOP("KSpread Solver Plugin"),  //short description
	KAboutData::License_BSD,  //license type
	I18N_NOOP("(c) 2006, Tomasz Maliszewski"),  //copyright
	0,
	"http://www.koffice.org/kspread/");
	aboutData->addAuthor("Tomasz Maliszewski", 0, "tmaliszewski@wp.pl");
	
	return aboutData;
}
	
void PluginSolver::slotDlgSolverShow()
{
	kdDebug() << "PluginSolver::slotDlgSolverShow(): start" << endl;
	if (this->m_dialog == NULL)
	{
		this->m_dialog = new KSpreadDlgSolverMain(kspreadView, "Solver Parameters");
		connect(m_dialog, SIGNAL(accepted()), this, SLOT(slotDlgSolverHidden()));
		connect(m_dialog, SIGNAL(rejected()), this, SLOT(slotDlgSolverHidden()));
	}
	if (!dialogIsVisible)
	{
		dialogIsVisible = true;
		this->m_dialog->show();
	}
}
	
void PluginSolver::slotDlgSolverHidden()
{
	dialogIsVisible = false;
	kdDebug() << "PluginSolver::slotDlgSolverHidden()" << endl;
}

#include "kspread_plugin_solver.moc"
